Package: aggreCAT
Type: Package
Title: Mathematically Aggregating Expert Judgments
Version: 1.1.0
Authors@R: c(person(given = "David",
                    family = "Wilkinson",
                    role = c("aut", "cre"),
                    email = "david.wilkinson.research@gmail.com",
                    comment = c(ORCID = "0000-0002-9560-6499")),
             person(given = "Elliot",
                    family = "Gould",
                    role = "aut",
                    comment = c(ORCID = "0000-0002-6585-538X")),
             person(given = "Aaron",
                    family = "Willcox",
                    role = "aut",
                    comment = c(ORCID = "0000-0003-2536-2596")),
             person(given = "Charles T.",
                    family = "Gray",
                    role = c("aut")),
             person(given = "Rose",
                    family = "E. O'Dea",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-8177-5075")),
             person(given = "Rebecca",
                    family = "Groenewegen",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-9177-8536")))
Description: The use of structured elicitation to inform decision making has grown dramatically in recent decades, however, judgements from multiple experts must be aggregated into a single estimate. Empirical evidence suggests that mathematical aggregation provides more reliable estimates than enforcing behavioural consensus on group estimates. 'aggreCAT' provides state-of-the-art mathematical aggregation methods for elicitation data including those defined in Hanea, A. et al. (2021) <doi:10.1371/journal.pone.0256919>. The package also provides functions to visualise and evaluate the performance of your aggregated estimates on validation data.
URL: https://replicats.research.unimelb.edu.au/,
        https://github.com/metamelb-repliCATS/aggreCAT,
        https://metamelb-replicats.github.io/aggreCAT/
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, covr, pointblank,
        janitor, qualtRics, here, readxl, readr, lubridate, forcats,
        ggforce, ggpubr, ggridges, rjags, tidybayes, tidyverse,
        usethis, nlme, gt, gtExtras, R.rsp
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
Imports: magrittr, GoFKernel, purrr, R2jags, coda, precrec, mathjaxr,
        cli, VGAM, crayon, dplyr, stats, stringr, tidyr, tibble,
        ggplot2, insight, DescTools, MLmetrics
VignetteBuilder: knitr, R.rsp
RdMacros: mathjaxr
BuildManual: TRUE
Config/testthat/parallel: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 12:42:37 UTC; wilko
Author: David Wilkinson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9560-6499>),
  Elliot Gould [aut] (ORCID: <https://orcid.org/0000-0002-6585-538X>),
  Aaron Willcox [aut] (ORCID: <https://orcid.org/0000-0003-2536-2596>),
  Charles T. Gray [aut],
  Rose E. O'Dea [aut] (ORCID: <https://orcid.org/0000-0001-8177-5075>),
  Rebecca Groenewegen [aut] (ORCID:
    <https://orcid.org/0000-0001-9177-8536>)
Maintainer: David Wilkinson <david.wilkinson.research@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 13:40:11 UTC
Built: R 4.5.2; ; 2026-05-04 17:03:33 UTC; unix

Package: aggreCAT
Type: Package
Title: Mathematically Aggregating Expert Judgments
Version: 1.1.0
Authors@R: c(person(given = "David",
                    family = "Wilkinson",
                    role = c("aut", "cre"),
                    email = "david.wilkinson.research@gmail.com",
                    comment = c(ORCID = "0000-0002-9560-6499")),
             person(given = "Elliot",
                    family = "Gould",
                    role = "aut",
                    comment = c(ORCID = "0000-0002-6585-538X")),
             person(given = "Aaron",
                    family = "Willcox",
                    role = "aut",
                    comment = c(ORCID = "0000-0003-2536-2596")),
             person(given = "Charles T.",
                    family = "Gray",
                    role = c("aut")),
             person(given = "Rose",
                    family = "E. O'Dea",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-8177-5075")),
             person(given = "Rebecca",
                    family = "Groenewegen",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-9177-8536")))
Description: The use of structured elicitation to inform decision making has grown dramatically in recent decades, however, judgements from multiple experts must be aggregated into a single estimate. Empirical evidence suggests that mathematical aggregation provides more reliable estimates than enforcing behavioural consensus on group estimates. 'aggreCAT' provides state-of-the-art mathematical aggregation methods for elicitation data including those defined in Hanea, A. et al. (2021) <doi:10.1371/journal.pone.0256919>. The package also provides functions to visualise and evaluate the performance of your aggregated estimates on validation data.
URL: https://replicats.research.unimelb.edu.au/,
        https://github.com/metamelb-repliCATS/aggreCAT,
        https://metamelb-replicats.github.io/aggreCAT/
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, covr, pointblank,
        janitor, qualtRics, here, readxl, readr, lubridate, forcats,
        ggforce, ggpubr, ggridges, rjags, tidybayes, tidyverse,
        usethis, nlme, gt, gtExtras, R.rsp
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
Imports: magrittr, GoFKernel, purrr, R2jags, coda, precrec, mathjaxr,
        cli, VGAM, crayon, dplyr, stats, stringr, tidyr, tibble,
        ggplot2, insight, DescTools, MLmetrics
VignetteBuilder: knitr, R.rsp
RdMacros: mathjaxr
BuildManual: TRUE
Config/testthat/parallel: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 12:42:37 UTC; wilko
Author: David Wilkinson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9560-6499>),
  Elliot Gould [aut] (ORCID: <https://orcid.org/0000-0002-6585-538X>),
  Aaron Willcox [aut] (ORCID: <https://orcid.org/0000-0003-2536-2596>),
  Charles T. Gray [aut],
  Rose E. O'Dea [aut] (ORCID: <https://orcid.org/0000-0001-8177-5075>),
  Rebecca Groenewegen [aut] (ORCID:
    <https://orcid.org/0000-0001-9177-8536>)
Maintainer: David Wilkinson <david.wilkinson.research@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 13:40:11 UTC
Built: R 4.6.0; ; 2026-05-04 17:22:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(AverageWAgg)
export(BayesianWAgg)
export(DistributionWAgg)
export(ExtremisationWAgg)
export(IntervalWAgg)
export(LinearWAgg)
export(ReasoningWAgg)
export(ShiftingWAgg)
export(confidence_score_evaluation)
export(confidence_score_heatmap)
export(confidence_score_ridgeplot)
export(method_placeholder)
export(postprocess_judgements)
export(preprocess_judgements)
export(weight_asym)
export(weight_interval)
export(weight_nIndivInterval)
export(weight_outlier)
export(weight_reason)
export(weight_reason2)
export(weight_varIndivInterval)
import(mathjaxr)
importFrom(DescTools,BrierScore)
importFrom(MLmetrics,Accuracy)
importFrom(insight,format_capitalize)
importFrom(magrittr,"%>%")
importFrom(precrec,auc)
importFrom(precrec,evalmod)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,var)

README
================

[![DOI](https://zenodo.org/badge/531484296.svg)](https://zenodo.org/badge/latestdoi/531484296)
[![R-CMD-check](https://github.com/metamelb-repliCATS/aggreCAT/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/metamelb-repliCATS/aggreCAT/actions/workflows/R-CMD-check.yaml)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `aggreCAT`: methods for mathematically aggregating expert judgements

## Problem Context

The use of structured elicitation to inform decision making has grown
dramatically across fields like ecology and environmental science over
the last decade, as researchers and managers are faced with the need to
act rapidly in the face of uncertainty and absent, uninformative data.
However, judgements from multiple experts must be aggregated into a
single estimate or distribution, and empirical evidence suggests that
mathematical aggregation provides more reliable estimates than
behavioural consensus models.

Unfortunately, there is a dearth of accessible tools for implementing
more complex aggregation methods other than linear averages, which are
arguably the most commonly used aggregation method, but may not be the
best approach for yielding robust estimates. The lack of readily
available aggregation methods severely limits users who may want to
utilise alternative aggregation methods more suitable for the task at
hand, but who do not have the time or technical capacity to implement.

The availability of methods implemented in R is even more limited, with
the most fully-fledged [package
`expert`](https://cran.r-project.org/package=expert) being archived from
CRAN in 2022, the [`SHELF`
package](https://cran.r-project.org/package=SHELF) implementing only a
single aggregation method (weighted linear pool / arithmetic mean), and
the [`opera` package](https://cran.r-project.org/package=opera)
aggregating non-point-estimate judgements only (time-series
predictions).

An archived version of `expert` is still available, however, the package
provides only three aggregation methods, and stipulates a structured
elicitation protocol that ‘calibrates’ experts through the use of seed
questions, which are required as additional input data to the
aggregation functions.

## The aggreCAT Package

The `aggreCAT` package aims to fill this void, implementing 22 unique
state-of-the-art and readily deployable methods for mathematically
aggregating expert judgements, described in Hanea et al. (2021).

Aggregation methods comprise unweighted linear combinations of
judgements, weighted linear combinations of judgements where weights are
proxies of forecasting performance constructed from characteristics of
participants and/or their judgements, and, and Bayesian methods that use
expert judgement to update uninformative and informative priors.

Aside from prescribing elicited judgements be derived from any
structured elicitation method that does not force behavioural consensus,
the aggreCAT package does not force users into adhering to one
particular approach to structured expert elicitation. However, some
methods are more prescriptive about input data types and the elicitation
method used to derive them than others. At minimum, a single
point-estimate is required for aggregation, and for some methods,
repliCATS IDEA protocol, is required to generate the necessary data as
inputs to the aggregation function. The IDEA (Investigate, Discuss,
Estimate, Aggregate) protocol generates robust estimates by leveraging
the wisdom-of-the-crowd, and is more onerous than collecting only single
point-estimates, but generates more robust and reliable estimates.

## Installation

You can install:

- the latest development version of `aggreCAT` package:

``` r
install.packages("devtools")
devtools::install_github("metamelb-repliCATS/aggreCAT")
```

- the most recent official version of `aggreCAT` from CRAN:

  - TBC! *We will upload to CRAN once our manuscript (Gould et al. in
    prep.) has been submitted.*

Then load the package:

``` r
library(aggreCAT)
```

# Getting Started with `aggreCAT`

Below we provide a brief summary of the package, for a detailed
overview, please consult the manuscript (Gould et al. in prep.).

## Core Functionality and Design

**Aggregation Functions**

Aggregation methods are grouped based on their mathematical properties
into eight ‘wrapper’ functions, denoted by the suffix `WAgg`, the
abbreviation of “weighted aggregation”: `LinearWAgg()`, `AverageWAgg()`,
`BayesianWAgg()`, `IntervalWAgg()`, `ShiftingWAgg()`, `ReasoningWAgg()`,
`DistributionWAgg()`, and `ExtremisationWAgg()`.

All wrapper functions adhere to the following basic argument structure:

``` r
args(AverageWAgg)
#> function (expert_judgements, type = "ArMean", name = NULL, placeholder = FALSE, 
#>     percent_toggle = FALSE, round_2_filter = TRUE) 
#> NULL
```

- expert judgements are contained a dataframe parsed to
  `expert_judgements`,
- the `type` argument specifies the specific flavour of the wrapper
  aggregation function to be executed on the `expert_judgements` data,
  with the available aggregation methods detailed in each wrapper
  function’s help page, e.g. `?AverageWAgg`,
- `name` allows a user-specified name with which to label the method in
  the results,
- toggling the `placeholder` argument to `TRUE` returns ‘placeholder’
  values, set to $0.65$,
- toggling `percent_toggle` to `TRUE` facilitates aggregating quantities
  rather than probabilities.

Each aggregation wrapper function returns a `dataframe` / `tibble` of
results, with one row or observation per unique judgement task.

**Elicitation Data**

`aggreCAT` includes datasets with judgements about the likely
replicability of research claims, collected by the [repliCATS
project](https://replicats.research.unimelb.edu.au) team as a pilot
study for the [DARPA SCORE
program](https://www.darpa.mil/research/programs/systematizing-confidence-in-open-research-and-evidence).
Data were elicited using a modified version of the IDEA protocol
(Hemming et al. 2017, Figure 1), whereby participants *Investigate*,
*Discuss, Estimate*, and finally *Aggregate* their judgements using
methods from the `aggreCAT` package (Fraser et al. 2021). Following the
IDEA protocol, best estimates, and upper and lower bounds are elicited
from each participant, over two rounds. The judgement data is contained
in the object `data_ratings`, described at `?data_ratings`.

<figure>
<img src="inst/ms/images/img_IDEA_repliCATS.png" id="fig-IDEA"
style="width:100.0%" data-fig-pos="hold"
alt="Figure 1: the repliCATS IDEA protocol was used to elicit judgements about the likely replicability of research claims, a pilot version of this dataset is included in the aggreCAT package" />
<figcaption aria-hidden="true">Figure 1: the repliCATS IDEA protocol was
used to elicit judgements about the likely replicability of research
claims, a pilot version of this dataset is included in the
<code>aggreCAT</code> package</figcaption>
</figure>

## A minimal working example with `AverageWAgg()`

<figure>
<img src="inst/ms/images/ArMean.png" id="fig-1" style="width:100.0%"
alt="Figure 2: Mathematically aggregating a small subset of expert judgements for the claim 28, using the unweighted arithmetic mean. The aggreCAT wrapper function AverageWAgg() is used on this dataset, with the type argument set to the default ArMean." />
<figcaption aria-hidden="true">Figure 2: Mathematically aggregating a
small subset of expert judgements for the claim <code>28</code>, using
the unweighted arithmetic mean. The <code>aggreCAT</code> wrapper
function <code>AverageWAgg()</code> is used on this dataset, with the
<code>type</code> argument set to the default
<code>ArMean</code>.</figcaption>
</figure>

Below we demonstrate how to use the most simple commonly implemented
aggregation method `ArMean`, which takes the arithmetic mean of
participant Best Estimates. We first use a small subset of 5
participants for a single claim, `28`, which is represented visually in
[Figure 1](#fig-1).

``` r
library(dplyr)
data(data_ratings)
set.seed(1234)

participant_subset <- data_ratings %>%
  distinct(user_name) %>%
  sample_n(5) %>%
  mutate(participant_name = paste("participant", rep(1:n())))

single_claim <- data_ratings %>% 
  filter(paper_id == "28") %>% 
  right_join(participant_subset, by = "user_name")

AverageWAgg(expert_judgements = single_claim, 
            type = "ArMean")
#> 
#> ── AverageWAgg: ArMean ─────────────────────────────────────────────────────────
#> 
#> ── Pre-Processing Options ──
#> 
#> ℹ Round Filter: TRUE
#> ℹ Three Point Filter: TRUE
#> ℹ Percent Toggle: FALSE
#> # A tibble: 1 × 4
#>   method paper_id    cs n_experts
#>   <chr>  <chr>    <dbl>     <int>
#> 1 ArMean 28        70.8         5
```

Often times during expert elicitation multiple quantities or measures
are put to experts to provide judgements for, and so we might want to
batch aggregation over more than a single judgement at a time (this time
called without explicitly specifying arguments):

``` r
data_ratings %>% AverageWAgg()
#> 
#> ── AverageWAgg: ArMean ─────────────────────────────────────────────────────────
#> 
#> ── Pre-Processing Options ──
#> 
#> ℹ Round Filter: TRUE
#> ℹ Three Point Filter: TRUE
#> ℹ Percent Toggle: FALSE
#> # A tibble: 25 × 4
#>    method paper_id    cs n_experts
#>    <chr>  <chr>    <dbl>     <int>
#>  1 ArMean 100       70.6        25
#>  2 ArMean 102       30.8        25
#>  3 ArMean 103       62.5        25
#>  4 ArMean 104       47.1        25
#>  5 ArMean 106       36.5        25
#>  6 ArMean 108       71.8        25
#>  7 ArMean 109       72.5        25
#>  8 ArMean 116       62.6        25
#>  9 ArMean 118       54.8        25
#> 10 ArMean 133       59.9        25
#> # … with 15 more rows
```

And other times, we might want to trial different aggregation methods
over those judgements, examining how their mathematical properties might
change the results, for example:

``` r
purrr::map_dfr(.x = list(AverageWAgg, IntervalWAgg, ShiftingWAgg),
                                .f = ~ .x(data_ratings))
```

# Attribution

This research was conducted as a part of the [repliCATS
project](https://replicats.research.unimelb.edu.au), funded by the
[DARPA SCORE
programme](https://www.darpa.mil/research/programs/systematizing-confidence-in-open-research-and-evidence)

The `aggreCAT` package is the culmination of the hard work and
persistence of a small team of researchers. Use of this package shall be
appropriately attributed and cited accordingly:

``` r
citation("aggreCAT")
#> 
#> To cite package 'aggreCAT' in publications use:
#> 
#>   Willcox A, Gray C, Gould E, Wilkinson D, Hanea A, Wintle B, E. O'Dea
#>   R (????). _aggreCAT: Mathematically Aggregating Expert Judgments_. R
#>   package version 0.0.0.9002,
#>   <https://replicats.research.unimelb.edu.au/>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Manual{,
#>     title = {aggreCAT: Mathematically Aggregating Expert Judgments},
#>     author = {Aaron Willcox and Charles T. Gray and Elliot Gould and David Wilkinson and Anca Hanea and Bonnie Wintle and Rose {E. O'Dea}},
#>     note = {R package version 0.0.0.9002},
#>     url = {https://replicats.research.unimelb.edu.au/},
#>   }
```

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-Fraser:2021" class="csl-entry">

Fraser, Hannah, Martin Bush, Bonnie Wintle, Fallon Mody, Eden T Smith,
Anca Hanea, Elliot Gould, et al. 2021. “Predicting Reliability Through
Structured Expert Elicitation with repliCATS (Collaborative Assessments
for Trustworthy Science).” MetaArXiv.
<https://doi.org/10.31222/osf.io/2pczv>.

</div>

<div id="ref-Gould2022" class="csl-entry">

Gould, Elliot, Charles T Gray, Aaron Willcox, Rose E O’Dea, Rebecca
Groenewegen, and David P Wilkinson. in prep. “aggreCAT: An r Package for
Mathematically Aggregating Expert Judgments.” MetaArXiv.
<https://doi.org/10.31222/osf.io/74tfv>.

</div>

<div id="ref-Hanea2021" class="csl-entry">

Hanea, Anca, David P Wilkinson, Marissa McBride, Aidan Lyon, Don van
Ravenzwaaij, Felix Singleton Thorn, Charles T Gray, et al. 2021.
“Mathematically Aggregating Experts’ Predictions of Possible Futures.”
*PLoS ONE* 16 (9).
https://doi.org/<https://doi.org/10.1371/journal.pone.0256919>.

</div>

<div id="ref-hemming2017" class="csl-entry">

Hemming, Victoria, Mark A. Burgman, Anca M. Hanea, Marissa F. McBride,
and Bonnie C. Wintle. 2017. “A Practical Guide to Structured Expert
Elicitation Using the IDEA Protocol.” Edited by Barbara Anderson.
*Methods in Ecology and Evolution* 9 (1): 169–80.
<https://doi.org/10.1111/2041-210x.12857>.

</div>

</div>

## ---- include=FALSE-----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(aggreCAT)
library(tidyverse)

## ----data_ratings-sample, message=TRUE, results='hold'------------------------
aggreCAT::data_ratings %>%
  dplyr::filter(paper_id == dplyr::first(paper_id),
                user_name == dplyr::first(user_name)) %>%
  head()


## ----data_supp_quiz-sample, message=TRUE, results='hold'----------------------
aggreCAT::data_supp_quiz

## ----data_supp_reasons-sample-------------------------------------------------
aggreCAT::data_supp_reasons %>%
  glimpse()

## ----data_supp_priors-sample--------------------------------------------------
aggreCAT::data_supp_priors

# install.packages("W3CMarkupValidator")
# install.packages("vnu.jar", type = "source",
#                  repos = "https://datacube.wu.ac.at/")

# MUST: devtools::document()
# MUST: devtools::build()

PKG_name_version <- glue::glue('{packageDescription("aggreCAT")$Package}_{packageDescription("aggreCAT")$Version}')

src <- glue::glue("../{PKG_name_version}.tar.gz")   # or just PKG if installed
out <- "aggreCAT.html"

tools::pkg2HTML(src, out)
library(W3CMarkupValidator)

results <- w3c_markup_validate(file = out, jar = TRUE)

if (length(results$type) > 0) results |> tibble::as_tibble()

stopifnot(
    all(fs::file_exists(c(src, out)))
)

file.remove(out)

#' @title
#' Aggregation Method: IntervalWAgg
#'
#' @description
#' Calculate one of several types of linear-weighted best estimates where the weights
#' are dependent on the lower and upper bounds of three-point elicitation (interval widths).
#'
#' @details
#' The width of the interval provided by individuals may be an indicator of certainty,
#' and arguably of accuracy of the best estimate contained between the bounds of the interval.
#'
#' `type` may be one of the following:
#'
#' **IntWAgg**: Weighted according to the interval width across individuals for that claim, rewarding narrow interval widths.
#' \loadmathjax
#' \mjdeqn{w\_Interval_{i,c}= \frac{1}{U_{i,c} - L_{i,c}}}{ascii}
#' \mjdeqn{\hat{p}_c( IntWAgg) = \sum_{i=1}^N \tilde{w}\_Interval_{i,c}B_{i,c}}{ascii}
#'
#' where \mjeqn{U_{i,d}  - L_{i,d}}{ascii} are individual \mjeqn{i}{ascii}'s judgements for claim \mjeqn{d}{ascii}. Then
#'
#' **IndIntWAgg**: Weighted by the rescaled interval width (interval width relative to largest
#' interval width provided by that individual)
#'
#' Because of the variability in interval widths between individuals across claims,
#' it may be beneficial to account for this individual variability by rescaling interval
#' widths across all claims per individual. This results in a re-scaled interval width weight,
#' for individual \mjeqn{i}{ascii} for claim \mjeqn{c}{ascii}, relative to the widest interval provided by
#' that individual across all claims \mjeqn{C}{ascii}:
#'
#' \mjdeqn{w\_nIndivInterval_{i,c}= \frac{1}{\frac{U_{i,c} - L_{i,c}}{max({ (U_{i,d}
#' - L_{i,d}): d = 1,\dots, C})}}}{ascii}
#'
#' \mjdeqn{\hat{p}_c\left( IndIntWAgg \right) = \sum_{i=1}^N \tilde{w}\_nIndivInterval_{i,c}B_{i,c}}{ascii}
#'
#' **AsymWAgg**: Weighted by the asymmetry of individuals' intervals, rewarding increasing asymmetry.
#'
#' We use the asymmetry of an interval relative to the corresponding best estimate
#' to define the following weights:
#'
#' \mjdeqn{w\_asym_{i,c}= \begin{cases}
#' 1 - 2 \cdot \frac{U_{i,c}-B_{i,c}}{U_{i,c}-L_{i,c}}, \text{for}\ B_{i,c} \ge
#' \frac{U_{i,c}-L_{i,c}}{2}+L_{i,c}\cr
#' 1 - 2 \cdot \frac{B_{i,c}-L_{i,c}}{U_{i,c}-L_{i,c}}, \text{otherwise}
#' \end{cases}}{ascii}
#'
#' then,
#'
#' \mjdeqn{\hat{p}_c(AsymWAgg) = \sum_{i=1}^N \tilde{w}\_asym_{i,c}B_{i,c}.}{ascii}
#'
#' **IndIntAsymWAgg**: Weighted by individuals’ interval widths and asymmetry
#'
#' This rewards both asymmetric and narrow intervals. We simply multiply the weights calculated
#' in the `"AsymWAgg"` and `"IndIntWAgg"` methods.
#'
#' \mjdeqn{w\_nIndivInterval\_asym_{i,c} = \tilde{w}\_nIndivInterval_{i,c} \cdot \tilde{w}\_asym_{i,c}}{ascii}
#'
#' \mjdeqn{\hat{p}_c( IndIntAsymWAgg) = \sum_{i=1}^N \tilde{w}\_nIndivInterval\_asym_{i,c}B_{i,c}}{ascii}
#'
#' **VarIndIntWAgg**: Weighted by the variation in individuals’ interval widths
#'
#' A higher variance in individuals' interval width across claims may indicate a higher responsiveness
#' to the supporting evidence of different claims. Such responsiveness might be predictive of more
#' accurate assessors. We define:
#'
#' \mjdeqn{w\_varIndivInterval_{i}= var{(U_{i,c} - L_{i,c}): c = 1,\dots, C},}{ascii}
#'
#' where the variance (\mjeqn{var}{ascii}) is calculated across all claims for individual \mjeqn{i}{ascii}.
#' Then,
#'
#' \mjdeqn{\hat{p}_c(VarIndIntWAgg) = \sum_{i=1}^N \tilde{w}\_varIndivInterval_{i}B_{i,c}}{ascii}
#'
#' **KitchSinkWAgg**: Weighted by everything but the kitchen sink
#'
#' This method is informed by the intuition that we want to reward narrow and asymmetric intervals,
#' as well as the variability of individuals' interval widths (across their estimates). Again, we multiply
#' the weights calculated in the `"AsymWAgg"`, `"IndIntWAgg"` and `"VarIndIntWAgg"` methods above.
#'
#' \mjdeqn{w\_kitchSink_{i,c} = \tilde{w}\_nIndivInterval_{i,c} \cdot \tilde{w}\_asym_{i,c} \cdot \tilde{w}\_varIndivInterval_{i}}{ascii}
#'
#' \mjdeqn{\hat{p}_c(KitchSinkWAgg) = \sum_{i=1}^N \tilde{w}\_kitchSink_{i,c}B_{i,c}}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"IntWAgg"`, `"IndIntWAgg"`, `"AsymWAgg"`, `"IndIntAsymWAgg"`, `"VarIndIntWAgg"`,
#' `"KitchSinkWAgg"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{IntervalWAgg(data_ratings)}
#'
#' @export
#' @md

IntervalWAgg <- function(
  expert_judgements,
  type = "IntWAgg",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
) {
  if (
    !(type %in%
      c(
        "IntWAgg",
        "IndIntWAgg",
        "AsymWAgg",
        "IndIntAsymWAgg",
        "VarIndIntWAgg",
        "KitchSinkWAgg"
      ))
  ) {
    stop(
      '`type` must be one of "IntWAgg", "IndIntWAgg", "AsymWAgg", "IndIntAsymWAgg", "VarIndIntWAgg" or "KitchSinkWAgg"'
    )
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("IntervalWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "IntWAgg" = {
        weights <- df %>%
          weight_interval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::select(-ub) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight)

        # standardise weight
        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "IndIntWAgg" = {
        weights <- df %>%
          weight_nIndivInterval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight)

        # standardise weight
        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "AsymWAgg" = {
        weights <- df %>%
          weight_asym() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight)

        # standardise weight
        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "VarIndIntWAgg" = {
        weights <- df %>%
          weight_varIndivInterval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::distinct()

        # standardise weight
        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "IndIntAsymWAgg" = {
        nindiv_weight <- df %>%
          weight_nIndivInterval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::rename(indiv = agg_weight)

        asym_weight <- df %>%
          weight_asym() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::rename(asym = agg_weight)

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(nindiv_weight, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(asym_weight, by = c("paper_id", "user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            indiv_weight = indiv / sum(indiv, na.rm = TRUE),
            asym_weight = asym / sum(asym, na.rm = TRUE),
            agg_weight = indiv_weight * asym_weight,
            agg_weight = agg_weight / sum(agg_weight, na.rm = TRUE)
          ) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "KitchSinkWAgg" = {
        nindiv_weight <- df %>%
          weight_nIndivInterval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::rename(indiv = agg_weight)

        asym_weight <- df %>%
          weight_asym() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::rename(asym = agg_weight)

        var_weight <- df %>%
          weight_varIndivInterval() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight) %>%
          dplyr::distinct() %>%
          dplyr::rename(var = agg_weight)

        df <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::left_join(nindiv_weight, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(asym_weight, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(var_weight, by = c("paper_id", "user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            indiv_weight = indiv / sum(indiv, na.rm = TRUE),
            asym_weight = asym / sum(asym, na.rm = TRUE),
            var_weight = var / sum(var, na.rm = TRUE),
            agg_weight = indiv_weight * asym_weight * var_weight,
            agg_weight = agg_weight / sum(agg_weight, na.rm = TRUE)
          ) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      }
    )

    df %>%
      dplyr::mutate(method = name) %>%
      postprocess_judgements()
  }
}

test_that("AverageWAgg Expected Errors", {
  expect_error(
    AverageWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "ArMean", "GeoMean", "Median", "LOArMean", "LOGeoMean", or "ProbitArMean"'
  )
})

test_that("AverageWAgg Expected Prob Judgements Error", {
  expect_error(
    AverageWAgg(
      expert_judgements = data_ratings,
      type = "LOArMean",
      percent_toggle = FALSE
    ),
    "LOArMean requires probabilistic judgements. Check your data compatability or `percent_toggle` argument."
  )
})


test_that("LinearWAgg Expected Errors", {
  expect_error(
    LinearWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "Judgement", "Judgement_LO", "Participant", "Participant_LO", "DistLimitWAgg", "GranWAgg", or "OutWAgg"'
  )
})

test_that("LinearWAgg Expected Prob Judgements Error", {
  expect_error(
    LinearWAgg(
      expert_judgements = data_ratings,
      type = "DistLimitWAgg",
      percent_toggle = FALSE
    ),
    "DistLimitWAgg requires judgements bounded 0-1. Check your data compatability or `percent_toggle` argument."
  )
})

test_that("LinearWAgg Expected Prob Judgements Error", {
  expect_error(
    LinearWAgg(
      expert_judgements = data_ratings,
      type = "GranWAgg",
      percent_toggle = FALSE
    ),
    "GranWAgg requires probabilistic judgements. Check your data compatability or `percent_toggle` argument."
  )
})

test_that("DistributionWAgg Expected Errors", {
  expect_error(
    DistributionWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "DistribArMean" or "TriDistribArMean"'
  )
})

test_that("ExtremisationWAgg Expected Errors", {
  expect_error(
    ExtremisationWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "BetaArMean" or "BetaArMean2'
  )
})

test_that("IntervalWAgg Expected Errors", {
  expect_error(
    IntervalWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "IntWAgg", "IndIntWAgg", "AsymWAgg", "IndIntAsymWAgg", "VarIndIntWAgg" or "KitchSinkWAgg"'
  )
})

test_that("ReasoningWAgg Expected Errors", {
  expect_error(
    ReasoningWAgg(
      expert_judgements = data_ratings,
      reasons = data_supp_ReasonWAgg,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "ReasonWAgg" or "ReasonWAgg2"'
  )
})

test_that("ShiftingWAgg Expected Errors", {
  expect_error(
    ShiftingWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "ShiftWAgg", "BestShiftWAgg", "IntShiftWAgg", "DistShiftWAgg", or "DistIntShiftWAgg"'
  )
})


test_that("BayesianWAgg Expected Errors", {
  expect_error(
    ShiftingWAgg(
      expert_judgements = data_ratings,
      type = "WrongType",
      percent_toggle = TRUE
    ),
    '`type` must be one of "ShiftWAgg", "BestShiftWAgg", "IntShiftWAgg", "DistShiftWAgg", or "DistIntShiftWAgg"'
  )
})


# Error for less than 2 claims provided

one_claim <- data_ratings %>%
  dplyr::filter(paper_id == "czttvy")

# test_that("BayTriVar stops on less than 2 ids provided", {
#
#   expect_error(BayesianWAgg(expert_judgements = one_claim,
#                             type = "BayTriVar",
#                             percent_toggle = TRUE),
#                'Model requires n > 1 ids to successfully execute.')
# })
#
# test_that("BayPRIORsAgg stops on less than 2 ids provided", {
#
#   expect_error(BayesianWAgg(expert_judgements = one_claim,
#                             priors = data_priors,
#                             type = "BayPRIORsAgg",
#                             placeholder = FALSE,
#                             percent_toggle = TRUE),
#                'Model requires n > 1 ids to successfully execute.')
# })

#' @title
#' Weighting method: Asymmetry of intervals
#'
#' @description
#' Calculates weights by asymmetry of intervals
#'
#' @details
#' This function is used inside [IntervalWAgg] to calculate the weights for the
#' aggregation type `"AsymWAgg"`, `"IndIntAsymWAgg"` and `"KitchSinkWAgg"`. Pre-processed
#' expert judgements (long format) are first converted to wide format then weighted by:
#' \loadmathjax
#' \mjdeqn{w\_asym_{i,c}= \begin{cases}
#' 1 - 2 \cdot \frac{U_{i,c}-B_{i,c}}{U_{i,c}-L_{i,c}}, \text{for}\ B_{i,c} \ge
#' \frac{U_{i,c}-L_{i,c}}{2}+L_{i,c}\cr
#' 1 - 2 \cdot \frac{B_{i,c}-L_{i,c}}{U_{i,c}-L_{i,c}}, \text{otherwise}
#' \end{cases}}{ascii}
#'
#' Data is converted back to long format, with only the weighted best estimates
#' retained.
#'
#' @param expert_judgements the long tibble exported from the `preprocess_judgements` function.
#'
#' @return A tibble in the form of the input `expert_judgements` argument with additional columns
#' supplying the calculated weight for each row's observation.
#'
#' @export
#'
#' @family weighting functions
#'
#' @examples
#' weight_asym(preprocess_judgements(data_ratings))

weight_asym <- function(expert_judgements) {
  expert_judgements %>%
    tidyr::pivot_wider(names_from = element, values_from = value) %>%
    dplyr::mutate(
      ul = three_point_upper - three_point_lower,
      ul = dplyr::if_else(ul == 0, .Machine$double.eps, ul),
      weight_obs = dplyr::if_else(
        three_point_best >= ((ul / 2) + three_point_lower),
        1 - 2 * (three_point_upper - three_point_best) / ul,
        1 - 2 * (three_point_best - three_point_lower) / ul
      ),

      agg_weight = dplyr::if_else(
        weight_obs < 0,
        0,
        weight_obs
      )
    ) %>%
    tidyr::pivot_longer(
      c(three_point_lower, three_point_best, three_point_upper),
      names_to = "element",
      values_to = "value"
    ) %>%
    dplyr::filter(element == "three_point_best")
}

#' Confidence Score Ridge Plot
#'
#' Display a ridge plot of confidence scores for each aggregation method
#'
#' @param confidence_scores A data frame of confidence scores in long format in the form of [data_confidence_scores]
#'
#' @importFrom stats quantile
#'
#' @return A density ridge plot of aggregation methods
#'
#' @examples
#' \donttest{confidence_score_ridgeplot(data_confidence_scores)}
#'
#' @export

confidence_score_ridgeplot <- function(confidence_scores = NULL) {
  if (is.null(confidence_scores)) {
    cli::cli_alert_info("No Confidence Scores Provided. Using Package Default")
    confidence_scores <- aggreCAT::data_confidence_scores
  } else {
    confidence_scores <- confidence_scores
  }

  number_of_claims <- dplyr::n_distinct(confidence_scores$paper_id)

  confidence_scores %>%
    ggplot2::ggplot(ggplot2::aes(
      x = cs,
      y = factor(method, levels = rev(levels(factor(method)))),
      fill = factor(ggplot2::after_stat(quantile))
      # fill = factor(ggplot2::after_stat(stats::quantile))
    )) +
    ggridges::stat_density_ridges(
      geom = "density_ridges_gradient",
      calc_ecdf = TRUE,
      quantiles = c(0.025, 0.975)
    ) +
    ggplot2::scale_fill_manual(
      name = "Probability",
      values = c("#D55E00", "#A0A0A0A0", "#0072B2"), # colour friendly
      labels = c("(0, 0.025]", "(0.025, 0.975]", "(0.975, 1]")
    ) +
    ggplot2::scale_x_continuous(breaks = c(0, .25, .5, .75, 1)) +
    ggplot2::scale_y_discrete(expand = c(0, 0)) +
    ggplot2::geom_vline(
      xintercept = .50,
      linetype = "dashed",
      color = "black",
      alpha = 1,
      show.legend = TRUE
    ) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      title = "Density of Aggregated Confidence Scores",
      subtitle = paste0("Claims Assessed N = ", number_of_claims),
      x = "Confidence Scores",
      y = "Aggregation Method"
    )
}

#' @title
#' Weighting method: Variation in individuals’ interval widths
#'
#' @description
#' Calculates weights based on the variability of interval widths
#' within individuals.
#'
#' @details
#' This function is used inside [IntervalWAgg] for aggregation types `"VarIndIntWAgg"`
#' and `"KitchSinkWAgg"`. It calculates the difference between individual's upper and
#' lower estimates, then calculates the variance in this interval across each individual's
#' claim assessments.
#' \loadmathjax
#' \mjdeqn{w\_varIndivInterval_{i} = var\{(U_{i,d}-L_{i,d}):c=1,...,C\}}{ascii}
#'
#' @param expert_judgements A dataframe in the form of [data_ratings]
#'
#' @importFrom stats var
#'
#' @return A tibble in the form of the input `expert_judgements` argument with additional columns
#' supplying the calculated weight for each row's observation.
#'
#' @family weighting functions
#'
#' @export

weight_varIndivInterval <- function(expert_judgements) {
  diff <- expert_judgements %>%
    tidyr::pivot_wider(names_from = element, values_from = value) %>%
    dplyr::mutate(
      diff = abs(three_point_upper - three_point_lower),
      diff = dplyr::if_else(diff == 0, .Machine$double.eps, diff)
    )

  diff %>%
    dplyr::group_by(user_name) %>%
    dplyr::summarise(agg_var = var(diff, na.rm = TRUE)) %>%
    dplyr::left_join(expert_judgements, by = "user_name") %>%
    dplyr::mutate(
      agg_weight = dplyr::if_else(
        !is.na(agg_var),
        agg_var,
        0
      )
    )
}

#' @title
#' Weighting method: Total number and diversity of judgement reasons
#'
#' @description
#' This function is used by [ReasoningWAgg] to weighting functions for the aggregation
#' type `"ReasonWAgg2"`. Weights are based on the number and diversity of reasoning
#' methods used by the participant to support their judgement.
#'
#' @details
#' \loadmathjax
#' An individual's weight is a product of the number of reasons given in support of
#' their judgement and the diversity of these reasons.
#' \mjdeqn{w\_{varReason}_{i,c} =\sum_{r=1}^{R} \mathbf{CR_i}(c,r) \cdot (1 - \frac{\sum_{c=1}^C
#' \mathbf{CR_i}(c,r)}{C})}{ascii}
#'
#' @param expert_reasons A dataframe in the form of [data_supp_reasons]
#'
#' @return A tibble of three columns `paper_id`, `user_name`, and `reason_count`
#'
#' @family weighting functions
#'
#' @export

weight_reason2 <- function(expert_reasons) {
  ## Calculate reason frequency/proportion

  reason_freq <- expert_reasons %>%
    dplyr::mutate(dplyr::across(
      .cols = dplyr::starts_with("RW"),
      .fns = ~ dplyr::if_else(.x > 1, 1, .x)
    )) %>%
    dplyr::select(-paper_id) %>%
    dplyr::group_by(user_name) %>%
    dplyr::summarise_all(mean, na.rm = TRUE) %>%
    dplyr::mutate(dplyr::across(
      .cols = dplyr::starts_with("RW"),
      .fns = function(x) {
        1 - x
      }
    )) %>%
    dplyr::rename_with(
      .fn = function(x) {
        paste0(x, "_FREQ")
      },
      .cols = dplyr::starts_with("RW")
    )

  ## Calculate reason counts

  expert_reasons %>%
    dplyr::left_join(reason_freq, by = "user_name") %>%
    tidyr::pivot_longer(
      data = .,
      cols = dplyr::starts_with("RW"),
      names_to = "var.names",
      values_to = "vals"
    ) %>%
    dplyr::mutate(var.names2 = stringr::str_remove(var.names, "_FREQ")) %>%
    dplyr::group_by(paper_id, user_name, var.names2) %>%
    dplyr::summarise(vals = dplyr::first(vals) * dplyr::last(vals)) %>%
    dplyr::group_by(paper_id, user_name) %>%
    dplyr::summarise(
      reason_count = sum(vals) %>%
        dplyr::na_if(., 0)
    )
}

.onAttach <- function(lib, pkg) {
  vers <- utils::packageDescription("aggreCAT", fields = "Version")

  bsu_rule_color <- "#7FD2FF"
  bsu_main_color <- "#18bc9c"

  bsu_main <- crayon::make_style(bsu_main_color)

  msg <- paste0(
    cli::rule(left = "aggreCAT", col = bsu_rule_color, line = 2),
    bsu_main('\nVersion: '),
    vers,
    " \n",
    bsu_main('Please do not feed the cat.'),
    " \n",
    cli::rule(left = "", col = bsu_rule_color, line = 2)
  )

  packageStartupMessage(msg)
}

#' @title
#' Aggregation Method: ExtremisationWAgg
#'
#' @description
#' Calculate beta-transformed arithmetic means of best estimates.
#'
#' @details
#' This method takes the average of best estimates and transforms it using the cumulative
#' distribution function of a beta distribution.
#'
#' `type` may be one of the following:
#'
#' **BetaArMean**: Beta transformation applied across the entire range of calculated confidence scores.
#' \loadmathjax
#'
#' \mjdeqn{\hat{p}_c\left( \text{BetaArMean} \right) = H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right),}{ascii}
#'
#' where \mjeqn{H_{\alpha \beta}}{ascii} is the cumulative distribution function of the beta distribution
#' with parameters \mjeqn{\alpha}{ascii} and \mjeqn{\beta}{ascii}, which default to 6 in the function.
#'
#' The justification for equal parameters (the 'shape1' and 'shape2' arguments in the `stats::pbeta` function)
#' are outlined in Satopää et al (2014) and the references therein (note that the method outlined in that paper
#' is called a beta-transformed linear opinion pool).
#' To decide on the default shape value of `6`, we explored the `data_ratings` dataset with random subsets of 5 assessments per claim,
#' which we expect to have for most of the claims assessed by repliCATS.
#'
#' **BetaArMean2**: Beta transformation applied only to calculated confidence scores that are outside a specified middle range. The premise being that we don't extremise "fence-sitter" confidence scores.
#'
#' \mjtdeqn{
#' \hat{p}_c\left( \text{BetaArMean2} \right) = \begin{cases}
#' \displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} < \textit{cutoff\_lower} \cr
#' \displaystyle \frac{1}{N} \sum_{i=1}^N B_{i,c}, \text{ for } \textit{cutoff\_lower} \leq \frac{1}{N} \sum_{i=1}^N B_{i,c} \le \textit{cutoff\_upper} \cr
#' \displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} > \textit{cutoff\_upper} \cr
#' \end{cases}}{%
#' \hat{p}_c\left( \text{BetaArMean2} \right) = \begin{cases}
#' \displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} \lt \textit{cutoff\_lower} \cr
#' \displaystyle \frac{1}{N} \sum_{i=1}^N B_{i,c}, \text{ for } \textit{cutoff\_lower} \leq \frac{1}{N} \sum_{i=1}^N B_{i,c} \le \textit{cutoff\_upper} \cr
#' \displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} \gt \textit{cutoff\_upper} \cr
#' \end{cases}}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"BetaArMean"` or `"BetaArMean2"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param alpha parameter for the 'shape1' argument in the `stats::pbeta` function (defaults to 6)
#' @param beta parameter for the 'shape2' argument in the `stats::pbeta` function (defaults to 6)
#' @param cutoff_lower Lower bound of middle region without extremisation in `"BetaArMean2"` aggregation `type`s.
#' @param cutoff_upper Upper bound of middle region without extremisation in `"BetaArMean2"` aggregation `type`s.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{ExtremisationWAgg(data_ratings, percent_toggle = TRUE)}
#'
#' @export
#' @md

ExtremisationWAgg <- function(
  expert_judgements,
  type = "BetaArMean",
  name = NULL,
  alpha = 6,
  beta = 6,
  cutoff_lower = NULL,
  cutoff_upper = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
) {
  if (!(type %in% c("BetaArMean", "BetaArMean2"))) {
    stop('`type` must be one of "BetaArMean" or "BetaArMean2')
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("ExtremisationWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::filter(element == "three_point_best") %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "BetaArMean" = {
        df <- df %>%
          dplyr::summarise(
            mean_judgement = mean(value, na.rm = TRUE),
            n_experts = dplyr::n()
          ) %>%
          dplyr::mutate(
            aggregated_judgement = stats::pbeta(
              q = mean_judgement,
              shape1 = alpha,
              shape2 = beta
            )
          )
      },
      "BetaArMean2" = {
        df <- df %>%
          dplyr::summarise(
            mean_judgement = mean(value, na.rm = TRUE),
            n_experts = dplyr::n()
          ) %>%
          dplyr::mutate(
            aggregated_judgement = dplyr::if_else(
              mean_judgement < cutoff_lower | mean_judgement > cutoff_upper,
              stats::pbeta(q = mean_judgement, shape1 = alpha, shape2 = beta),
              mean_judgement
            )
          )
      }
    )

    df %>%
      dplyr::mutate(method = name) %>%
      postprocess_judgements()
  }
}

#' @title
#' Confidence Score Evaluation
#'
#' @description
#' Evaluate the performance of the confidence scores generated by one or more aggregation
#' methods. Assumes probabilistic confidence scores for the metrics selected.
#'
#' @param confidence_scores A dataframe in the format output by the `aggreCAT::` aggregation methods
#' @param outcomes A dataframe with two columns: `paper_id` (corresponding to the id's from the
#' confidence_scores), and `outcome` containing the known outcome of replication studies
#'
#' @importFrom DescTools BrierScore
#' @importFrom MLmetrics Accuracy
#' @importFrom precrec auc
#' @importFrom precrec evalmod
#' @importFrom stats cor
#'
#' @return
#' Evaluated dataframe with five columns: `method` (character variable describing the aggregation method),
#' `AUC` (Area Under the Curve (AUC) scores of ROC curves - see `?precrec::auc`), `Brier_Score` (see
#' `?DescTools::BrierScore`), `Classification_Accuracy`(classification accuracy measured for pcc =
#' percent correctly classified; see `?MLmetrics::Accuracy`), and `Correlation` (point-biserial correlation;
#' see `?stats::cor`).
#'
#' @examples
#' \donttest{
#' confidence_score_evaluation(data_confidence_scores,
#'                             data_outcomes)
#' }
#'
#' @export

confidence_score_evaluation <- function(confidence_scores, outcomes) {
  ## Combine inputs

  cs_df <- confidence_scores %>%
    dplyr::left_join(outcomes, by = "paper_id")

  ## Add threshold prediction to be able to calculate Accuracy

  cs_df <- cs_df %>%
    dplyr::mutate(cl = as.numeric(cs > 0.5))

  ## Prediction evaluation

  eval_df <- cs_df %>%
    dplyr::filter(!is.na(outcome)) %>%
    dplyr::group_by(method) %>%
    dplyr::summarise(
      AUC = precrec::auc(precrec::evalmod(scores = cs, labels = outcome))[
        1,
        "aucs"
      ],
      Brier_Score = DescTools::BrierScore(x = outcome, pred = cs),
      Classification_Accuracy = MLmetrics::Accuracy(
        y_pred = cl,
        y_true = outcome
      ),
      Correlation = stats::cor(x = cs, y = outcome, method = "pearson")
    )

  return(eval_df)
}

# Generated by pointblank
library(pointblank)

tbl <- confidence_score_evaluation(confidence_scores = data_confidence_scores,
                                   outcomes = data_outcomes)

test_that("column `method` exists", {

  expect_col_exists(
    tbl,
    columns = vars(method),
    threshold = 1
  )
})

test_that("column `AUC` exists", {

  expect_col_exists(
    tbl,
    columns = vars(AUC),
    threshold = 1
  )
})

test_that("column `Brier_Score` exists", {

  expect_col_exists(
    tbl,
    columns = vars(Brier_Score),
    threshold = 1
  )
})

test_that("column `Classification_Accuracy` exists", {

  expect_col_exists(
    tbl,
    columns = vars(Classification_Accuracy),
    threshold = 1
  )
})

test_that("column `AUC` is of type: numeric", {

  expect_col_is_numeric(
    tbl,
    columns = vars(AUC),
    threshold = 1
  )
})

test_that("column `Brier_Score` is of type: numeric", {

  expect_col_is_numeric(
    tbl,
    columns = vars(Brier_Score),
    threshold = 1
  )
})

test_that("column `Classification_Accuracy` is of type: numeric", {

  expect_col_is_numeric(
    tbl,
    columns = vars(Classification_Accuracy),
    threshold = 1
  )
})

test_that("column `method` is of type: character", {

  expect_col_is_character(
    tbl,
    columns = vars(method),
    threshold = 1
  )
})

test_that("values in `AUC` should be between `0` and `1`", {

  expect_col_vals_between(
    tbl,
    columns = vars(AUC),
    left = 0,
    right = 1,
    threshold = 1
  )
})

test_that("all values in `method` should not be NULL", {

  expect_col_vals_not_null(
    tbl,
    columns = vars(method),
    threshold = 1
  )
})

test_that("all values in `AUC` should not be NULL", {

  expect_col_vals_not_null(
    tbl,
    columns = vars(AUC),
    threshold = 1
  )
})

test_that("all values in `Brier_Score` should not be NULL", {

  expect_col_vals_not_null(
    tbl,
    columns = vars(Brier_Score),
    threshold = 1
  )
})

test_that("all values in `Classification_Accuracy` should not be NULL", {

  expect_col_vals_not_null(
    tbl,
    columns = vars(Classification_Accuracy),
    threshold = 1
  )
})

test_that("values in `method` should be in the set of `ArMean`, `Median`, `LOArMean` (and 36 more)", {

  expect_col_vals_in_set(
    tbl,
    columns = vars(method),
    set = c("ArMean", "Median", "LOArMean", "BetaArMean",
            "IndIntAsymWAgg", "ShiftWAgg", "ReasonWAgg", "QuizWAgg",
            "BayTriVar", "BayPRIORsAgg", "IntWAgg", "IndIntWAgg",
            "VarIndIntWAgg", "AsymWAgg", "KitchSinkWAgg", "OutWAgg",
            "DistLimitWAgg", "GranWAgg", "DistribArMean", "CompWAgg",
            "ReasonWAgg2", "EngWAgg", "LOGeoMean", "ProbitArMean",
            "BestShiftWAgg", "TriDistribArMean", "GeoMean", "BetaArMean2",
            "BetaReasonWAgg", "BetaReasonWAgg2", "BadgeWAgg", "ExperienceWAgg",
            "ExperienceWAgg2", "ExperienceWAgg3", "QuizWAgg2", "QuizWAgg3",
            "DistShiftWAgg", "IntShiftWAgg", "DistIntShiftWAgg"),
    threshold = 1
  )
})

#' #' Confidence Score Ridge Plot
#' #'
#' #' Display a ridge plot of confidence scores for each aggregation method faceted by its linear, non-linear
#' #' and Bayesian characteristic.
#' #'
#' #' @param confidence_scores A data frame of confidence scores in long format in the form of [data_confidence_scores]
#' #'
#' #' @importFrom stats quantile
#' #' 
#' #' @return A density ridge plot of aggregation methods
#' #'
#' #' @examples
#' #' \donttest{confidence_score_plot(data_confidence_scores)}
#' #'
#' #'
#' #' @export
#' confidence_score_plot <- function(confidence_scores = NULL){
#' 
#'   if(is.null(confidence_scores)){
#'     cli::cli_alert_info("No Confidence Scores Provided. Using Package Default")
#'     confidence_scores <- aggreCAT::data_confidence_scores
#'   } else {
#'     confidence_scores <- confidence_scores
#'   }
#' 
#'   number_of_claims <- dplyr::n_distinct(confidence_scores$paper_id)
#' 
#'   confidence_scores <- method_types(confidence_scores)
#' 
#'   confidence_scores %>%
#'     ggplot2::ggplot(ggplot2::aes(x = cs,
#'                                  y = factor(method,
#'                                             levels = rev(levels(factor(method)))),
#'                                  fill = factor(ggplot2::after_stat(quantile))
#'     )) +
#'     ggridges::stat_density_ridges(
#'       geom = "density_ridges_gradient",
#'       calc_ecdf = TRUE,
#'       quantiles = c(0.025, 0.975)) +
#'     ggplot2::scale_fill_manual(
#'       name = "Probability", values = c("#D55E00", "#A0A0A0A0", "#0072B2"), # colour friendly
#'       labels = c("(0, 0.025]", "(0.025, 0.975]", "(0.975, 1]")) +
#'     ggplot2::scale_x_continuous(breaks = c(0, .25, .5, .75, 1)) +
#'     ggplot2::scale_y_discrete(expand = c(0,0)) +
#'     ggplot2::geom_vline(xintercept=.50,
#'                         linetype="dashed",
#'                         color = "white",
#'                         alpha = .75,
#'                         show.legend = TRUE) +
#'     ggplot2::theme(axis.ticks = ggplot2::element_blank(),
#'                    axis.ticks.x = ggplot2::element_blank(),
#'                    axis.ticks.y = ggplot2::element_blank(),
#'                    axis.text.y = ggplot2::element_text(face = "bold"),
#'                    axis.text.x = ggplot2::element_text(face = "bold"),
#'                    axis.line = ggplot2::element_blank(),
#'                    panel.grid = ggplot2::element_blank(),
#'                    panel.background = ggplot2::element_rect(fill = "transparent",
#'                                                             colour = NA),
#'                    plot.background = ggplot2::element_rect(fill = "white",
#'                                                            colour = NA),
#'                    strip.placement = "outside",
#'                    strip.background = ggplot2::element_blank(),
#'                    strip.text = ggplot2::element_text(face = "bold"),
#'                    plot.title = ggplot2::element_text(face = "bold"),
#'                    plot.title.position = "plot") +
#'     ggplot2::labs(title = "Density Plot of Aggregated Confidence Scores",
#'                   subtitle = paste0("Claims Assessed N = ", number_of_claims),
#'                   x = "Predicted Replication Scores",
#'                   y = "Aggregation Method",
#'                   caption = "Prediction cutoff threshold depicted by dashed line (0.5)") +
#'     ggforce::facet_row(~type, scales = "free", space = "fixed")
#' }
#' 
#' 
#' #' Confidence Score Heat Map
#' #'
#' #' Confidence scores displayed on a colour spectrum across generated methods and assessed claims,
#' #' split into predicted replication outcomes (TRUE/FALSE). White indicative of around `.5` with higher predicted
#' #' confidence scores more blue (`>.5`) and lower more red (`<.5`). Each predicted replication outcome is then
#' #' split into the group type of the underlying statistical characteristic for each aggregation method (non-weighted linear,
#' #' weighted linear & Bayesian).
#' #'
#' #' @param confidence_scores A data frame of confidence scores generated from the aggregation methods in the form of [data_confidence_scores].
#' #' Defaults to [data_confidence_scores] if no argument supplied.
#' #' @param data_outcomes A data frame of unique claims and the associated binary outcome in the form of [data_outcomes]. If no argument supplied
#' #' then defaults to [data_outcomes] supplied within package.
#' #' @param x_label Bottom x axis label name or ID. Default is blank.
#' #'
#' #' @importFrom insight format_capitalize
#' #'
#' #' @return Plot in viewer
#' #'
#' #' @examples
#' #' \dontrun{confidence_score_heatmap(data_confidence_scores, data_outcomes)}
#' #'
#' #' @export
#' 
#' confidence_score_heatmap <- function(confidence_scores = NULL,
#'                                      data_outcomes = NULL,
#'                                      x_label = NULL){
#' 
#' 
#'   # Defaults to example dataset if none provided
#'   if(is.null(confidence_scores)){
#'     cli::cli_alert_info("No Confidence Scores Provided. Using Package Default")
#'     confidence_scores <- aggreCAT::data_confidence_scores
#'   } else {
#'     confidence_scores <- confidence_scores
#'   }
#' 
#'   if(is.null(data_outcomes)){
#'     cli::cli_alert_info("No Outcome Scores Provided. Using Package Default")
#'     data_outcomes <- aggreCAT::data_outcomes
#'   } else {
#'     data_outcomes <- data_outcomes
#'   }
#' 
#'   # formatting
#' 
#'   if(!is.null(x_label)) {
#'     x_label <- insight::format_capitalize(x_label)
#'   }
#' 
#'   data_confidencescores_lf <- confidence_scores
#' 
#'   # object for checking the number of claims
#'   number_of_claims <- dplyr::n_distinct(data_confidencescores_lf$paper_id)
#' 
#'   # object to throw a display warning
#'   number_of_methods <- dplyr::n_distinct(data_confidencescores_lf$method)
#' 
#'   if(number_of_claims <= 10) cli::cli_alert_warning("Claims less than ~ 10 might not display effectively.")
#' 
#'   data_cs_with_outcomes <- data_confidencescores_lf %>%
#'     dplyr::left_join(data_outcomes, by = "paper_id") %>%
#'     dplyr::mutate(replicated_outcome = ifelse(outcome == 1, "TRUE", "FALSE")) %>%
#'     dplyr::select(paper_id, method, cs, outcome, replicated_outcome)
#' 
#'   # Generate accuracy scores
#'   evaluated_outcomes <- aggreCAT::confidence_score_evaluation(confidence_scores = data_confidencescores_lf,
#'                                                               outcomes = data_outcomes)
#' 
#'   # Labels names
#'   labels_repl <- c("Replicated", "Did Not Replicate")
#'   names(labels_repl) <- c("TRUE", "FALSE")
#' 
#'   # Aggregation type grouping
#'   evaluated_outcomes <- method_types(evaluated_outcomes)
#' 
#'   # predicted replication TRUE
#'   plot1 <- evaluated_outcomes %>%
#'     dplyr::left_join(data_cs_with_outcomes) %>%
#'     dplyr::filter(replicated_outcome == "TRUE") %>%
#'     dplyr::mutate(AUC = round(AUC, digits = 3)) %>%
#'     dplyr::mutate(Brier_Score = round(Brier_Score, digits = 2)) %>%
#'     dplyr::mutate(paper_id = forcats::fct_reorder(paper_id, -cs)) %>%
#'     ggplot2::ggplot() +
#'     ggplot2::aes(x = paper_id,
#'                  y = factor(method, levels = rev(levels(factor(method)))),
#'                  fill = cs,
#'                  colour = cs) +
#'     ggplot2::geom_tile(linetype = 1, linejoin = "mitre") +
#'     ggplot2::scale_fill_distiller(palette = "RdYlBu", direction = 1) +
#'     ggplot2::scale_color_distiller(palette = "RdYlBu", direction = 1) +
#'     ggplot2::labs(x = "",
#'                   y = "Aggregation Method",
#'                   title = "",
#'                   subtitle = "Predicted Outcome: TRUE",
#'                   caption = "",
#'                   fill = "Confidence \nScore", color = "Confidence \nScore") +
#'     ggplot2::theme(
#'       axis.text.y = ggplot2::element_text(face = "bold"),
#'       axis.text.x = ggplot2::element_text(angle = 75, hjust = 1),
#'       plot.title = ggplot2::element_text(face = "bold"),
#'       plot.subtitle = ggplot2::element_text(),
#'       axis.ticks.y = ggplot2::element_blank(),
#'       legend.position = "none",
#'       legend.text = ggplot2::element_text(),
#'       legend.title =  ggplot2::element_text(),
#'       panel.border = ggplot2::element_blank(),
#'       strip.background = ggplot2::element_blank(),
#'       plot.background = ggplot2::element_blank(),
#'       panel.background = ggplot2::element_blank(),
#'       strip.text = ggplot2::element_text(face = "bold")
#'     ) +
#'     ggforce::facet_col(~type, scales = "free_y", space = "free")
#' 
#'   # predicted replication FALSE
#'   plot2 <- evaluated_outcomes %>%
#'     dplyr::left_join(data_cs_with_outcomes) %>%
#'     dplyr::filter(replicated_outcome == "FALSE") %>%
#'     dplyr::mutate(AUC = round(AUC, digits = 3)) %>%
#'     dplyr::mutate(Brier_Score = round(Brier_Score, digits = 2)) %>%
#'     dplyr::mutate(paper_id = forcats::fct_reorder(paper_id, -cs)) %>%
#'     ggplot2::ggplot() +
#'     ggplot2::aes(x = paper_id, y = factor(method, levels = rev(levels(factor(method)))),
#'                  fill = cs,
#'                  colour = cs) +
#'     ggplot2::geom_tile(linetype = 1, linejoin = "mitre") +
#'     ggplot2::scale_fill_distiller(palette = "RdYlBu", direction = 1) +
#'     ggplot2::scale_color_distiller(palette = "RdYlBu", direction = 1) +
#'     ggplot2::labs(x = "",
#'                   y = "Aggregation Method",
#'                   title = "",
#'                   subtitle = "Predicted Outcome: FALSE",
#'                   caption = "",
#'                   fill = "Confidence \nScore", color = "Confidence \nScore") +
#'     ggplot2::theme(
#'       axis.text.x = ggplot2::element_text(angle = 75, hjust = 1),
#'       axis.text.y = ggplot2::element_blank(),
#'       axis.ticks.y = ggplot2::element_blank(),
#'       axis.title.y = ggplot2::element_blank(),
#'       plot.title = ggplot2::element_text(face = "bold"),
#'       plot.subtitle = ggplot2::element_text(),
#'       legend.position = "none",
#'       legend.text = ggplot2::element_text(),
#'       legend.title =  ggplot2::element_text(),
#'       panel.border = ggplot2::element_blank(),
#'       strip.background = ggplot2::element_blank(),
#'       plot.background = ggplot2::element_blank(),
#'       panel.background = ggplot2::element_blank(),
#'       strip.text = ggplot2::element_text(face = "bold")
#'     ) +
#'     ggforce::facet_col(~type, scales = "free_y", space = "free")
#' 
#'   figure <- ggpubr::ggarrange(plot1, plot2,
#'                               #labels = c("A", "B"),
#'                               #font.label = font,
#'                               common.legend = TRUE,
#'                               legend = "right",
#'                               heights = c(2,1),
#'                               widths = c(1.23, 1))
#' 
#'   if(number_of_methods <= 5) cli::cli_alert_warning("Provide more than 5 methods for improved contrasting plot")
#' 
#'   ggpubr::annotate_figure(figure,
#'                           bottom = ggpubr::text_grob(x_label,
#' 
#'                                                      just = "centre"))
#' }
#' 
#' 
#' method_types <- function(conf_scores){
#' 
#'   # Aggregation type grouping
#'   NWL = c("ArMean", "GeoMean", "Median", "LOArMean", "LOGeoMean", "ProbitArMean")
#'   WLCI = c("ShiftWAgg", "BestShiftWAgg", "IntShiftWAgg", "DistShiftWAgg", "DistIntShiftWAgg",
#'            "IntWAgg", "IndIntWAgg", "AsymWAgg", "IndIntAsymWAgg", "VarIndIntWAgg", "KitchSinkWAgg",
#'            "DistLimitWAgg", "GranWAgg", "OutWAgg", "BetaArMean", "BetaArMean2", "DistribArMean", "TriDistribArMean",
#'            "CompWAgg", "EngWAgg","ReasonWAgg", "ReasonWAgg2", "QuizWAgg", "QuizWAgg2", "QuizWAgg3",
#'            "BetaReasonWAgg", "BetaReasonWAgg2", "BadgeWAgg",
#'            "ExperienceWAgg", "ExperienceWAgg2", "ExperienceWAgg3")
#'   BAYES = c("BayTriVar", "BayPRIORsAgg")
#' 
#'   conf_scores <- conf_scores %>%
#'     dplyr::mutate(type = dplyr::case_when(method %in% NWL ~ "Non-weighted Linear Combintation",
#'                                           method %in% WLCI ~ "Weighted Linear Combinations",
#'                                           #method %in% WLCE ~ "Weighted Linear Combinations (Supplementary Data)",
#'                                           method %in% BAYES ~ "Bayesian Methods"))
#' 
#' 
#'   # Levels for the plot output
#'   conf_scores$type <- factor(conf_scores$type,
#'                              levels = c("Non-weighted Linear Combintation",
#'                                         "Weighted Linear Combinations",
#'                                         #"Weighted Linear Combinations (Supplementary Data)",
#'                                         "Bayesian Methods"))
#' 
#'   return(conf_scores)
#' 
#' }

#' @title
#' Aggregation Method: ReasoningWAgg
#'
#' @description
#' Calculate one of several types of linear-weighted best estimates using supplementary
#' participant reasoning data to create weights.
#'
#' @details
#' Weighted by the breadth of reasoning provided to support the individuals’ estimate.
#'
#' `type` may be one of the following:
#'
#' \loadmathjax
#' **ReasonWAgg**: Weighted by the number of supporting reasons
#'
#' Giving greater weight to best estimates that are accompanied by a greater number of
#' supporting reasons may be beneficial. We will consider \mjeqn{w\_{reason}_{i,c}}{ascii}
#' to be the number of unique reasons provided by that individual \mjeqn{i}{ascii} in
#' support of their estimate for claim \mjeqn{c}{ascii}.
#'
#' \mjdeqn{\hat{p}_c(ReasonWAgg) = \sum_{i=1}^N \tilde{w}\_reason_{i,c}B_{i,c}}{ascii}
#'
#' See Hanea et al. (2021) for an example of reason coding.
#'
#' **ReasonWAgg2**: Incorporates both the number of reasons and their diversity across claims.
#'
#' The claim diversity component of this score is calculated per individual from all claims
#' they assessed. We assume each individual answers at least two claims. If an individual has
#' assessed only one claim, there weighting for that claim is equivalent to "ReasonWAgg".
#'
#' We will consider \mjeqn{w\_{varReason}_{i,c}}{ascii} to be the weighted "number of unique
#' reasons" provided by participant \mjeqn{i}{ascii} in support of their estimate for claim
#' \mjeqn{c}{ascii}. Assume there are \mjeqn{R}{ascii} total unique reasons any participant
#' can use to justify their numerical answers. Then, for each participant \mjeqn{i}{ascii} we
#' can construct a matrix \mjeqn{\mathbf{CR_i}}{ascii} with \mjeqn{R}{ascii} columns, each
#' corresponding to a unique reason, \mjeqn{r}{ascii}, and \mjeqn{C}{ascii} rows, where
#' \mjeqn{C}{ascii} is the number of claims assessed by that participant. Each element of
#' this matrix \mjeqn{\mathbf{CR_i}(r,c)}{ascii} can be either 1 or 0.
#' \mjeqn{\mathbf{CR_i}(r,c) = 1}{ascii} if reason \mjeqn{R_r}{ascii} was used to justify
#' the estimates assessed for \mjeqn{c}{ascii}, and \mjeqn{\mathbf{CR_i}(r,c) = 0}{ascii} if
#' reason \mjeqn{R_r}{ascii} was not mentioned when assessing claim \mjeqn{c}{ascii}. The
#' more frequently that a participant uses a given reason reduces the amount it contributes
#' to the weight assigned to that participant.
#'
#' \mjdeqn{w\_{varReason}_{i,c} =\sum_{r=1}^{R} \mathbf{CR_i}(c,r) \cdot (1 - \frac{\sum_{c=1}^C
#' \mathbf{CR_i}(c,r)}{C})}{ascii}
#'
#' \mjdeqn{\hat{p}_c(ReasonWAgg2) = \sum_{i=1}^N \tilde{w}\_varReason_{i,c}B_{i,c}}{ascii}
#'
#' @note
#'
#' When `flag_loarmean` is set to `TRUE`, two additional columns will be returned; `method_applied` (a character variable describing the method actually applied with values of either `LoArMean` or `ReasonWAgg`) and `no_reason_score` (a logical variable describing whether no reasoning scores were supplied for any user for the given claim, where `TRUE` indicates no reasoning scores supplied and `FALSE` indicates that at least one participant for that claim had a reasoning score greater than 0).
#'
#' named method_applied (with values LoArMean or ReasonWAgg), and no_reason_score, a logical variable describing whether or not there were no reasoning scores for that claim.
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param reasons A dataframe in the form of [data_supp_reasons]
#' @param type One of `"ReasonWAgg"`, `"ReasonWAgg2"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param beta_transform Toggle switch to extremise confidence scores with the beta distribution. Defaults to `FALSE`.
#' @param beta_param Length two vector of alpha and beta parameters of the beta distribution. Defaults to `c(6,6)`.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param flag_loarmean A toggle to impute LOArMean instead of ArMean when no participants have a reasoning weight for a specific claim (defaults `FALSE`).
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{ReasoningWAgg(data_ratings, data_supp_reasons)}
#'
#' @export
#' @md

ReasoningWAgg <- function(
  expert_judgements,
  reasons = NULL,
  type = "ReasonWAgg",
  name = NULL,
  beta_transform = FALSE,
  beta_param = c(6, 6),
  placeholder = FALSE,
  percent_toggle = FALSE,
  flag_loarmean = FALSE,
  round_2_filter = TRUE
) {
  if (!(type %in% c("ReasonWAgg", "ReasonWAgg2"))) {
    stop('`type` must be one of "ReasonWAgg" or "ReasonWAgg2"')
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("ReasoningWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::filter(element == "three_point_best") %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "ReasonWAgg" = {
        reason_weights <- weight_reason(reasons)
      },
      "ReasonWAgg2" = {
        reason_weights <- weight_reason2(reasons)
      }
    )

    # work out which claims have no participants with reasoning scores

    with_rs <-
      reason_weights %>%
      dplyr::right_join(df, by = c("user_name", "paper_id"))

    # output summary data
    # identify claims that don't have quiz scores associated
    claim_summary_data <- with_rs %>%
      dplyr::group_by(paper_id) %>%
      dplyr::summarise(
        n_experts = dplyr::n(),
        no_reason_scores_for_claim = all(is.na(reason_count))
      )

    # loarmean for when there aren't quiz scores
    loarmean_data <- df %>%
      dplyr::full_join(claim_summary_data, by = "paper_id") %>%
      # filter to no quiz scores
      dplyr::filter(no_reason_scores_for_claim)

    loarmean_results <- if (nrow(loarmean_data) > 0) {
      loarmean_data %>%
        # Taking the core function of LOArMean
        dplyr::group_by(paper_id) %>%
        dplyr::mutate(
          value = dplyr::if_else(
            value == 1 | value == 0,
            value + .Machine$double.eps,
            value
          )
        ) %>%
        dplyr::mutate(log_odds = log(abs(value / (1 - value)))) %>%
        dplyr::summarise(
          aggregated_judgement = mean(log_odds, na.rm = TRUE),
          n_experts = dplyr::n()
          # first = min(timestamp,
          #             na.rm = TRUE),
          # last = max(timestamp,
          #            na.rm = TRUE)
        ) %>%
        dplyr::mutate(
          aggregated_judgement = exp(aggregated_judgement) /
            (1 + exp(aggregated_judgement))
        ) %>%
        dplyr::mutate(method = name)
    } else {
      NULL
    }

    # Process the no quiz scores dataframe to formatted output
    if (!is.null(loarmean_results)) {
      loarmean_results <- postprocess_judgements(loarmean_results)
    }

    # calculate reason weight
    reasonwagg_data <-
      claim_summary_data %>%
      dplyr::select(paper_id, no_reason_scores_for_claim) %>%
      dplyr::full_join(with_rs) %>%
      # filter to paper_ids with quiz scores associated
      dplyr::filter(!no_reason_scores_for_claim)

    ReasonWAgg_results <- reasonwagg_data %>%
      # calculate scaled weight by claim
      dplyr::group_by(paper_id) %>%
      dplyr::summarise(agg_sum = sum(reason_count, na.rm = TRUE)) %>%
      dplyr::full_join(reasonwagg_data, by = "paper_id") %>%
      dplyr::mutate(agg_weight = reason_count / agg_sum) %>%
      dplyr::group_by(paper_id) %>%
      dplyr::summarise(
        aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
        n_experts = dplyr::n()
      ) %>%
      dplyr::left_join(claim_summary_data) %>%
      # output
      dplyr::mutate(method = name) %>%
      postprocess_judgements()

    if (isTRUE(beta_transform)) {
      ReasonWAgg_results <- ReasonWAgg_results %>%
        dplyr::mutate(
          cs = stats::pbeta(
            q = cs,
            shape1 = beta_param[1],
            shape2 = beta_param[2]
          )
        )
    }

    output_data <-
      dplyr::bind_rows(loarmean_results, ReasonWAgg_results)

    if (flag_loarmean) {
      claim_summary_data %>%
        dplyr::select(paper_id, no_reason_scores_for_claim) %>%
        dplyr::full_join(output_data) %>%
        dplyr::mutate(
          method_applied = dplyr::if_else(
            no_reason_scores_for_claim,
            "LOArMean",
            name
          )
        )
    } else {
      output_data
    }
  }
}

# Generated by pointblank

library(pointblank)


method_test_loarmean <- function(df){

  tbl <- df

  test_that("column `paper_id` is of type: character", {

    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {

    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })


  # test_that("column `method_applied` is of type: character", {
  #
  #   expect_col_is_character(
  #     tbl,
  #     columns = vars(method_applied),
  #     threshold = 1
  #   )
  # })
#
#   test_that("column `no_quiz_scores_for_claim` is of type: logical", {
#
#     expect_col_is_logical(
#       tbl,
#       columns = ends_with("_for_claim"),
#       threshold = 1
#     )
#   })

  test_that("values in `cs` should be between `0` and `1`", {

    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })


  test_that("column `cs` is of type: numeric", {

    expect_col_is_numeric(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {

    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {

    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })


}

# Test LOArMean Flag for Quiz, Engwagg and Reasoning methods


# Quiz --------------------------------------------------------------------


QuizWAgg <- LinearWAgg(data_ratings,
                       weights = data_supp_quiz %>%
                         dplyr::rename(weight = quiz_score),
                       type = "Participant_LO",
                       name = "QuizWAgg",
                       placeholder = FALSE,
                       flag_loarmean = TRUE,
                       percent_toggle = TRUE)


method_test_loarmean(QuizWAgg)

# Reasons -----------------------------------------------------------------


ReasonWAgg <- ReasoningWAgg(data_ratings,
                            reasons = data_supp_reasons,
                            type = "ReasonWAgg",
                            percent_toggle = TRUE,
                            flag_loarmean = TRUE,
                            placeholder = FALSE)

ReasonWAgg2 <- ReasoningWAgg(data_ratings,
                             reasons = data_supp_reasons,
                             type = "ReasonWAgg2",
                             placeholder = FALSE,
                             flag_loarmean = TRUE,
                             percent_toggle = TRUE)

BetaReasonWAgg <- ReasoningWAgg(data_ratings,
                                reasons = data_supp_reasons,
                                type = "ReasonWAgg",
                                name = "BetaReasonWAgg",
                                beta_transform = TRUE,
                                beta_param = c(6,6),
                                placeholder = FALSE,
                                flag_loarmean = TRUE,
                                percent_toggle = TRUE)

BetaReasonWAgg2 <- ReasoningWAgg(data_ratings,
                                 reasons = data_supp_reasons,
                                 type = "ReasonWAgg2",
                                 name = "BetaReasonWAgg2",
                                 beta_transform = TRUE,
                                 beta_param = c(6,6),
                                 placeholder = FALSE,
                                 flag_loarmean = TRUE,
                                 percent_toggle = TRUE)




method_test_loarmean(ReasonWAgg)
method_test_loarmean(ReasonWAgg2)
method_test_loarmean(BetaReasonWAgg)
method_test_loarmean(BetaReasonWAgg2)

#' @title
#' Pre-process the data
#'
#' @description
#' Process input data with filters and meaningful variable names.
#'
#' This function is called at the head of every aggregation method function.
#'
#' @details
#' This pre-processing function takes input data in the format of
#' [data_ratings] and outputs a dataframe that:
#'
#' 1. Applies any filters or manipulations required by the aggregation method.
#' 2. Converts the input data into variables with more
#' meaningful names for coding, to avoid errors in the wrangling process.
#'
#' @param expert_judgements A dataframe with the same variables (columns) as
#' [data_ratings].
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#' @param three_point_filter Defaults `TRUE` to filter three point estimates. `FALSE` will
#' filter the involved_binary question.
#' @param percent_toggle Change the values to probabilities from percentages. Default is `FALSE`
#'
#' @return a long tibble of expert judgements, with six columns:
#'  `round`, `paper_id`, `user_name`, `element` (i.e. question type),
#'  and `value` (i.e. participant response).
#'
#' @examples
#' \donttest{preprocess_judgements(data_ratings)}
#'
#' @export

preprocess_judgements <- function(
  expert_judgements,
  round_2_filter = TRUE,
  three_point_filter = TRUE,
  percent_toggle = FALSE
) {
  # Processing Alerts -------------------------------------------------------

  cli::cli_h2("Pre-Processing Options")
  cli::cli_alert_info("Round Filter: {.val {round_2_filter}}")
  cli::cli_alert_info("Three Point Filter: {.val {three_point_filter}}")
  cli::cli_alert_info("Percent Toggle: {.val {percent_toggle}}")

  if (any(is.na(expert_judgements$value))) {
    cli::cli_abort("NAs Found in Values")
  }

  # Variables of focus
  expert_judgements <- expert_judgements %>%
    dplyr::select(round, paper_id, user_name, element, value)

  # Round Filter ------------------------------------------------------------

  filter_round <- function(expert_judgements, round_2_filter) {
    output_df <- if (isTRUE(round_2_filter)) {
      expert_judgements %>%
        dplyr::filter(round %in% "round_2")
    } else {
      expert_judgements
    }
  }

  # Three Point Filter ------------------------------------------------------

  filter_element <- function(expert_judgements, three_point_filter) {
    # Default
    # Filters to three point estimates
    # Otherwise removes the involved binary
    output_df <- if (isTRUE(three_point_filter)) {
      expert_judgements %>%
        dplyr::group_by(round, paper_id, user_name) %>%
        dplyr::filter(
          element %in%
            c("three_point_best", "three_point_lower", "three_point_upper")
        )
    } else {
      expert_judgements %>%
        dplyr::filter(element != "binary_question")
    }
  }

  # Convert Values ----------------------------------------------------------

  change_value <- function(expert_judgements, percent_toggle) {
    # Converts values to 0,1
    output_df <- if (isTRUE(percent_toggle)) {
      expert_judgements %>%
        dplyr::mutate(
          value = dplyr::case_when(
            element %in%
              c(
                "three_point_best",
                "three_point_lower",
                "three_point_upper"
              ) ~ value / 100,
            TRUE ~ value
          )
        )
    } else {
      expert_judgements
    }
  }

  # Validator ---------------------------------------------------------------

  check_values <- function(
    expert_judgements,
    round_2_filter,
    three_point_filter,
    percent_toggle
  ) {
    if (isTRUE(percent_toggle)) {
      if (isTRUE(max(expert_judgements$value) > 1)) {
        cli::cli_alert_warning(
          "Non probability value {.val {max(expert_judgements$value)}} outside 0,1 "
        )
      }
    }

    return(expert_judgements)
  }

  # Processing Data Frame ---------------------------------------------------

  method_out <- expert_judgements %>%
    filter_round(round_2_filter) %>%
    filter_element(three_point_filter) %>%
    change_value(percent_toggle) %>%
    dplyr::bind_rows() %>%
    check_values(round_2_filter, three_point_filter, percent_toggle) %>%
    dplyr::ungroup()

  return(method_out)
}

weight_nans <- function(method_to_test) {
  agg_weight <-
    data_ratings %>%
    preprocess_judgements() %>%
    method_to_test() %>%
    purrr::pluck("agg_weight")

  expect_false(any(is.na(agg_weight)))
  expect_false(any(is.nan(agg_weight)))
  expect_false(any(is.infinite(agg_weight)))
  expect_type(agg_weight, "double")
}


# test weights ------------------------------------------------------------

test_that("weight_asym method", {
  weight_nans(weight_asym)
})

test_that("weight_interval method", {
  weight_nans(weight_interval)
})

test_that("weight_nIndivInterval method", {
  weight_nans(weight_nIndivInterval)
})

test_that("weight_outlier method", {
  weight_nans(weight_outlier)
})

test_that("weight_varIndivInterval method", {
  weight_nans(weight_varIndivInterval)
})

#' @title
#' Aggregation Method: AverageWAgg
#'
#' @description
#' Calculate one of several types of averaged best estimates.
#'
#' @details
#' This function returns the average, median and transformed averages of
#' best-estimate judgements for each claim.
#'
#' `type` may be one of the following:
#' \loadmathjax
#'
#' **ArMean**: Arithmetic mean of the best estimates
#' \mjdeqn{\hat{p}_c\left(ArMean \right ) = \frac{1}{N}\sum_{i=1}^N B_{i,c}}{ascii}
#' **Median**: Median of the best estimates
#' \mjdeqn{\hat{p}_c \left(\text{median} \right) = \text{median} \{ B^i_c\}_{i=1,...,N}}{ascii}
#' **GeoMean**: Geometric mean of the best estimates
#' \mjdeqn{GeoMean_{c}=  \left(\prod_{i=1}^N  B_{i,c}\right)^{\frac{1}{N}}}{ascii}
#' **LOArMean**: Arithmetic mean of the log odds transformed best estimates
#' \mjdeqn{LogOdds_{i,c}= \frac{1}{N} \sum_{i=1}^N log\left( \frac{B_{i,c}}{1-B_{i,c}}\right)}{ascii}
#' The average log odds estimate is then back transformed to give a final group estimate:
#' \mjdeqn{\hat{p}_c\left( LOArMean \right) = \frac{e^{LogOdds_{i,c}}}{1+e^{LogOdds_{i,c}}}}{ascii}
#' **ProbitArMean**: Arithmetic mean of the probit transformed best estimates
#' \mjdeqn{Probit_{c}= \frac{1}{N} \sum_{i=1}^N \Phi^{-1}\left( B_{i,c}\right)}{ascii}
#' The average probit estimate is then back transformed to give a final group estimate:
#' \mjdeqn{\hat{p}_c\left(ProbitArMean \right) = \Phi\left({Probit_{c}}\right)}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"ArMean"`, `"Median"`, `"GeoMean"`, `"LOArMean"`, or `"ProbitArMean"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{AverageWAgg(data_ratings)}
#'
#' @export
#' @md

AverageWAgg <- function(
  expert_judgements,
  type = "ArMean",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
) {
  if (
    !(type %in%
      c("ArMean", "GeoMean", "Median", "LOArMean", "LOGeoMean", "ProbitArMean"))
  ) {
    stop(
      '`type` must be one of "ArMean", "GeoMean", "Median", "LOArMean", "LOGeoMean", or "ProbitArMean"'
    )
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("AverageWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::filter(element == "three_point_best") %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "ArMean" = {
        df <- df %>%
          dplyr::summarise(
            aggregated_judgement = mean(value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "GeoMean" = {
        df <- df %>%
          dplyr::summarise(
            n_experts = dplyr::n(),
            aggregated_judgement = (prod(value, na.rm = TRUE))^(1 / n_experts)
          )
      },
      "Median" = {
        df <- df %>%
          dplyr::summarise(
            aggregated_judgement = median(value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "LOArMean" = {
        if (any(df$value < 0) | any(df$value > 1)) {
          stop(
            "LOArMean requires probabilistic judgements. Check your data compatability or `percent_toggle` argument."
          )
        }

        df <- df %>%
          dplyr::mutate(
            value = dplyr::case_when(
              value == 1 ~ value - .Machine$double.eps,
              value == 0 ~ value + .Machine$double.eps,
              TRUE ~ value
            ),
            log_odds = log(abs(value / (1 - value)))
          ) %>%
          dplyr::summarise(
            aggregated_judgement = mean(log_odds, na.rm = TRUE),
            n_experts = dplyr::n()
          ) %>%
          dplyr::mutate(
            aggregated_judgement = exp(aggregated_judgement) /
              (1 + exp(aggregated_judgement))
          )
      },
      "LOGeoMean" = {
        if (any(df$value < 0) | any(df$value > 1)) {
          stop(
            "LOGeoMean requires probabilistic judgements. Check your data compatability or `percent_toggle` argument."
          )
        }

        df <- df %>%
          dplyr::mutate(
            value = dplyr::case_when(
              value == 1 ~ value - .Machine$double.eps,
              value == 0 ~ value + .Machine$double.eps,
              value == 0.5 ~ value + .Machine$double.eps,
              TRUE ~ value
            ),
            log_odds = log(abs(value / (1 - value)))
          ) %>%
          # dplyr::summarise(n_experts = dplyr::n(),
          #                  aggregated_judgement = (prod(log_odds, na.rm = TRUE))) %>%
          dplyr::summarise(
            n_experts = dplyr::n(),
            aggregated_judgement = (prod(log_odds, na.rm = TRUE))^(1 /
              n_experts)
          ) %>%
          dplyr::mutate(
            aggregated_judgement = exp(aggregated_judgement) /
              (1 + exp(aggregated_judgement))
          )
      },
      "ProbitArMean" = {
        df <- df %>%
          dplyr::mutate(
            probit = VGAM::probitlink(value, bvalue = .Machine$double.eps)
          ) %>%
          dplyr::summarise(
            aggregated_judgement = mean(probit, na.rm = TRUE),
            n_experts = dplyr::n()
          ) %>%
          dplyr::mutate(
            aggregated_judgement = VGAM::probitlink(
              aggregated_judgement,
              inverse = TRUE
            )
          )
      }
    )

    df %>%
      dplyr::mutate(method = name) %>%
      postprocess_judgements()
  }
}

#' @title
#' Aggregation Method: ShiftingWAgg
#'
#' @description
#' Weighted by judgements that shift the most after discussion
#'
#' @details
#' When judgements are elicited using the IDEA protocol (or any other protocol that allows
#' experts to revisit their original estimates), the second round of estimates may differ
#' from the original first set of estimates an expert provides. Greater changes between rounds
#' will be given greater weight.
#'
#' `type` may be one of the following:
#'
#' \loadmathjax
#' **ShiftWAgg**: Takes into account the shift in all three estimates
#'
#' Considers shifts across lower, \mjeqn{L_{i,c}}{ascii}, and upper,
#' \mjeqn{U_{i,c}}{ascii}, confidence limits, and the best estimate, \mjeqn{B_{i,c}}{ascii}.
#' More emphasis is placed on changes in the best estimate such that:
#'
#' \mjdeqn{w\_Shift_{i,c} = |B1_{i,c} - B_{i,c}| + \frac{|L1_{i,c} - L_{i,c}|+|U1_{i,c} - U_{i,c}|}{2},}{ascii}
#'
#' where \mjeqn{L1_{i,c}, B1_{i,c},U1_{i,c}}{ascii} are the first round lower, best and upper estimates (prior
#' to discussion) and \mjeqn{L_{i,c}, B_{i,c},U1_{i,c}}{ascii} are the individual’s revised second round estimates
#' (after discussion).
#'
#' \mjdeqn{\hat{p}_c(ShiftWAgg) = \sum_{i=1}^N \tilde{w}\_Shift_{i,c}B_{i,c}}{ascii}
#'
#' **BestShiftWAgg**: Weighted according to shifts in best estimates alone
#'
#' Taking into account the fact that the scales best estimates are measured on are bounded,
#' we can calculate shifts relative to the largest possible shift.
#'
#' \mjtdeqn{
#' w\_BestShift_{i,c}=
#' \begin{cases}
#' \frac{|B1_{i,c} - B_{i,c}|}{B1_{i,c}},
#' \begin{aligned}
#' \displaystyle &\ for\ (B1_{i,c} > 0.5\ and\ B_{i,c} \le 0.5) \cr
#' \displaystyle &\ or\ B_{i,c} < B1_{i,c} \le 0.5\ or\ B1_{i,c} > B_{i,c} > 0.5
#' \end{aligned} \cr
#' \frac{|B1_{i,c} - B_{i,c}|}{1- B1_{i,c}},
#' \begin{aligned}
#' \displaystyle &\ for\ (B1_{i,c} < 0.5\ and\ B_{i,c} \ge 0.5) \cr
#' \displaystyle &\ or\ B1_{i,c} < B_{i,c} < 0.5\ or\ B_{i,c} > B1_{i,c} > 0.5.
#' \end{aligned}
#' \end{cases}}{%
#' w\_BestShift_{i,c}=
#' \begin{cases}
#' \frac{|B1_{i,c} - B_{i,c}|}{B1_{i,c}},
#' \begin{aligned}
#' \displaystyle &\ for\ (B1_{i,c} \gt 0.5\ and\ B_{i,c} \le 0.5) \cr
#' \displaystyle &\ or\ B_{i,c} \lt B1_{i,c} \le 0.5\ or\ B1_{i,c} \gt B_{i,c} \gt 0.5
#' \end{aligned} \cr
#' \frac{|B1_{i,c} - B_{i,c}|}{1- B1_{i,c}},
#' \begin{aligned}
#' \displaystyle &\ for\ (B1_{i,c} \lt 0.5\ and\ B_{i,c} \ge 0.5) \cr
#' \displaystyle &\ or\ B1_{i,c} \lt B_{i,c} \lt 0.5\ or\ B_{i,c} \gt B1_{i,c} \gt 0.5.
#' \end{aligned}
#' \end{cases}}{ascii}
#'
#' \mjdeqn{\hat{p}_c(BestShiftWAgg) = \sum_{i=1}^N \tilde{w}\_BestShift_{i,c}B_{i,c}}{ascii}
#'
#' **IntShiftWAgg**: Weighted by shifts in interval widths alone.
#'
#' Individuals whose interval widths narrow between rounds are given more weight.
#'
#' \mjdeqn{w\_IntShift_{i,c} = \frac{1}{(U_{i,c}-L_{i,c})-(U1_{i,c}-L1_{i,c})+1}}{ascii}
#'
#' \mjdeqn{\hat{p}_c(IntShiftWAgg) = \sum_{i=1}^N \tilde{w}\_IntShift_{i,c}B_{i,c}}{ascii}
#'
#' **DistShiftWAgg**: Weighted by whether best estimates become more extreme (closer to 0 or 1) between rounds.
#'
#' \mjdeqn{w\_DistShift_{i,c} = 1 - (\min (B_{i,c}, 1-B_{i,c}) - \min (B1_{i,c}, 1-B1_{i,c}))}{ascii}
#'
#' \mjdeqn{\hat{p}_c(DistShiftWAgg) = \sum_{i=1}^N \tilde{w}\_DistShift_{i,c}B_{i,c}}{ascii}
#'
#' **DistIntShiftWAgg**: Rewards both narrowing of intervals and shifting towards the certainty limits between rounds.
#'
#' We simply multiply the weights calculated in the "DistShiftWAgg" and "IntShiftWAgg" methods.
#'
#' \mjdeqn{w\_DistIntShift_{i,c} = \tilde{w}\_IntShift_{i,c} \cdot \tilde{w}\_DistShift_{i,c}}{ascii}
#'
#' \mjdeqn{\hat{p}_c(DistIntShiftWAgg) = \sum_{i=1}^N \tilde{w}\_DistIntShift_{i,c}B_{i,c}}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"ShiftWAgg"`, `"BestShiftWAgg"`, `"IntShiftWAgg"`, `"DistShiftWAgg"`, or `"DistIntShiftWAgg"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{ShiftingWAgg(data_ratings)}
#'
#' @export
#' @md

ShiftingWAgg <- function(
  expert_judgements,
  type = "ShiftWAgg",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE
) {
  if (
    !(type %in%
      c(
        "ShiftWAgg",
        "BestShiftWAgg",
        "IntShiftWAgg",
        "DistShiftWAgg",
        "DistIntShiftWAgg"
      ))
  ) {
    stop(
      '`type` must be one of "ShiftWAgg", "BestShiftWAgg", "IntShiftWAgg", "DistShiftWAgg", or "DistIntShiftWAgg"'
    )
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("ShiftingWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = FALSE
      ) %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "ShiftWAgg" = {
        n_experts <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::summarise(n = dplyr::n()) %>%
          dplyr::count() %>%
          dplyr::rename(n_experts = n)

        ## Calculate absolute value of change between rounds

        weights_best <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(diff_best = abs(round_1 - round_2))

        weights_lower <- df %>%
          dplyr::filter(element == "three_point_lower") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(diff_lower = abs(round_1 - round_2)) %>%
          dplyr::select(paper_id, user_name, diff_lower)

        weights_upper <- df %>%
          dplyr::filter(element == "three_point_upper") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(diff_upper = abs(round_1 - round_2)) %>%
          dplyr::select(paper_id, user_name, diff_upper)

        weights <- weights_best %>%
          dplyr::left_join(weights_lower, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_upper, by = c("paper_id", "user_name")) %>%
          dplyr::mutate(
            aggregate_diff = diff_best + (diff_lower / 2) + (diff_upper / 2)
          ) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            aggregate_diff = dplyr::case_when(
              all(aggregate_diff == 0) ~ 1,
              TRUE ~ aggregate_diff
            )
          ) %>%
          dplyr::ungroup()

        ## Weight calculation

        agg_weights <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(aggregate_diff, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df <- weights %>%
          dplyr::left_join(agg_weights, by = "paper_id") %>%
          dplyr::mutate(
            agg_weight = aggregate_diff / agg_sum
          ) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * round_2, na.rm = TRUE),
          ) %>%
          dplyr::left_join(n_experts, by = "paper_id")
      },
      "BestShiftWAgg" = {
        n_experts <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::summarise(n = dplyr::n()) %>%
          dplyr::count() %>%
          dplyr::rename(n_experts = n)

        ## Calculate absolute value of change between rounds

        weights <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(
            diff_best = dplyr::if_else(
              round_2 < round_1 &
                round_1 <= 0.5 |
                round_1 > round_2 & round_2 > 0.5 |
                round_1 > 0.5 & round_2 <= 0.5,
              abs(round_1 - round_2) / round_1,
              abs(round_1 - round_2) / (1 - round_1)
            )
          ) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            aggregate_diff = dplyr::case_when(
              all(diff_best == 0) ~ 1,
              TRUE ~ diff_best
            )
          ) %>%
          dplyr::ungroup()

        ## Weight calculation

        agg_weights <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(aggregate_diff, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df <- weights %>%
          dplyr::left_join(agg_weights, by = "paper_id") %>%
          dplyr::mutate(agg_weight = aggregate_diff / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * round_2, na.rm = TRUE)
          ) %>%
          dplyr::left_join(n_experts, by = "paper_id")
      },
      "IntShiftWAgg" = {
        n_experts <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::summarise(n = dplyr::n()) %>%
          dplyr::count() %>%
          dplyr::rename(n_experts = n)

        ## Calculate interval width change between rounds

        weights <- df %>%
          tidyr::pivot_wider(names_from = "element", values_from = "value") %>%
          dplyr::mutate(width = three_point_upper - three_point_lower) %>%
          dplyr::select(user_name, paper_id, round, width) %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "width",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(diff_width = round_2 - round_1) %>%
          dplyr::select(-round_2, -round_1)

        agg_weights <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::mutate(agg_weight = 1 / (diff_width + 1)) %>%
          dplyr::ungroup()

        # standardise weight
        df <- agg_weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE)) %>%
          dplyr::full_join(agg_weights, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * round_2, na.rm = TRUE)
          ) %>%
          dplyr::left_join(n_experts, by = "paper_id")
      },
      "DistShiftWAgg" = {
        n_experts <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::summarise(n = dplyr::n()) %>%
          dplyr::count() %>%
          dplyr::rename(n_experts = n)

        ## Calculate absolute value of change between rounds

        weights <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(
            dist_1 = min(round_1, 1 - round_1, na.rm = TRUE),
            dist_2 = min(round_2, 1 - round_2, na.rm = TRUE),
            diff_dist = dist_2 - dist_1,
            aggregate_diff = 1 - diff_dist
          )

        ## Weight calculation

        agg_weights <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(aggregate_diff, na.rm = TRUE))

        df <- weights %>%
          dplyr::left_join(agg_weights, by = "paper_id") %>%
          dplyr::mutate(agg_weight = aggregate_diff / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * round_2, na.rm = TRUE)
          ) %>%
          dplyr::left_join(n_experts, by = "paper_id")
      },
      "DistIntShiftWAgg" = {
        n_experts <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::summarise(n = dplyr::n()) %>%
          dplyr::count() %>%
          dplyr::rename(n_experts = n)

        ## Calculate interval width change between rounds

        weights <- df %>%
          tidyr::pivot_wider(names_from = "element", values_from = "value") %>%
          dplyr::mutate(width = three_point_upper - three_point_lower) %>%
          dplyr::select(user_name, paper_id, round, width) %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          dplyr::mutate(row = dplyr::row_number()) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "width",
            values_fill = NA
          ) %>%
          dplyr::select(-row) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(diff_width = round_2 - round_1) %>%
          dplyr::select(-round_2, -round_1)

        int_shift <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::mutate(agg_weight = 1 / (diff_width + 1)) %>%
          dplyr::rename(int_shift = agg_weight) %>%
          dplyr::ungroup()

        dist_shift <- df %>%
          dplyr::filter(element == "three_point_best") %>%
          dplyr::group_by(user_name, paper_id, round) %>%
          tidyr::pivot_wider(
            names_from = "round",
            values_from = "value",
            values_fill = NA
          ) %>%
          dplyr::filter(!is.na(round_2)) %>%
          dplyr::mutate(
            dist_1 = min(round_1, 1 - round_1, na.rm = TRUE),
            dist_2 = min(round_2, 1 - round_2, na.rm = TRUE),
            diff_dist = dist_2 - dist_1,
            dist_shift = 1 - diff_dist
          ) %>%
          dplyr::select(paper_id, user_name, dist_shift)

        ## Weight calculation

        agg_weights <- int_shift %>%
          dplyr::left_join(dist_shift, by = c("paper_id", "user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            int_weight = int_shift /
              sum(int_shift, na.rm = TRUE),
            dist_shift = dist_shift /
              sum(dist_shift, na.rm = TRUE),
            agg_weight = int_weight * dist_shift
          )

        # standardise weight
        df <- agg_weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE)) %>%
          dplyr::full_join(agg_weights, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          # calculate aggregated judgement by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * round_2, na.rm = TRUE)
          ) %>%
          dplyr::left_join(n_experts, by = "paper_id")
      }
    )

    df %>%
      dplyr::mutate(method = name) %>%
      postprocess_judgements()
  }
}

#' @title Weighting method: Total number of judgement reasons
#'
#' @description
#' This function is used by [ReasoningWAgg] to weighting functions for the aggregation
#' type `"ReasonWAgg"`. Calculates weights based on the number of judgement reasoning
#' methods used by an individual
#'
#' @details
#' Individuals' weight is equal to the maximum number of judgement reasons given
#'
#' @param expert_reasons A dataframe in the form of [data_supp_reasons]
#'
#' @return A tibble of three columns `paper_id`, `user_name`, and `reason_count`
#'
#' @family weighting functions
#'
#' @export

weight_reason <- function(expert_reasons) {
  ## Calculate reason counts

  expert_reasons %>%
    dplyr::mutate(dplyr::across(
      .cols = dplyr::starts_with("RW"),
      .fns = ~ dplyr::if_else(.x > 1, 1, .x)
    )) %>%
    dplyr::mutate(
      reason_count = rowSums(dplyr::select(., -paper_id, -user_name)),
      reason_count = dplyr::na_if(reason_count, 0)
    ) %>%
    dplyr::select(paper_id, user_name, reason_count)
}

#' aggreCAT: mathematically aggregating expert judgements
#'
#' To learn more about \pkg{aggreCAT}, start with the vignettes: \code{vignette(package = "aggreCAT")}
#'
#' @docType package
#' @name aggreCAT
#' @import mathjaxr
#'
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' @title
#' Aggregation Method: LinearWAgg
#'
#' @description
#' Calculate one of several types of linear-weighted best estimates.
#'
#' @details
#' This function returns weighted linear combinations of the best-estimate judgements for each claim.
#'
#' `type` may be one of the following:
#' \loadmathjax
#'
#' **Judgement**: Weighted by user-supplied weights at the judgement level
#' \mjdeqn{\hat{p}_c\left( JudgementWeights \right) = \sum_{i=1}^N
#' \tilde{w}\_judgement_{i,c}B_{i,c}}{ascii}
#'
#' **Participant**: Weighted by user-supplied weights at the participant level
#' \mjdeqn{\hat{p}_c\left( ParticipantWeights \right) = \sum_{i=1}^N
#' \tilde{w}\_participant_{i}B_{i,c}}{ascii}
#'
#' **DistLimitWAgg**: Weighted by the distance of the best estimate from the
#' closest certainty limit. Giving greater weight to best estimates that are closer to certainty
#' limits may be beneficial.
#' \mjdeqn{w\_distLimit_{i,c} = \max \left(B_{i,c}, 1-B_{i,c}\right)}{ascii}
#' \mjdeqn{\hat{p}_c\left( DistLimitWAgg \right) = \sum_{i=1}^N
#' \tilde{w}\_distLimit_{i,c}B_{i,c}}{ascii}
#'
#' **GranWAgg**: Weighted by the granularity of best estimates
#'
#' Individuals are weighted by whether or not their best estimates are more
#' granular than a level of 0.05 (i.e., not a multiple of 0.05).
#' \mjdeqn{w\_gran_{i} = \frac{1}{C} \sum_{d=1}^C \left\lceil{\frac{B_{i,d}}
#' {0.05}-\left\lfloor{\frac{B_{i,d}}{0.05}}\right\rfloor}\right\rceil,}{ascii}
#'
#' where \mjeqn{\lfloor{\ }\rfloor}{ascii} and \mjeqn{\lceil{\ }\rceil}{ascii}
#' are the mathematical floor and ceiling functions respectively.
#' \mjdeqn{\hat{p}_c\left( GranWAgg \right) = \sum_{i=1}^N \tilde{w}\_gran_{i}
#' B_{i,c}}{ascii}
#'
#' **OutWAgg**: Down weighting outliers
#'
#' This method down-weights outliers by using the differences from the central
#' tendency (median) of an individual's best estimates.
#' \mjdeqn{d_{i,c} = \left(median\{{B_{i,c}}_{_{i=1,...,N}}\} - B_{i,c}\right)^2}{ascii}
#' \mjdeqn{w\_out_{i} = 1 - \frac{d_{i,c}}{\max({d_c})})}{ascii}
#' \mjdeqn{\hat{p}_c\left( OutWAgg \right) = \sum_{i=1}^N \tilde{w}\_out_{i}B_{i,c}}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"Judgement"`, `"Participant"`, `"DistLimitWAgg"`, `"GranWAgg"`, or `"OutWAgg"`.
#' @param weights (Optional) A two column dataframe (`user_name` and `weight`) for `type = "Participant"`
#' or a three two column dataframe (`paper_id', 'user_name` and `weight`) for `type = "Judgement"`
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param flag_loarmean A toggle to impute log mean (defaults `FALSE`).
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{LinearWAgg(data_ratings, percent_toggle = TRUE)}
#'
#' @export
#' @md

LinearWAgg <- function(
  expert_judgements,
  type = "DistLimitWAgg",
  weights = NULL,
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  flag_loarmean = FALSE,
  round_2_filter = TRUE
) {
  if (
    !(type %in%
      c(
        "Judgement",
        "Judgement_LO",
        "Participant",
        "Participant_LO",
        "DistLimitWAgg",
        "GranWAgg",
        "OutWAgg"
      ))
  ) {
    stop(
      '`type` must be one of "Judgement", "Judgement_LO", "Participant", "Participant_LO", "DistLimitWAgg", "GranWAgg", or "OutWAgg"'
    )
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("LinearWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::filter(element == "three_point_best") %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "Judgement" = {
        ## User-supplied weights at the judgement-level

        weights_sum <- df %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(weight, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df <- df %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "Judgement_LO" = {
        ## User-supplied weights at the judgement-level with log-odds default for no weights

        weights_df <- df %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(no_weights_for_claim = all(is.na(weight))) %>%
          dplyr::ungroup()

        weights_sum <- weights_df %>%
          dplyr::filter(no_weights_for_claim == FALSE) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(weight, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df_judgement <- weights_df %>%
          dplyr::filter(no_weights_for_claim == FALSE) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )

        df_LO_data <- weights_df %>%
          dplyr::filter(no_weights_for_claim == TRUE)

        df_LO <- if (nrow(df_LO_data) > 0) {
          df_LO_data %>%
            dplyr::group_by(paper_id) %>%
            dplyr::mutate(
              value = dplyr::if_else(
                value == 1 | value == 0,
                value + .Machine$double.eps,
                value
              )
            ) %>%
            dplyr::mutate(log_odds = log(abs(value / (1 - value)))) %>%
            dplyr::summarise(
              aggregated_judgement = mean(log_odds, na.rm = TRUE),
              n_experts = dplyr::n()
            ) %>%
            dplyr::mutate(
              aggregated_judgement = exp(aggregated_judgement) /
                (1 + exp(aggregated_judgement))
            )
        } else {
          NULL
        }

        df <- dplyr::bind_rows(df_judgement, df_LO)

        LO_papers <- df_LO_data %>%
          dplyr::pull(paper_id) %>%
          unique()
      },
      "Participant" = {
        ## User-supplied weights at the participant-level

        weights_sum <- df %>%
          dplyr::left_join(weights, by = c("user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(weight, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df <- df %>%
          dplyr::left_join(weights, by = c("user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "Participant_LO" = {
        ## User-supplied weights at the participant-level with log odds default for no weights

        weights_df <- df %>%
          dplyr::left_join(weights, by = c("user_name")) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(no_weights_for_claim = all(is.na(weight))) %>%
          dplyr::ungroup()

        weights_sum <- weights_df %>%
          dplyr::filter(no_weights_for_claim == FALSE) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(weight, na.rm = TRUE),
            # set to arbitrarily small number if 0
            agg_sum = dplyr::if_else(agg_sum == 0, .Machine$double.eps, agg_sum)
          )

        df_participant <- weights_df %>%
          dplyr::filter(no_weights_for_claim == FALSE) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )

        df_LO_data <- weights_df %>%
          dplyr::filter(no_weights_for_claim == TRUE)

        df_LO <- if (nrow(df_LO_data) > 0) {
          df_LO_data %>%
            dplyr::group_by(paper_id) %>%
            dplyr::mutate(
              value = dplyr::if_else(
                value == 1 | value == 0,
                value + .Machine$double.eps,
                value
              )
            ) %>%
            dplyr::mutate(log_odds = log(abs(value / (1 - value)))) %>%
            dplyr::summarise(
              aggregated_judgement = mean(log_odds, na.rm = TRUE),
              n_experts = dplyr::n()
            ) %>%
            dplyr::mutate(
              aggregated_judgement = exp(aggregated_judgement) /
                (1 + exp(aggregated_judgement))
            )
        } else {
          NULL
        }

        df <- dplyr::bind_rows(df_participant, df_LO)

        LO_papers <- df_LO_data %>%
          dplyr::pull(paper_id) %>%
          unique()
      },
      "DistLimitWAgg" = {
        if (any(df$value < 0) | any(df$value > 1)) {
          stop(
            "DistLimitWAgg requires judgements bounded 0-1. Check your data compatability or `percent_toggle` argument."
          )
        }

        weights <- df %>%
          dplyr::group_by(paper_id, user_name) %>%
          dplyr::mutate(agg_weight = max(value, 1 - value, na.rm = TRUE)) %>%
          dplyr::ungroup()

        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- weights %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "GranWAgg" = {
        if (any(df$value < 0) | any(df$value > 1)) {
          stop(
            "GranWAgg requires probabilistic judgements. Check your data compatability or `percent_toggle` argument."
          )
        }

        weights <- df %>%
          ## Calculate observation-level value
          ## .Machine$double.eps correction is to handle nonsense floating point issue: e.g. floor(0.15/0.05)
          dplyr::mutate(
            gran_binary = ceiling(
              value / 0.05 - floor(value / 0.05 + .Machine$double.eps * 4)
            )
          ) %>%
          ## Caclulate weight by individual
          dplyr::group_by(user_name) %>%
          dplyr::summarise(
            agg_weight = mean(gran_binary, na.rm = TRUE),
            # to avoid dividing by 0
            agg_weight = dplyr::if_else(
              agg_weight == 0,
              .Machine$double.eps,
              agg_weight
            )
          )

        weights_sum <- df %>%
          dplyr::left_join(weights, by = "user_name") %>%
          ## Calculate scaled weight by claim
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(agg_sum = sum(agg_weight, na.rm = TRUE))

        df <- df %>%
          dplyr::left_join(weights, by = "user_name") %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      },
      "OutWAgg" = {
        # get weights
        weights <- df %>%
          weight_outlier() %>%
          dplyr::group_by(paper_id) %>%
          dplyr::mutate(
            agg_weight = dplyr::case_when(
              all(agg_weight == 0) ~ 1,
              TRUE ~ agg_weight
            )
          ) %>%
          dplyr::ungroup() %>%
          dplyr::select(paper_id, user_name, agg_weight)

        # standardise weights
        weights_sum <- weights %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            agg_sum = sum(agg_weight, na.rm = TRUE)
          )

        df <- df %>%
          dplyr::left_join(weights, by = c("paper_id", "user_name")) %>%
          dplyr::left_join(weights_sum, by = "paper_id") %>%
          dplyr::mutate(agg_weight = agg_weight / agg_sum) %>%
          dplyr::group_by(paper_id) %>%
          dplyr::summarise(
            aggregated_judgement = sum(agg_weight * value, na.rm = TRUE),
            n_experts = dplyr::n()
          )
      }
    )

    df <- df %>%
      dplyr::mutate(method = name)

    if (flag_loarmean) {
      df <- df %>%
        dplyr::mutate(
          method = dplyr::if_else(paper_id %in% LO_papers, "LOArMean", method)
        )
    }

    df %>%
      postprocess_judgements()
  }
}

#' @title
#' Post-processing.
#'
#' @description
#' Standardise the output from aggregation method's.
#' This function is called by every aggregation method as a final step.
#'
#' @param method_output tibble created from one of the aggregation methods after
#' pre-processing' with columns for the aggregation `method`, `paper_id`, `aggregated_judgement` and
#' `n_experts`
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`, corresponding
#' with an aggregation `method` (character).
#'
#' @export

postprocess_judgements <- function(method_output) {
  method_output %>%
    dplyr::select(method, paper_id, aggregated_judgement, n_experts) %>%
    dplyr::rename(
      cs = aggregated_judgement
    )
}

#' @title
#' Weighting method: Individually scaled interval widths
#'
#' @description
#' Weighted by the rescaled interval width within individuals across claims.
#'
#' @details
#' \loadmathjax
#' This function is used inside [IntervalWAgg] for aggregation types `"IndIntWAgg"`,
#' `"IndIntAsymWAgg"` and `"KitchSinkWAgg"`. Interval width weights are rescaled
#' relative to an individuals interval widths across all claims.
#'
#' \mjdeqn{w\_nIndivInterval_{i,c} = \frac{1}{\frac{U_{i,c}-L_{i,c}}{\max\left(\{(U_{i,d}-L_{i,d}):d=1,...,C\}\right)}}}{ascii}
#'
#' @param expert_judgements A dataframe in the form of [data_ratings]
#'
#' @return A tibble in the form of the input `expert_judgements` argument with additional columns
#' supplying the calculated weight for each row's observation.
#'
#' @family weighting functions
#'
#' @export

weight_nIndivInterval <- function(expert_judgements) {
  expert_judgements %>%
    tidyr::pivot_wider(names_from = element, values_from = value) %>%
    dplyr::group_by(user_name) %>%

    # calculate weight weight
    dplyr::summarise(
      max_agg = abs(max(three_point_upper - three_point_lower, na.rm = TRUE))
    ) %>%
    dplyr::full_join(expert_judgements, by = "user_name") %>%
    tidyr::pivot_wider(names_from = element, values_from = value) %>%
    dplyr::mutate(
      int_agg = abs(three_point_upper - three_point_lower),
      # check we're not dividing by zero
      max_agg = dplyr::if_else(max_agg == 0, .Machine$double.eps, max_agg),
      int_agg = dplyr::if_else(int_agg == 0, .Machine$double.eps, int_agg),
      agg_weight = 1 / (int_agg / max_agg)
    ) %>%
    # Inverse of re-scaled interval width)
    tidyr::pivot_longer(
      c(three_point_upper, three_point_lower, three_point_best),
      names_to = "element",
      values_to = "value"
    ) %>%
    dplyr::select(-max_agg, -int_agg) %>%
    dplyr::filter(element == "three_point_best")
}

#' @title
#' Aggregation Method: DistributionWAgg
#'
#' @description
#' Calculate the arithmetic mean of distributions created with expert judgements.
#' The aggregate is the median of the average distribution fitted on the individual estimates.
#'
#' @details
#'
#' \loadmathjax
#'
#' This method assumes that the elicited probabilities and bounds can be considered
#' to represent participants' subjective distributions associated with relative
#' frequencies (rather than unique events). That is to say that we considered that
#' the lower bound of the individual per claim corresponds to the 5th percentile
#' of their subjective distribution on the probability of replication, denoted
#' \mjeqn{q_{5,i}}{ascii}, the best estimate corresponds to the median, \mjeqn{q_{50,i}}{ascii}, and the upper
#' bound corresponds to the 95th percentile, \mjeqn{q_{95,i}}{ascii}. With these three
#' percentiles, we can fit parametric or non-parametric distributions and aggregate
#' them rather than the (point) best estimates.
#'
#' `type` may be one of the following:
#'
#' **DistribArMean**: Applies a non-parametric distribution evenly across upper, lower and best estimates.
#'
#' Using the three percentiles we can build the minimally
#' informative non-parametric distribution that spreads the mass uniformly between
#' the three percentiles.
#'
#' \mjtdeqn{
#' F_{i}(x) = \begin{cases}
#' \displaystyle 0, \text{ for } x < 0 \cr
#' \displaystyle \frac{0.05}{q_{5,i}}\cdot x, \text{ for } 0 < x < q_{5,i}\cr
#' \displaystyle \frac{0.45}{q_{50,i}-q_{5,i}}\cdot(x-q_{5,i})+0.05, \text{ for } q_{5,i}\le x < q_{50,i}\cr
#' \displaystyle \frac{0.45}{q_{95,i}-q_{50,i}}\cdot(x-q_{50,i})+0.5, \text{ for } q_{50,i}\le x < q_{95,i}\cr
#' \displaystyle \frac{0.05}{1 - q_{95,i}}\cdot(x-q_{95,i})+0.95, \text{ for } q_{95,i}\le x < 1\cr
#' \displaystyle 1,  \text{ for } x \ge 1.
#' \end{cases}}{%
#' F_{i}(x) = \begin{cases}
#' \displaystyle 0, \text{ for } x \lt 0 \cr
#' \displaystyle \frac{0.05}{q_{5,i}}\cdot x, \text{ for } 0 \le x \lt q_{5,i}\cr
#' \displaystyle \frac{0.45}{q_{50,i}-q_{5,i}}\cdot(x-q_{5,i})+0.05, \text{ for } q_{5,i}\le x \lt q_{50,i}\cr
#' \displaystyle \frac{0.45}{q_{95,i}-q_{50,i}}\cdot(x-q_{50,i})+0.5, \text{ for } q_{50,i}\le x \lt q_{95,i}\cr
#' \displaystyle \frac{0.05}{1 - q_{95,i}}\cdot(x-q_{95,i})+0.95, \text{ for } q_{95,i}\le x \lt 1\cr
#' \displaystyle 1,  \text{ for } x \ge 1.
#' \end{cases}}{ascii}
#'
#' Then take the average of all constructed distributions of participants for each claim:
#'
#' \mjdeqn{AvDistribution = \frac{1}{N}\sum_{i=1}^N F_i(x),}{ascii}
#'
#' and the aggregation is the median of the average distribution:
#'
#' \mjdeqn{\hat{p}_c\left( DistribArMean \right) = AvDistribution^{-1}(0.5).}{ascii}
#'
#' **TriDistribArMean**: Applies a triangular distribution to the upper, lower and best estimates.
#'
#' A more restrictive fit with different assumptions about the
#' elicited best estimates, upper and lower bounds. We can assume that the lower and upper
#' bounds form the support of the distribution, and the best estimate corresponds to the mode.
#'
#' \mjtdeqn{
#' F_{i}(x) = \begin{cases}
#' \displaystyle 0, \text{ for } x < L_{i} \cr
#' \displaystyle \frac{\left(x-L_{i}\right)^2}{\left( U_{i}-L_{i}\right)\left(B_{i}-L_{i}\right)}, \text{ for } L_{i} \le x < B_{i}\cr
#' \displaystyle 1 - \frac{\left( U_{i}-x\right)^2}{\left( U_{i}-L_{i}\right)\left( U_{i}-B_{i}\right)}, \text{ for } B_{i} \le x < U_{i}\cr
#' \displaystyle 1,  \text{ for } x \ge U_{i}.
#' \end{cases}}{%
#' F_{i}(x) = \begin{cases}
#' \displaystyle 0, \text{ for } x \lt L_{i} \cr
#' \displaystyle \frac{\left(x-L_{i}\right)^2}{\left( U_{i}-L_{i}\right)\left(B_{i}-L_{i}\right)}, \text{ for } L_{i} \le x \lt B_{i}\cr
#' \displaystyle 1 - \frac{\left( U_{i}-x\right)^2}{\left( U_{i}-L_{i}\right)\left( U_{i}-B_{i}\right)}, \text{ for } B_{i} \le x \lt U_{i}\cr
#' \displaystyle 1,  \text{ for } x \ge U_{i}.
#' \end{cases}}{ascii}
#'
#' Then take the average of all constructed distributions of participants for each claim:
#'
#' \mjdeqn{    AvDistribution = \frac{1}{N}\sum_{i=1}^N F_i(x),}{ascii}
#'
#' and the aggregation is the median of the average distribution:
#'
#' \mjdeqn{    \hat{p}_c\left(TriDistribArMean\right) = AvDistribution^{-1}(0.5).}{ascii}
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"DistribArMean"` or `"TriDistribArMean"`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \donttest{DistributionWAgg(data_ratings, percent_toggle = TRUE)}
#'
#' @export
#' @md

DistributionWAgg <- function(
  expert_judgements,
  type = "DistribArMean",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
) {
  if (!(type %in% c("DistribArMean", "TriDistribArMean"))) {
    stop('`type` must be one of "DistribArMean" or "TriDistribArMean"')
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("DistributionWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }}
      ) %>%
      dplyr::group_by(paper_id)

    switch(
      type,
      "DistribArMean" = {
        Fx_fun <- function(x, lower, best, upper) {
          dplyr::case_when(
            x < 0 ~ 0,
            x >= 0 & x < lower ~ 0.05 / lower * x,
            x >= lower &
              x < best ~ 0.45 / (best - lower) * (x - lower) + 0.05,
            x >= best &
              x < upper ~ 0.45 / (upper - best) * (x - best) + 0.5,
            x >= upper &
              x < 1 ~ 0.05 / (1 - upper) * (x - upper) + 0.95,
            x > 1 ~ 1
          )
        }

        avdist_fun <- function(dq, claim_input) {
          claim_input %>%
            dplyr::mutate(
              Fx = purrr::pmap(
                list(three_point_lower, three_point_best, three_point_upper),
                .f = function(l, b, u) {
                  function(x) {
                    Fx_fun(
                      x,

                      lower = l,
                      best = b,
                      upper = u
                    )
                  }
                }
              )
            ) %>%
            purrr::pluck("Fx") %>%
            purrr::map_dbl(
              .f = function(Fx_fun) {
                Fx_fun(dq)
              }
            ) %>%
            mean()
        }
      },
      "TriDistribArMean" = {
        Fx_fun <- function(x, lower, best, upper) {
          dplyr::case_when(
            x < lower ~ 0,
            x >= lower &
              x < best ~ ((x - lower)^2) / ((upper - lower) * (best - lower)),
            x >= best &
              x < upper ~ 1 -
              (((upper - x)^2) / ((upper - lower) * (upper - best))),
            x >= upper ~ 1
          )
        }

        avdist_fun <- function(dq, claim_input) {
          claim_input %>%
            dplyr::mutate(
              Fx = purrr::pmap(
                list(three_point_lower, three_point_best, three_point_upper),
                .f = function(l, b, u) {
                  function(x) {
                    Fx_fun(
                      x,

                      lower = l,
                      best = b,
                      upper = u
                    )
                  }
                }
              )
            ) %>%
            purrr::pluck("Fx") %>%
            purrr::map_dbl(
              .f = function(Fx_fun) {
                Fx_fun(dq)
              }
            ) %>%
            mean()
        }
      }
    )

    agg_judge_df <- df %>%
      tidyr::pivot_wider(names_from = element, values_from = value) %>%
      dplyr::mutate(
        three_point_upper = dplyr::if_else(
          three_point_upper == 1,
          three_point_upper - .Machine$double.eps,
          three_point_upper
        ),
        three_point_lower = dplyr::if_else(
          three_point_lower == 0,
          three_point_lower + .Machine$double.eps,
          three_point_lower
        ),
        # ensure the values are in order
        three_point_upper = dplyr::if_else(
          three_point_upper < three_point_best,
          three_point_best,
          three_point_upper
        ),
        three_point_lower = dplyr::if_else(
          three_point_lower > three_point_best,
          three_point_best,
          three_point_lower
        ),
      ) %>%
      dplyr::group_by(paper_id) %>%
      tidyr::nest() %>%
      dplyr::mutate(
        avdist = purrr::map(
          data,
          .f = function(data) {
            function(x) {
              avdist_fun(x, claim_input = data)
            }
          }
        )
      ) %>%
      dplyr::ungroup()

    quantiles <- agg_judge_df %>%
      dplyr::mutate(
        avdist_preimage = purrr::map(
          avdist,
          .f = function(f) {
            GoFKernel::inverse(f, lower = 0, upper = 1)
          }
        )
      ) %>%
      dplyr::mutate(
        aggregated_judgement = purrr::map_dbl(
          avdist_preimage,
          .f = function(f) {
            # suppressing warnings on preimage
            f(0.5)
          }
        )
      )

    n_experts <- df %>%
      dplyr::group_by(paper_id, user_name) %>%
      dplyr::summarise(n_experts = dplyr::n(), .groups = "drop_last") %>%
      dplyr::count() %>%
      dplyr::rename("n_experts" = "n") %>%
      dplyr::ungroup()

    x <- quantiles %>%
      dplyr::left_join(n_experts, by = "paper_id") %>%
      dplyr::select(paper_id, aggregated_judgement, n_experts) %>%
      dplyr::ungroup()

    x %>%
      dplyr::group_by(paper_id) %>%
      dplyr::mutate(method = name) %>%
      dplyr::ungroup() %>%
      postprocess_judgements()
  }
}

#' P1_ratings
#'
#' Anonymized expert judgements of known-outcome
#' claims, assessed at the 2019 SIPS repliCATS workshop
#'
#' @format A table with 6880 rows and 7 columns:
#' \describe{
#'   \item{round}{character string identifying whether the round was 1 (pre-discussion) or 2 (post-discussion)}
#'   \item{paper_id}{character string of the claim ids (25 unique claims total)}
#'   \item{user_name}{character string of anonymized IDs for each participant (25 participants included in this dataset)}
#'   \item{question}{character string for the question type, with four options: direct_replication, involved_binary, belief_binary, or comprehension}
#'   \item{element}{character string for the type of response coded in the row, with five options: three_point_lower, three_point_best, three_point_upper, binary_question, or likert_binary}
#'   \item{value}{numeric value for the participant's response}
#'   \item{group}{character string of group IDs that contained the participants}
#' }
"data_ratings"

#' Free-text justifications for expert judgements
#'
#' @format A table with 5630 rows and 9 columns:
#' \describe{
#'   \item{round}{character string identifying whether the round was 1 (pre-discussion) or 2 (post-discussion)}
#'   \item{paper_id}{character string of the paper ids (25 papers total)}
#'   \item{user_name}{character string of anonymized IDs for each participant (25 participants included in this dataset)}
#'   \item{question}{character string for the question type, with five options: flushing_freetext, involved_binary, belief_binary, direct_replication, and comprehension}
#'   \item{justification}{character string with participant's free-text rationale for their responses}
#'   \item{justification_id}{character string with a unique ID for each row}
#'   \item{vote_count}{numeric of recorded votes (all 0 or 1)}
#'   \item{vote_sum}{numeric of summed vote counts(all 0 or 1)}
#'   \item{group}{character string of group IDs that contained the participants}
#' }
"data_justifications"

#' data_comments
#' @format A tibble with 2 rows and 10 columns
#' \describe{
#'   \item{round}{character string, both 'round_1' (before discussion)}
#'   \item{paper_id}{character string identifying 2 unique papers}
#'   \item{user_name}{factor for anonymized IDs for two participants}
#'   \item{question}{character string for the type of question, both 'comprehension'}
#'   \item{justification_id}{character string identifying 2 unique justifications}
#'   \item{comment_id}{character string identifying 2 unique comments}
#'   \item{commenter}{redundant column, same as user_name}
#'   \item{comment}{character string with free-text response for the user}
#'   \item{vote_count}{numeric, both 0}
#'   \item{vote_sum}{numeric, both 0}
#'   \item{group}{character string of group IDs that contained the participants}
#' }
#'
"data_comments"

#' A table of prior means, to be fed into the BayPRIORsAgg aggregation method
#'
#' @format A tibble of 25 rows and 2 columns
#' \describe{
#'   \item{paper_id}{character string with a unique id for each row corresponding to the assessed claim (from 125 papers total)}
#'   \item{prior_means}{numeric with the average prior probability for the claim corresponding to the paper_id}
#' }
#'
"data_supp_priors"

#' A table of scores on the quiz to assess prior knowledge, to be fed into the QuizWAgg aggregation method
#'
#' @format A tibble 19 rows and 2 columns
#' \describe{
#'   \item{user_name}{factor for anonymized IDs for each participant}
#'   \item{quiz_score}{numeric for the participant's score on the quiz (min of 0, max of 16, NA if no questions answered)}
#'   \item{quiz_score_even}{numeric quiz scores where all questions are weighted equally (the original quiz score assigned only half marks for true/false questions)}
#'   \item{quiz_score_stats}{numeric participant score based only on questions pertaining to statistics}
#' }
#'
"data_supp_quiz"

#' Categories of reasons provided by participants for their expert judgements
#'
#' @format a tibble with 625 rows and 15 columns
#' \describe{
#'   \item{paper_id}{character string for the paper ID}
#'   \item{user_name}{character string for participant ID}
#'   \item{RW04 Date of publication}{numeric; references to the date of publication, for example in relation to something being published prior to the 'replication crisis' within the relevant discipline, or a study being difficult to re-run now because of changes in social expectations. }
#'   \item{RW15 Effect size}{numeric; any references to the effect size that indicate that the participant considered the size of the effect when assessing the claim. Don’t use if the term "effect size" is used in unrelated ways, but err on the side of considering statements as relevant to the participant’s assessment.}
#'   \item{RW16 Interaction effect}{numeric; references to when the effect was an interaction effect (rather than a direct effect).}
#'   \item{RW17 Interval or range measure for statistical uncertainty (CI, SD, etc )}{numeric; references to the inclusion, absence, or size of the uncertainty measure for a given effect.}
#'   \item{RW18 Outside participants areas of expertise}{numeric; references to the claim under assessment being outside the participant's areas of expertise.}
#'   \item{RW20 Plausibility}{numeric; references to the plausibility of the claim.}
#'   \item{RW21 Population or subject characteristics (sampling practices)}{numeric; references to the characteristics of the sample population or subjects used in a study that affect the participant’s assessment of the claim, including references to low response rate and any other questions or appreciation of the sampling practices.}
#'   \item{RW22 Power adequacy and or sample size}{numeric; combines 2 nodes for references to the adequacy (or not) of the statistical power of the study &/or sample size.}
#'   \item{RW32 Reputation}{numeric; references to the reputation of the journal/institute/author.}
#'   \item{RW37 Revision statements}{numeric; .}
#'   \item{RW42 Significance, statistical (p-value etc )}{numeric; references to a test of statistical significance for the claim as it impacts on the participant’s assessment. This explicitly includes p-values, t-values, critical alpha and p-rep.}
#' }
"data_supp_reasons"

#' Replication outcomes for the papers
#'
#' @format a tibble with 25 rows and 2 columns
#' \describe{
#'   \item{paper_id}{character string for the paper ID}
#'   \item{outcome}{numeric value of replication outcome. 1 = replication success, 0 = replication failure}
#' }
"data_outcomes"

#' Confidence Scores generated for 25 papers with 22 aggregation methods
#'
#' @format a tibble with 550 rows and 5 columns
#' \describe{
#'   \item{method}{character string of method name}
#'   \item{paper_id}{character string of paper IDs}
#'   \item{cs}{numeric of generated confidence scores}
#'   \item{n_experts}{numeric of the number of expert judgements aggregated in confidence score}
#' }
"data_confidence_scores"

utils::globalVariables(c(
  "element",
  "value",
  "three_point_upper",
  "three_point_lower",
  "three_point_best",
  "user_name",
  "agg_var",
  "paper_id",
  "AUC",
  "Brier_Score",
  "agg_sum",
  "agg_weight",
  "aggregate_diff",
  "aggregated_judgement",
  "asym",
  "avdist",
  "avdist_preimage",
  "cl",
  "claim_count",
  "cs",
  "d",
  "d_max",
  "data",
  "diff_best",
  "diff_dist",
  "diff_lower",
  "diff_upper",
  "diff_width",
  "dist_1",
  "dist_2",
  "element",
  "gran_binary",
  "indiv",
  "indiv_weight",
  "int_agg",
  "int_weight",
  "log_odds",
  "max_agg",
  "mean_judgement",
  "median_best",
  "method",
  "n",
  "n_experts",
  "no_reason_scores_for_claim",
  "no_weights_for_claim",
  "outcome",
  "paper_id",
  "probit",
  "reason_count",
  "replicated_outcome",
  "rgamma",
  "rnorm",
  "round_1",
  "round_2",
  "sd",
  "stat",
  "three_point_best",
  "three_point_lower",
  "three_point_upper",
  "timestamp",
  "ub",
  "ul",
  "user_name",
  "vals",
  "value",
  "var.names",
  "var.names2",
  "weight",
  "weight_obs",
  "width",
  "x",
  "."
))

#' @title
#' Aggregation Method: BayesianWAgg
#'
#' @description
#' Bayesian aggregation methods with either uninformative or informative prior distributions.
#'
#' **JAGS Install**
#'
#' For instructions on installing JAGS onto your system visit \url{https://gist.github.com/dennisprangle/e26923fae7477566510757ab3341f54c}
#'
#' @details
#'
#' `type` may be one of the following:
#'
#' **BayTriVar**: The Bayesian Triple-Variability Method, fit with JAGS.
#'
#'
#' \loadmathjax
#' Three kinds of variability around best estimates are considered:
#' 1. generic claim variability: variation across individuals within a claim
#' 2. generic participant variability: variation within an individual across claims
#' 3. claim - participant specific uncertainty (operationalised by bounds): informed
#' by interval widths given by individual \mjeqn{i}{ascii} for claim \mjeqn{c}{ascii}.
#'
#' The model takes the log odds transformed individual best estimates as input (data),
#' uses a normal likelihood function and derives a posterior distribution for the
#' probability of replication.
#'
#' \mjdeqn{log( \frac{B_{i,c}}{1-B_{i,c}}) \sim N(\mu_c, \sigma_{i,c}),}{ascii}
#'
#' where \mjeqn{\mu_c}{ascii} denotes the mean estimated probability of replication for claim
#' \mjeqn{c}{ascii}, and  \mjeqn{\sigma_{i,c}}{ascii} denotes the standard deviation of the
#' estimated probability of replication for claim \mjeqn{c}{ascii} and individual
#' \mjeqn{i}{ascii} (on the logit scale).  Parameter \mjeqn{\sigma_{i,c}}{ascii} is calculated as:
#' \mjdeqn{\sigma_{i,c} = (U_{i,c} - L_{i,c} + 0.01) \times \sqrt{\sigma_i^2+\sigma_c^2}}{ascii}
#' with \mjeqn{\sigma_i}{ascii} denoting the standard deviation of estimated probabilities of
#' replication for individual \mjeqn{i}{ascii} and  \mjeqn{\sigma_c}{ascii} denoting the standard
#' deviation of the estimated probability of replication for claim \mjeqn{c}{ascii}.
#'
#' The uninformative priors for specifying this Bayesian model are
#' \mjeqn{\mu_c \sim N(0,\ 3)}{ascii}, \mjeqn{\sigma_i \sim U(0,\ 10)}{ascii} and
#' \mjeqn{\sigma_c \sim U(0,\ 10)}{ascii}. After obtaining the median of the posterior
#' distribution of \mjeqn{\mu_c}{ascii}, we can back transform to obtain
#' \mjeqn{\hat{p}_c}{ascii}:
#'
#' \mjdeqn{\hat{p}_c\left( BayTriVar \right) = \frac{e^{\mu_c}}{1+e^{\mu_c}}}{ascii}
#'
#' **BayPRIORsAgg**: Priors derived from predictive models, updated with best estimates.
#'
#' This method uses Bayesian updating to update a prior probability of
#' replication estimated from a predictive model with an aggregate of the individuals’ best
#' estimates for any given claim. Methodology is the same as `type` `"BayTriVar"` except an
#' informative prior is used for \mjeqn{\mu_c}{ascii}. Conceptually the parameters of the prior
#' distribution of \mjeqn{\mu_c}{ascii} are informed by the PRIORS model (Gould et al. 2021)
#' which is a multilevel logistic regression model that predicts the probability of
#' replication using attributes of the original study. However, any model providing predictions of
#' the probability of replication can be used to generate the required priors.
#'
#' # Warning
#'
#' Both `BayTriVar` and `BayPRIORsAgg` methods require a minimum of two claims for which judgements are supplied to `expert_judgements`. This is due to the mathematical definition of these aggregators: `BayesianWAgg` calculates the variance in best estimates across multiple claims as well as the variance in best estimates across claims per individual. Thus when only one claim is provided in `expert_judgements`, the variance is 0, hence more than one claim is required for the successful execution of both Bayesian methods.
#'
#' @param expert_judgements A dataframe in the format of [data_ratings].
#' @param type One of `"BayTriVar"`, or `"BayPRIORsAgg"`.
#' @param priors (Optional) A dataframe of priors in the format of [data_supp_priors], required for `type` `BayPRIORsAgg`.
#' @param name Name for aggregation method. Defaults to `type` unless specified.
#' @param placeholder Toggle the output of the aggregation method to impute placeholder data.
#' @param percent_toggle Change the values to probabilities. Default is `FALSE`.
#' @param round_2_filter Note that the IDEA protocol results in both a Round 1
#' and Round 2 set of probabilities for each claim. Unless otherwise specified,
#' we will assume that the final Round 2 responses (after discussion) are being
#' referred to.
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @examples
#' \dontrun{BayesianWAgg(data_ratings, percent_toggle = TRUE)}
#'
#' @export
#' @md

BayesianWAgg <- function(
  expert_judgements,
  type = "BayTriVar",
  priors = NULL,
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
) {
  if (!(type %in% c("BayTriVar", "BayPRIORsAgg"))) {
    stop('`type` must be one of "BayTriVar" or "BayPRIORsAgg"')
  } else if (Sys.which("jags") == "") {
    cli::cli_abort(
      "JAGS is require for {.emph {type}}:
                 {.url https://gist.github.com/dennisprangle/e26923fae7477566510757ab3341f54c}"
    )
  }

  ## Check for n
  if (dplyr::n_distinct(expert_judgements$paper_id) < 2) {
    cli::cli_abort("Model requires n > 1 ids to successfully execute.")
  }

  ## Set name argument

  name <- ifelse(is.null(name), type, name)

  cli::cli_h1(sprintf("BayesianWAgg: %s", name))

  if (isTRUE(placeholder)) {
    method_placeholder(expert_judgements, name)
  } else {
    df <- expert_judgements %>%
      preprocess_judgements(
        percent_toggle = {{ percent_toggle }},
        round_2_filter = {{ round_2_filter }},
        three_point_filter = TRUE
      ) %>%
      dplyr::group_by(paper_id, user_name) %>%
      tidyr::pivot_wider(names_from = element)

    ## Standard code

    user <- df %>%
      dplyr::ungroup() %>%
      dplyr::select(user_name) %>%
      dplyr::distinct() %>%
      dplyr::mutate(user = 1:dplyr::n())

    df <- df %>%
      dplyr::left_join(user) %>%
      dplyr::select(-user_name)

    paper_id <- dplyr::tibble(paper_id = unique(df$paper_id)) #store effectID's for after model fitting

    ## Create claim * participant matrices of best/lower/upper

    Best <- df %>%
      dplyr::ungroup() %>%
      dplyr::select(user_name, paper_id, three_point_best) %>%
      tidyr::pivot_wider(
        names_from = user_name,
        values_from = three_point_best
      ) %>%
      dplyr::select(-paper_id) %>%
      as.matrix()

    Lower <- df %>%
      dplyr::ungroup() %>%
      dplyr::select(user_name, paper_id, three_point_lower) %>%
      tidyr::pivot_wider(
        names_from = user_name,
        values_from = three_point_lower
      ) %>%
      dplyr::select(-paper_id) %>%
      as.matrix()

    Upper <- df %>%
      dplyr::ungroup() %>%
      dplyr::select(user_name, paper_id, three_point_upper) %>%
      tidyr::pivot_wider(
        names_from = user_name,
        values_from = three_point_upper
      ) %>%
      dplyr::select(-paper_id) %>%
      as.matrix()

    ## Create a matrix with an index of non-missing values

    index_matrix <- Best >= 0

    n_claim <- nrow(Best)

    n_participant <- ncol(Best)

    ## Calculate claim and participant sd in advance

    claim_sd <- apply(Best, 1, sd, na.rm = TRUE)

    claim_sd <- ifelse(claim_sd %in% c(0, NA), .Machine$double.eps, claim_sd)

    participant_sd <- apply(Best, 2, sd, na.rm = TRUE)

    participant_sd <- ifelse(
      participant_sd %in% c(0, NA),
      .Machine$double.eps,
      participant_sd
    )

    ## Create sd matrices

    claim_sd_matrix <- matrix(claim_sd, n_claim, n_participant)

    claim_sd_matrix <- claim_sd_matrix * index_matrix

    participant_sd_matrix <- matrix(
      participant_sd,
      n_claim,
      n_participant,
      byrow = TRUE
    )

    participant_sd_matrix <- participant_sd_matrix * index_matrix

    ## Calculate number of participants per claim

    p_count <- df %>%
      dplyr::group_by(paper_id) %>%
      dplyr::summarise(p_count = dplyr::n()) %>%
      dplyr::arrange(match(paper_id, unique(df$paper_id))) %>%
      dplyr::pull(p_count)

    ## Build  matrices of generic participant * claim

    Generic_Best <- matrix(NA, n_claim, max(p_count))

    Generic_Lower <- Generic_Upper <- Generic_Claim_sd <- Generic_Participant_sd <- Generic_Best

    for (i in seq_len(nrow(Generic_Best))) {
      val <- Best[i, ][!is.na(Best[i, ])]

      val <- c(val, rep(NA, max(p_count) - p_count[i]))

      Generic_Best[i, ] <- val
    }

    for (i in seq_len(nrow(Generic_Lower))) {
      val <- Lower[i, ][!is.na(Lower[i, ])]

      val <- c(val, rep(NA, max(p_count) - p_count[i]))

      Generic_Lower[i, ] <- val
    }

    for (i in seq_len(nrow(Generic_Upper))) {
      val <- Upper[i, ][!is.na(Upper[i, ])]

      val <- c(val, rep(NA, max(p_count) - p_count[i]))

      Generic_Upper[i, ] <- val
    }

    for (i in seq_len(nrow(Generic_Claim_sd))) {
      val <- claim_sd_matrix[i, ][!is.na(claim_sd_matrix[i, ])]

      val <- c(val, rep(NA, max(p_count) - p_count[i]))

      Generic_Claim_sd[i, ] <- val
    }

    for (i in seq_len(nrow(Generic_Participant_sd))) {
      val <- participant_sd_matrix[i, ][!is.na(participant_sd_matrix[i, ])]

      val <- c(val, rep(NA, max(p_count) - p_count[i]))

      Generic_Participant_sd[i, ] <- val
    }

    ## Logit transform Best estimate

    Logit_Generic_Best <- log(.99 * Generic_Best / (1 - .99 * Generic_Best))

    switch(
      type,
      "BayTriVar" = {
        ## Create list of JAGS inputs

        JAGS_list <- list(
          Logit_Best = Logit_Generic_Best,
          Lower = Generic_Lower,
          Upper = Generic_Upper,
          Claim_SD = Generic_Claim_sd,
          Participant_SD = Generic_Participant_sd,
          n_claim = n_claim,
          n_assessments = p_count
        )

        ## Define Jags Model

        BayesModel <- "model{
  ## PRIORS

  ### Prior for claim location

  for (i in 1:n_claim)
  {

   mu[i] ~ dnorm(0,3) # claim location, weakly informative prior

  }

  ## DATA

  ### For each claim

  for (i in 1:n_claim) # For each claim
  {

    ### For each participant, up to the number of participants on that claim.
    ### No more problems with missing values due to varying number of assessors.

    for (j in 1:n_assessments[i])
    {

      Margin[i,j] <- (Upper[i,j] - Lower[i,j]) + .01

      sigma[i,j] <-  Margin[i,j] * sqrt(pow(Claim_SD[i,j], 2) + pow(Participant_SD[i,j], 2))

      lambda[i,j] <- pow(sigma[i,j],-2)

      Logit_Best[i,j] ~  dnorm(mu[i],lambda[i,j])

    }
  }
}"
      },
      "BayPRIORsAgg" = {
        ## adding prior values to the paper_ids

        prior_means <- paper_id %>%
          dplyr::left_join(priors, by = "paper_id")

        ## Create list of JAGS inputs

        JAGS_list <- list(
          Logit_Best = Logit_Generic_Best,
          Lower = Generic_Lower,
          Upper = Generic_Upper,
          Claim_SD = Generic_Claim_sd,
          Participant_SD = Generic_Participant_sd,
          Prior_means = prior_means$prior_means,
          n_claim = n_claim,
          n_assessments = p_count
        )

        ## Define Jags Model

        BayesModel <- "model{
  ## PRIORS

  ### Prior for claim location

  for (i in 1:n_claim)
  {

   mu[i] ~ dnorm(Prior_means[i],3) # claim location, weakly informative prior

  }

  ## DATA

  ### For each claim

  for (i in 1:n_claim) # For each claim
  {

    ### For each participant, up to the number of participants on that claim.
    ### No more problems with missing values due to varying number of assessors.

    for (j in 1:n_assessments[i])
    {

      Margin[i,j] <- (Upper[i,j] - Lower[i,j]) + .01

      sigma[i,j] <-  Margin[i,j] * sqrt(pow(Claim_SD[i,j], 2) + pow(Participant_SD[i,j], 2))

      lambda[i,j] <- pow(sigma[i,j],-2)

      Logit_Best[i,j] ~  dnorm(mu[i],lambda[i,j])

    }
  }
}"
      }
    )

    # Fit Model

    JAGSparams <- c("mu", "sigma")

    JAGSinits <- function() {
      list("mu" = rnorm(1), "sigma" = rgamma(1, .1, .1))
    }

    CSsamples <- R2jags::jags(
      progress.bar = "none",
      quiet = TRUE,
      data = JAGS_list,
      parameters.to.save = JAGSparams,
      n.thin = 1,
      n.iter = 10000,
      n.burnin = 5000,
      n.chains = 3,
      model.file = textConnection(BayesModel)
    )

    CSmcmc <- coda::as.mcmc(CSsamples)

    CSmu <- matrix(, JAGS_list$n_claim, 5000) # 5000

    for (i in seq_len(JAGS_list$n_claim)) {
      CSmu[i, ] <- CSmcmc[, sprintf("mu[%s]", i)][[1]]
    }

    # Extract JAGS model output
    # Values in table

    Out <- CSsamples$BUGSoutput$summary
    Pars <- Out[grep("mu", rownames(Out)), ]
    ProbPars <- exp(Pars) / (1 + exp(Pars))
    row.names(ProbPars) <- paper_id$paper_id #get the effectID's

    aggregation_analysis_output <-
      ProbPars %>%
      tibble::as_tibble() %>%
      dplyr::select(mean) %>%
      dplyr::mutate(method = name, paper_id = row.names(ProbPars)) %>%
      dplyr::select(-mean, dplyr::everything(), mean) %>%
      dplyr::rename(aggregated_judgement = mean)

    df %>%
      dplyr::group_by(paper_id) %>%
      dplyr::summarise(
        n_experts = dplyr::n()
      ) %>%
      dplyr::right_join(aggregation_analysis_output) %>%
      postprocess_judgements()
  }
}

#' Confidence Score Heat Map
#'
#' Confidence scores displayed on a colour spectrum across generated methods and
#' assessed claims, split into predicted replication outcomes (TRUE/FALSE).
#'
#' @param confidence_scores A data frame of confidence scores generated from the aggregation methods in the form of [data_confidence_scores].
#' Defaults to [data_confidence_scores] if no argument supplied.
#' @param data_outcomes A data frame of unique claims and the associated binary outcome in the form of [data_outcomes]. If no argument supplied
#' then defaults to [data_outcomes] supplied within package.
#'
#' @importFrom insight format_capitalize
#'
#' @return Plot in viewer
#'
#' @examples
#' \donttest{confidence_score_heatmap(data_confidence_scores, data_outcomes)}
#'
#' @export

confidence_score_heatmap <- function(
  confidence_scores = NULL,
  data_outcomes = NULL
) {
  # Defaults to example dataset if none provided
  if (is.null(confidence_scores)) {
    cli::cli_alert_info("No Confidence Scores Provided. Using Package Default")
    confidence_scores <- aggreCAT::data_confidence_scores
  } else {
    confidence_scores <- confidence_scores
  }

  if (is.null(data_outcomes)) {
    cli::cli_alert_info("No Outcome Scores Provided. Using Package Default")
    data_outcomes <- aggreCAT::data_outcomes
  } else {
    data_outcomes <- data_outcomes
  }

  # object for checking the number of claims
  number_of_claims <- dplyr::n_distinct(confidence_scores$paper_id)

  # object to throw a display warning
  number_of_methods <- dplyr::n_distinct(confidence_scores$method)

  if (number_of_claims <= 10) {
    cli::cli_alert_warning(
      "Claims less than ~ 10 might not display effectively."
    )
  }

  data_cs_with_outcomes <- confidence_scores %>%
    dplyr::left_join(data_outcomes, by = "paper_id") %>%
    dplyr::mutate(
      replicated_outcome = ifelse(outcome == 1, "TRUE", "FALSE")
    ) %>%
    dplyr::select(paper_id, method, cs, outcome, replicated_outcome)

  # Generate accuracy scores
  evaluated_outcomes <- aggreCAT::confidence_score_evaluation(confidence_scores = confidence_scores,
                                                              outcomes = data_outcomes)
  
  # Plot

  evaluated_outcomes %>%
    dplyr::left_join(data_cs_with_outcomes) %>%
    # dplyr::filter(replicated_outcome == "FALSE") %>%
    dplyr::mutate(
      AUC = round(AUC, digits = 3),
      Brier_Score = round(Brier_Score, digits = 2),
      paper_id = forcats::fct_reorder(paper_id, -cs),
      replicated_outcome = paste0("Outcome: ", replicated_outcome),
      replicated_outcome = factor(
        replicated_outcome,
        levels = c("Outcome: TRUE", "Outcome: FALSE")
      )
    ) %>%
    ggplot2::ggplot() +
    ggplot2::aes(x = paper_id, 
                 y = factor(method, 
                            levels = rev(levels(factor(method)))),
                 fill = cs,
                 colour = cs) +
    ggplot2::geom_tile(linetype = 1, 
                       linejoin = "mitre") +
    ggplot2::scale_fill_distiller(palette = "RdYlBu", 
                                  direction = 1, 
                                  breaks = seq(0,1,0.2)) +
    ggplot2::scale_color_distiller(palette = "RdYlBu", 
                                   direction = 1, 
                                   breaks = seq(0,1,0.2)) +
    ggplot2::labs(x = "Paper ID",
                  y = "Aggregation Method",
                  title = "",
                  fill = "Confidence \nScore", color = "Confidence \nScore") +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90,
                                                       hjust = 1,
                                                       vjust = 0.5)) +
    ggplot2::facet_wrap(~replicated_outcome,
                        scales = "free_x")
  
}

#' @title
#' Weighting method: Down weighting outliers
#'
#' @description
#' This method down-weights outliers.
#'
#' @details
#' This function is used by [LinearWAgg] to weighting functions for the aggregation type
#' `"OutWAgg"`. Outliers are given less weight by using the squared difference between the
#' median of an individual's best estimates across all claims and their best estimate
#' for the claim being assessed:
#' \loadmathjax
#' \mjdeqn{d_{i,c} = \left(median\{{B_{i,c}}_{_{i=1,...,N}}\} - B_{i,c}\right)^2}{ascii}
#'
#' Weights are given by 1 minus the proportion of the individual's squared difference
#' relative to the maximum squared difference for the claim across all individuals:
#'
#' \mjdeqn{w\_out_{i} = 1 - \frac{d_{i,c}}{\max({d_c})})}{ascii}
#'
#' @param expert_judgements A dataframe in the form of [data_ratings]
#'
#' @importFrom stats median
#'
#' @return A tibble in the form of the input `expert_judgements` argument with additional columns
#' supplying the calculated weight for each row's observation.
#'
#' @family weighting functions
#'
#' @export

weight_outlier <- function(expert_judgements) {
  expert_judgements_pp <- expert_judgements %>%
    dplyr::filter(element == "three_point_best")

  expert_judgements_pp_d <- expert_judgements_pp %>%
    # get central tendency of best estimates for claim
    dplyr::group_by(paper_id) %>%
    dplyr::summarise(median_best = median(value, na.rm = TRUE)) %>%
    dplyr::left_join(expert_judgements_pp, by = "paper_id") %>%
    # calculate difference for individual best estimates
    dplyr::mutate(d = (median_best - value)^2)

  expert_judgements_pp_d %>%
    dplyr::group_by(paper_id) %>%
    # get max difference
    dplyr::summarise(d_max = max(d, na.rm = TRUE)) %>%

    # adjust max value if 0
    dplyr::mutate(
      d_max = dplyr::if_else(
        d_max == 0,
        .Machine$double.eps,
        d_max
      )
    ) %>%
    dplyr::left_join(expert_judgements_pp_d, by = "paper_id") %>%
    # calculate weight
    dplyr::mutate(agg_weight = 1 - (d / d_max)) %>%
    dplyr::select(-d, -d_max, -median_best)
}

#' Placeholder function with TA2 output
#'
#' This functions stands in for when we haven't completed coding the method.
#'
#' This function expects input from [preprocess_judgements] and outputs
#' for [postprocess_judgements].
#'
#' @param expert_judgements A data frame in the form of ratings
#' @param method_name Aggregation method to place into placeholder mode
#'
#' @examples
#' \dontrun{method_placeholder(data_ratings, method_name = "TestMethod")}
#'
#' @return A tibble of confidence scores `cs` for each `paper_id`.
#'
#' @export

method_placeholder <- function(expert_judgements, method_name) {
  placeholder = TRUE

  cli::cli_alert_info("Placeholder: {.val {placeholder}}")

  expert_judgements %>%
    preprocess_judgements() %>%
    dplyr::filter(element == "three_point_best") %>%
    dplyr::group_by(paper_id) %>%
    dplyr::summarise(cs = 0.65, n_experts = 0) %>%
    dplyr::mutate(method = {{ method_name }}) %>%
    dplyr::select(method, paper_id, cs, n_experts)
}

#' @title
#' Weighting method: Width of intervals
#'
#' @description
#' Calculates weights by interval width
#'
#' @details
#' This function is used inside [IntervalWAgg] for aggregation type `"IntWAgg"`. It
#' calculates the width of each three-point judgement (upper - lower), then returns
#' the weight as the inverse of this interval.
#'
#' @param expert_judgements A dataframe in the form of [data_ratings]
#'
#' @return A tibble in the form of the input `expert_judgements` argument with additional columns
#' supplying the calculated weight for each row's observation.
#'
#' @family weighting functions
#'
#' @export

weight_interval <- function(expert_judgements) {
  # calculate obs-level weight
  expert_judgements %>%
    tidyr::pivot_wider(names_from = element, values_from = value) %>%
    dplyr::mutate(
      ub = three_point_upper - three_point_lower,
      ub = dplyr::if_else(ub == 0, .Machine$double.eps, ub),
      agg_weight = 1 / ub,
    ) %>%
    dplyr::rename(value = three_point_best)
}

# Place holder Unit Tests Function -----------------------------------------

placeholder_test <- function(method_to_test, type) {
  tbl <- data_ratings %>%
    method_to_test(placeholder = TRUE, type = {{ type }})

  test_that("column `method` exists", {
    pointblank::expect_col_exists(
      tbl,
      columns = c("method"),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    pointblank::expect_col_exists(
      tbl,
      columns = c("paper_id"),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    pointblank::expect_col_exists(
      tbl,
      columns = c("cs"),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    pointblank::expect_col_exists(
      tbl,
      columns = c("n_experts"),
      threshold = 1
    )
  })

  test_that("values in `cs` should be == `0.65`", {
    pointblank::expect_col_vals_equal(
      tbl,
      columns = c("cs"),
      value = 0.65,
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be == `0`", {
    pointblank::expect_col_vals_equal(
      tbl,
      columns = c("n_experts"),
      value = 0,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    pointblank::expect_col_is_character(
      tbl,
      columns = c("method"),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    pointblank::expect_col_is_character(
      tbl,
      columns = c("paper_id"),
      threshold = 1
    )
  })
}


# Place holder Tests -------------------------------------------------------

placeholder_test(AverageWAgg, type = "ArMean")
placeholder_test(AverageWAgg, type = "Median")
placeholder_test(AverageWAgg, type = "GeoMean")
placeholder_test(AverageWAgg, type = "LOArMean")
placeholder_test(AverageWAgg, type = "ProbitArMean")
placeholder_test(LinearWAgg, type = "DistLimitWAgg")
placeholder_test(LinearWAgg, type = "GranWAgg")
placeholder_test(LinearWAgg, type = "OutWAgg")
placeholder_test(IntervalWAgg, type = "IntWAgg")
placeholder_test(IntervalWAgg, type = "IndIntWAgg")
placeholder_test(IntervalWAgg, type = "AsymWAgg")
placeholder_test(IntervalWAgg, type = "IndIntAsymWAgg")
placeholder_test(IntervalWAgg, type = "VarIndIntWAgg")
placeholder_test(IntervalWAgg, type = "KitchSinkWAgg")
placeholder_test(DistributionWAgg, type = "DistribArMean")
placeholder_test(DistributionWAgg, type = "TriDistribArMean")
placeholder_test(ShiftingWAgg, type = "ShiftWAgg")
placeholder_test(ShiftingWAgg, type = "BestShiftWAgg")
placeholder_test(ShiftingWAgg, type = "IntShiftWAgg")
placeholder_test(ShiftingWAgg, type = "DistShiftWAgg")
placeholder_test(ShiftingWAgg, type = "DistIntShiftWAgg")
placeholder_test(ExtremisationWAgg, type = "BetaArMean")

placeholder_test(LinearWAgg, type = "Participant_LO")
placeholder_test(LinearWAgg, type = "Participant")
placeholder_test(LinearWAgg, type = "Judgement")

placeholder_test(ReasoningWAgg, type = "ReasonWAgg")
placeholder_test(ReasoningWAgg, type = "ReasonWAgg2")

# placeholder_test(BayesianWAgg, type = "BayTriVar")
# placeholder_test(BayesianWAgg, type = "BayPRIORsAgg")

placeholder_test(ExtremisationWAgg, type = "BetaArMean")
placeholder_test(ExtremisationWAgg, type = "BetaArMean2")

library(pointblank)


method_schema_default <- function(method_to_test, type) {
  tbl <- data_ratings %>%
    method_to_test(type = {{ type }}, percent_toggle = TRUE)

  test_that("column `method` exists", {
    expect_col_exists(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    expect_col_exists(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    expect_col_exists(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    expect_col_exists(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be between `0` and `25`", {
    expect_col_vals_between(
      tbl,
      columns = vars(n_experts),
      left = 0,
      right = 25,
      threshold = 1
    )
  })

  test_that("values in `cs` should be between `0` and `1`", {
    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `n_experts` is of type: integer", {
    expect_col_is_integer(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("all values in `method` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("all values in `paper_id` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })
}

# Measures of unweighted aggregation
method_schema_default(AverageWAgg, type = "ArMean")
method_schema_default(AverageWAgg, type = "Median")
method_schema_default(AverageWAgg, type = "GeoMean")
method_schema_default(AverageWAgg, type = "LOArMean")
method_schema_default(AverageWAgg, type = "ProbitArMean")

method_schema_default(LinearWAgg, type = "DistLimitWAgg")
method_schema_default(LinearWAgg, type = "GranWAgg")
method_schema_default(LinearWAgg, type = "OutWAgg")

method_schema_default(IntervalWAgg, type = "IntWAgg")
method_schema_default(IntervalWAgg, type = "IndIntWAgg")
method_schema_default(IntervalWAgg, type = "AsymWAgg")
method_schema_default(IntervalWAgg, type = "IndIntAsymWAgg")
method_schema_default(IntervalWAgg, type = "VarIndIntWAgg")
method_schema_default(IntervalWAgg, type = "KitchSinkWAgg")

method_schema_default(DistributionWAgg, type = "DistribArMean")
method_schema_default(DistributionWAgg, type = "TriDistribArMean")

method_schema_default(ShiftingWAgg, type = "ShiftWAgg")
method_schema_default(ShiftingWAgg, type = "BestShiftWAgg")
method_schema_default(ShiftingWAgg, type = "IntShiftWAgg")
method_schema_default(ShiftingWAgg, type = "DistShiftWAgg")
method_schema_default(ShiftingWAgg, type = "DistIntShiftWAgg")


# Method Test Needing Additional Arguments --------------------------------

method_schema_linear_weights <- function(method_to_test, weights, type, name) {
  tbl <- data_ratings %>%
    method_to_test(
      weights,
      type = {{ type }},
      name = {{ name }},
      percent_toggle = TRUE
    )

  test_that("column `method` exists", {
    expect_col_exists(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    expect_col_exists(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    expect_col_exists(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    expect_col_exists(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be between `0` and `25`", {
    expect_col_vals_between(
      tbl,
      columns = vars(n_experts),
      left = 0,
      right = 25,
      threshold = 1
    )
  })

  test_that("values in `cs` should be between `0` and `1`", {
    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `n_experts` is of type: integer", {
    expect_col_is_integer(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("all values in `method` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("all values in `paper_id` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })
}


# LinearWAgg Additional Arguments -----------------------------------------

method_schema_linear_weights(
  LinearWAgg,
  weights = data_supp_quiz %>%
    dplyr::rename(weight = quiz_score),
  type = "Participant_LO",
  name = "QuizWAgg"
)
#
#
# method_schema_linear_weights(LinearWAgg,
#                              weights = data_ratings %>%
#                                dplyr::filter(element == "three_point_best",
#                                              round == "round_2") %>%
#                                dplyr::select(paper_id,
#                                              user_name,
#                                              timestamp) %>%
#                                dplyr::group_by(user_name) %>%
#                                dplyr::arrange(timestamp) %>%
#                                dplyr::mutate(claim_count = dplyr::row_number()) %>%
#                                dplyr::ungroup() %>%
#                                dplyr::mutate(weight = log(claim_count) + 1) %>%
#                                dplyr::select(paper_id,
#                                              user_name,
#                                              weight),
#                              type = "Judgement",
#                              name = "ExperienceWAgg")

method_schema_linear_weights(
  LinearWAgg,
  weights = data_ratings %>%
    dplyr::filter(question == "comprehension", round == "round_2") %>%
    dplyr::select(paper_id, user_name, value) %>%
    dplyr::rename(weight = value),
  type = "Judgement",
  name = "CompWAgg"
)

method_schema_linear_weights(
  LinearWAgg,
  weights = data_justifications %>%
    dplyr::mutate(
      n_words = stringr::str_count(justification, pattern = " ") + 1
    ) %>%
    dplyr::group_by(user_name, paper_id) %>%
    dplyr::summarise(word_count = sum(n_words, na.rm = TRUE)) %>%
    dplyr::select(paper_id, user_name, word_count) %>%
    dplyr::rename(weight = word_count),
  type = "Judgement",
  name = "EngWAgg"
)


# Reasoning ---------------------------------------------------------------

method_schema_reasoning <- function(
  method_to_test,
  reasons,
  type,
  name,
  beta_transform
) {
  tbl <- data_ratings %>%
    method_to_test(
      reasons,
      type = {{ type }},
      name = {{ name }},
      beta_transform = {{ beta_transform }},
      percent_toggle = TRUE
    )

  test_that("column `method` exists", {
    expect_col_exists(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    expect_col_exists(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    expect_col_exists(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    expect_col_exists(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be between `0` and `25`", {
    expect_col_vals_between(
      tbl,
      columns = vars(n_experts),
      left = 0,
      right = 25,
      threshold = 1
    )
  })

  test_that("values in `cs` should be between `0` and `1`", {
    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `n_experts` is of type: integer", {
    expect_col_is_integer(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("all values in `method` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("all values in `paper_id` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })
}


method_schema_reasoning(
  ReasoningWAgg,
  reasons = data_supp_reasons,
  type = "ReasonWAgg",
  name = "ReasonWAgg",
  beta_transform = FALSE
)

method_schema_reasoning(
  ReasoningWAgg,
  reasons = data_supp_reasons,
  type = "ReasonWAgg2",
  name = "ReasonWAgg2",
  beta_transform = FALSE
)

method_schema_reasoning(
  ReasoningWAgg,
  reasons = data_supp_reasons,
  type = "ReasonWAgg",
  name = "BetaReasonWAgg",
  beta_transform = TRUE
)

method_schema_reasoning(
  ReasoningWAgg,
  reasons = data_supp_reasons,
  type = "ReasonWAgg2",
  name = "BetaReasonWAgg2",
  beta_transform = TRUE
)


# Bayes --------------------------------------------------------------------

method_schema_bays <- function(method_to_test, priors, type, name) {
  tbl <- data_ratings %>%
    method_to_test(
      priors,
      type = {{ type }},
      name = {{ name }},
      percent_toggle = TRUE
    )

  test_that("column `method` exists", {
    expect_col_exists(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    expect_col_exists(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    expect_col_exists(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    expect_col_exists(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be between `0` and `25`", {
    expect_col_vals_between(
      tbl,
      columns = vars(n_experts),
      left = 0,
      right = 25,
      threshold = 1
    )
  })

  test_that("values in `cs` should be between `0` and `1`", {
    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `n_experts` is of type: integer", {
    expect_col_is_integer(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("all values in `method` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("all values in `paper_id` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })
}

# method_schema_bays(BayesianWAgg, priors = NULL, type = "BayTriVar", name = "BayTriVar")

# method_schema_bays(BayesianWAgg, priors = data_supp_priors, type = "BayPRIORsAgg", name = "BayTriVar")

# Extreme Wag -------------------------------------------------------------

method_schema_extreme <- function(
  method_to_test,
  type,
  name,
  cutoff_lower,
  cutoff_upper
) {
  tbl <- data_ratings %>%
    method_to_test(
      type = {{ type }},
      name = {{ name }},
      cutoff_lower = {{ cutoff_lower }},
      cutoff_upper = {{ cutoff_upper }},
      percent_toggle = TRUE
    )

  test_that("column `method` exists", {
    expect_col_exists(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` exists", {
    expect_col_exists(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `cs` exists", {
    expect_col_exists(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("column `n_experts` exists", {
    expect_col_exists(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("values in `n_experts` should be between `0` and `25`", {
    expect_col_vals_between(
      tbl,
      columns = vars(n_experts),
      left = 0,
      right = 25,
      threshold = 1
    )
  })

  test_that("values in `cs` should be between `0` and `1`", {
    expect_col_vals_between(
      tbl,
      columns = vars(cs),
      left = 0,
      right = 1,
      threshold = 1
    )
  })

  test_that("column `method` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("column `paper_id` is of type: character", {
    expect_col_is_character(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("column `n_experts` is of type: integer", {
    expect_col_is_integer(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })

  test_that("all values in `method` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(method),
      threshold = 1
    )
  })

  test_that("all values in `paper_id` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(paper_id),
      threshold = 1
    )
  })

  test_that("all values in `cs` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(cs),
      threshold = 1
    )
  })

  test_that("all values in `n_experts` should not be NULL", {
    expect_col_vals_not_null(
      tbl,
      columns = vars(n_experts),
      threshold = 1
    )
  })
}

method_schema_extreme(
  ExtremisationWAgg,
  type = "BetaArMean",
  name = "BetaArMean",
  cutoff_lower = NULL,
  cutoff_upper = NULL
)

method_schema_extreme(
  ExtremisationWAgg,
  type = "BetaArMean2",
  name = "BetaArMean2",
  cutoff_lower = 0.4,
  cutoff_upper = 0.6
)

library(pointblank)

cli::cli_alert_info("Testing Data Assets")

tbl <- data_ratings

test_that("column `round` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("round"),
    threshold = 1
  )
})

test_that("column `paper_id` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("paper_id"),
    threshold = 1
  )
})

test_that("column `user_name` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("user_name"),
    threshold = 1
  )
})

test_that("column `question` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("question"),
    threshold = 1
  )
})

test_that("column `element` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("element"),
    threshold = 1
  )
})

test_that("column `value` exists", {
  pointblank::expect_col_exists(
    tbl,
    columns = c("value"),
    threshold = 1
  )
})

test_that("column `round` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("round"),
    threshold = 1
  )
})

test_that("column `paper_id` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("paper_id"),
    threshold = 1
  )
})

test_that("column `user_name` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("user_name"),
    threshold = 1
  )
})

test_that("column `question` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("question"),
    threshold = 1
  )
})

test_that("column `element` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("element"),
    threshold = 1
  )
})


test_that("column `value` is of type: numeric", {
  pointblank::expect_col_is_numeric(
    tbl,
    columns = c("value"),
    threshold = 1
  )
})

test_that("values in `value` should be between `-1` and `100`", {
  pointblank::expect_col_vals_between(
    tbl,
    columns = c("value"),
    left = -1,
    right = 100,
    threshold = 1
  )
})

test_that("column `group` is of type: character", {
  pointblank::expect_col_is_character(
    tbl,
    columns = c("group"),
    threshold = 1
  )
})

---
title: "Tidy Aggregation and Required Data Inputs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tidy Aggregation and Required Data Inputs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
date-modified: last-modified
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The design philosophy of [aggreCAT]{.pkg} is principled on 'tidy' data
[@Wickham:2014vp]. Each aggregation method expects a
[data.frame]{.class} or [tibble]{.class} of judgements (`data_ratings`)
as its input, and returns a [tibble]{.class} containing the variables
`method`, `paper_id`, `cs` and `n_experts` (see @sec-AverageWAgg for
illustration of outputs); where `method` is a character vector
corresponding to the aggregation method name specified in the `type`
argument. Each aggregation is applied as a summary function
[@Wickham2017R], and therefore returns a single row or observation with
a single confidence score `cs` for each claim or `paper_id`. The number
of expert judgements summarised in the aggregated confidence score is
returned in the column `n_experts`. Because of the tidy nature of the
aggregation outputs, multiple aggregations can be applied to the same
data with the results of all aggregation methods row bound together in a
single `tibble` (See the example repliCATS workflow in @sec-workflow).

The tibble of judgements to be aggregated (`data_ratings`) requires the
columns `round`, `paper_id`, `user_name`, `question`, `element`, `value`
and `group`. Each observation in the judgement data corresponds to a
single `value` for a single `question` elicited from a single
`user_name` about a given `paper_id` in a single `round`. There are four
types of `question`s that elicited `values` correspond to. Estimates
about the event probability for a given `paper_id` correspond to
`"direct_replication"` in the `question` variable. The type of estimate
the `value` belongs to is recorded in the `element` variable, and may be
one of `"three_point_lower"`, `"three_point_best"`, or
`"three_point_upper"`.

Every aggregation function requires at least one `value` derived from
three-point elicitation (`question == "direct_replication"`) in the
dataframe supplied to the `expert_judgements` argument, however, some
methods require only the best-estimates
(`element == "three_point_best"`) for mathematical aggregation.
Similarly some aggregation methods require multiple `round`s of
judgements, while others require only a single round. Only the
aggregation method *CompWAgg* requires `value`s for the `comprehension`
question. For a summary of each aggregation method, its calling function
and data requirements and sources, see @tbl-method-summary-table.

```{r setup}
library(aggreCAT)
```

---
title: "aggreCAT datasets"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{aggreCAT datasets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: vignette_references.bib
---

```{r, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message=FALSE, warning=FALSE}
library(aggreCAT)
library(tidyverse)
```

### DARPA SCORE program and the repliCATS project

The [aggreCAT]{.pkg} package, and the mathematical aggregators therein,
were developed by [the repliCATS (Collaborative Assessment for
Trustworthy Science)
project](https://replicats.research.unimelb.edu.au/) as a part of the
[SCORE
program](https://www.darpa.mil/program/systematizing-confidence-in-open-research-and-evidence)
(Systematizing Confidence in Open Research and Evidence), funded by
DARPA (Defense Advanced Research Projects Agency) [@alipourfard2021].
The SCORE program is the largest replication project in science to date,
and aims to build automated tools that can rapidly and reliably assign
"Confidence Scores" to research claims from empirical studies in the
Social and Behavioural Sciences (SBS). Confidence Scores are
quantitative measures of the likely reproducibility or replicability of
a research claim or result, and may be used by consumers of scientific
research as a proxy measure for their credibility in the absence of
replication effort [@alipourfard2021].

Replications are time-consuming and costly [@Isager2020], and studies
have shown that replication outcomes can be reliably elicited from
researchers [@Gordon2020]. Consequently, the DARPA SCORE program
generated Confidence Scores for $> 4000$ SBS claims using expert
elicitation based on two very different strategies -- prediction markets
[@Gordon2020] and the IDEA protocol [@hemming2017], the latter of which
is used by the repliCATS project [@Fraser:2021]. A proportion of these
research claims were randomly selected for direct replication, against
which the elicited and aggregated Confidence Scores are 'ground-truthed'
or verified. The aim of the DARPA SCORE project is to aid the
development of artificial intelligence tools that can automatically
assign Confidence Scores.


# Datasets

The [aggreCAT]{.pkg} package includes the core dataset `data_ratings`
consisting of judgements elicited during a pilot experiment exploring
the performance of IDEA groups in assessing replicability of a set of
claims with "known outcomes." "Known-outcome" claims are SBS research
claims that have been subject to replication studies in previous
large-scale replication projects[^1]. Data were collected using the
repliCATS IDEA protocol at a two day workshop[^2] in the Netherlands, on
July 2019, at which 25 participants assessed the replicability of 25
unique SBS claims. In addition to the probabilistic estimates provided
for each research claim assessed, participants were also asked to rate
the claim's plausibility and comprehensibility, answer whether they were
involved in any aspect of the original study, and to provide their
reasoning in support of their quantitative estimates, which were used to
form measures of reasoning breadth and engagement [@Fraser:2021].

[^1]: Many labs 1, 2 and 3 @Klein2014, @Klein2018ManyL2, @Ebersole2016,
    the Social Sciences Replication Project @Camerer2018 and the
    Reproducibility Project Psychology @aac4716.

[^2]: See @Hanea2021 for details. The workshop was held at the annual
    meeting of the Society for the Improvement of Psychological Science
    (SIPS), [\<https://osf.io/ndzpt/\>](https://osf.io/ndzpt/){.uri}.

## Formatted Judgement Data

`data_ratings` is a *tidy* [data.frame]{.class} wherein each
*observation* (or row) corresponds to a single value in the set of
`value`s constituting a participant's complete assessment of a research
claim. Each research claim is assigned a unique `paper_id`, and each
participant has a unique (and anonymous) `user_name`. The variable
`round` denotes the round in which each `value` was elicited (`round_1`
or `round_2`). `question` denotes the type of question the `value`
pertains to; `direct_replication` for probabilistic judgements about the
replicability of the claim, `belief_binary` for participants' belief in
the plausibility of the claim, `comprehension` for participants'
comprehensibility ratings, and `involved_binary` for involvement in the
original study. An additional column `element` maintains the tidy
structure of the data, while capturing the multiple `value`s that
comprise a full assessment of the replicability (`direct_replication`)
of a claim; `three_point_best`, `three_point_lower` and
`three_point_upper` denote the best estimate and lower and upper bounds
respectively. `binary_question` describes the `element` for both the
plausibility rating (`belief_binary`) and involvement
(`involved_binary`) questions, whereas `likert_binary` is the `element`
describing a participant's `comprehension` rating. Judgements are
recorded in column `value` in the form of percentage probabilities
ranging from (0,100). The `binary_question`s corresponding to
comprehensibility and involvement consist of binary values (`1` for the
affirmative, and `-1` for the negative). Finally, values corresponding
to participants' comprehension ratings are on a `likert_binary` scale
from `1` through `7`. Note that additional columns with participant
attributes can be included in the ratings dataset if required by the
user; we include the `group` column in `data-ratings`, which describes
the group number the participant was a part of. Below we show some
example data for a single user for a single claim to illustrate this
structure of the core `data_ratings` dataset.

```{r data_ratings-sample, message=TRUE, results='hold'}
aggreCAT::data_ratings %>%
  dplyr::filter(paper_id == dplyr::first(paper_id),
                user_name == dplyr::first(user_name)) %>%
  head()

```

Not all data necessary for constructing weights on performance is
contained in `data_ratings`. Additional data collected as part of the
repliCATS IDEA protocol are contained within separate datasets to
`data_ratings`. Participants provided justifications for giving
particular judgements, and these are contained in `data_justifications`.
On the repliCATS platform users were given the option to comment on
others' justifications (`data_comments`), to vote on others' comments
(`data_comment_ratings`) and on others' justifications
(`data_justification_ratings`). Finally, [aggreCAT]{.pkg} contains three
'supplementary' datasets containing data collected externally to the
repliCATS IDEA protocol: `data_supp_quiz`, `data_supp_priors`, and
`data_supp_reasons`.

## Quiz Score Data {#sec-quiz-supplementary-data}

Prior to the workshop, participants were asked to complete an optional
quiz on statistical concepts and meta-research which we thought would
aid in reliably evaluating the replicability of research claims. Quiz
responses are contained in `data_supp_quiz` and are used to construct
performance weights for the aggregation method `QuizWAgg`, where each
participant receives a `quiz_score` if they completed the quiz, and `NA`
if they did not attempt the quiz [see @Hanea2021 for further details]. Additional methods of scoring the quiz responses are provided in `data_supp_quiz`.

```{r data_supp_quiz-sample, message=TRUE, results='hold'}
aggreCAT::data_supp_quiz
```


## Reasoning Data {#sec-reasonwagg-supplementary-data}

The `ReasonWAgg` aggregation type uses the number of unique reasons given by participants to
support a Best Estimate for a given claim $B_{i,c}$ to construct
performance weights, and is contained within `data_supp_reasons`.
Qualitative statements made by individuals during claim evaluation were
recorded on the repliCATS platform [@Pearson2021] and coded as falling
into one of 25 unique reasoning categories by the repliCATS Reasoning
team [@Wintle:2021]. Reasoning categories include plausibility of the
claim, effect size, sample size, presence of a power analysis,
transparency of reporting, and journal reporting [@Hanea2021]. Within
`data_supp_reasons`, each of the reasoning categories that passed our
inter-coder reliability threshold are distributed as columns in the
dataset whose names are prefixed with `RW`, and for each claim
`paper_id`, each participant `user_id` is assigned a logical `1` or `0`
if they included that reasoning category in support of their Best
estimate for that claim. See `ReasoningWAgg()` for details on the
`ReasonWAgg` aggregation method.

```{r data_supp_reasons-sample}
aggreCAT::data_supp_reasons %>%
  glimpse()
```


## Bayesian Prior Data {#sec-bayesian-supplementary-data}

The method `BayPRIORsAgg` (implemented in `BayesianWAgg()`) uses Bayesian updating to update a prior
probability of a claim replicating estimated from a predictive model
[@Gould2021a] using an aggregate of the best estimates for all
participants assessing a given claim $c$ [@Hanea2021]. The prior data is
contained in `data_supp_priors` with each claim in column `paper_id`
being assigned a prior probability (on the logit scale) of the claim
replicating in column `prior_means`.

```{r data_supp_priors-sample}
aggreCAT::data_supp_priors
```


# TODO

- [ ] `data_comments`
- [ ] `data_confidence_scores`
- [ ] `data_justifications`
- [ ] `data_outcomes`

# References

---
format:
  html:
    code-copy: hover
    code-link: true
    embed-resources: true #TODO turn off for submission, only for sharing output .html
  pdf:
    documentclass: scrreprt
    papersize: A4
title: "Introducing {aggreCAT}: Mathematical aggregation of expert judgments"
bibliography: ISC_blog.bib
csl: cell.csl
eval: true
date-modified: last-modified
fig-responsive: true
author:
  - name: David P. Wilkinson
    orcid: 0000-0002-9560-6499
    email: dwilkinson@unimelb.edu.au
  - name: Elliot Gould
    orcid: 0000-0002-6585-538X
---

```{r}
#| label: setup
#| include: false
#| echo: false
knitr::opts_chunk$set(
  comment = "#>"
)
# enable coloured cli text in output chunks
options(crayon.enabled = TRUE,
        cli.num_colors = 256)
fansi::set_knit_hooks(knitr::knit_hooks, which = c("output", "message", "error"))
```

```{=html}
<style>
/* 1) Collapse spacing on the <pre> blocks produced by fansi */
pre.fansi {
  margin: 0 !important;
}

/* Optional: if you still see gaps, also collapse adjacent fansi blocks */
pre.fansi + pre.fansi {
  margin-top: 0 !important;
}

/* 2) remove the default margin that Quarto adds around output code blocks */
.cell pre {
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}
</style>
```

Expert judgements and forecasts are playing an increasingly important role in a variety of fields such as ecological management and decision making [@Legge2022], biosecurity risk analyses [@Wittmann2015], replicability forecasts [@Mody2026], and horizon scanning exercises [@Sutherland2017]. Eliciting judgements from multiple experts is always preferable to relying on single estimates, but it comes with the challenge of aggregating different opinions. Achieving *behavioural consensus* amongst a group of experts, i.e. continued deliberation until everyone agrees on the same answer, is difficult, time consuming, and not always appropriate when multiple, valid viewpoints exist. The alternative is to make use of *mathematical consensus*, whereby data from a group of experts is mathematically aggregated into a single score.

Aggregated elicited judgements form a critical component of decision-making across multiple contexts, but there has been a dearth of accessible tools for implementing anything more complex than linear averages. Existing software options are either closed-source, locked behind paid software licenses, non-reproducible, or written in programming languages scantly used by ecologists or metascientists. Methods implemented in R, one of the most widely used programming languages in the life sciences [@Gao2025], are also limited, either being small in scope or having been archived from CRAN over the years.

# Elicitation at scale: The SCORE program and a need for easy and reliable aggregation methods

{aggreCAT} was initially developed as part of the [repliCATS](https://arts.unimelb.edu.au/school-of-historical-and-philosophical-studies/our-research/research-projects/replicats) (Collaborative Assessment of Trustworthy Science) project within the international [DARPA SCORE program](https://www.darpa.mil/research/programs/systematizing-confidence-in-open-research-and-evidence) (Systemizing Confidence in Open Research and Evidence). As part of SCORE, repliCATS used expert elicitation to evaluate the replicability of 4000 research claims published in the social and behavioural sciences (the results of which have just been released, see Mody et al. 2026 [-@Mody2026]). {aggreCAT} arose out of a need for a software solution to reliably conduct mathematical aggregation at scale. While originally a bespoke implementation for repliCATS, we have since developed {aggreCAT} into a more general R package to handle most types of elicitation data requiring mathematical aggregation allowing for wider applicability.

# What does {aggreCAT} offer?

{aggreCAT} is an open, reliable, and modular R package for the mathematical aggregation of expert elicitation data with a straightforward user interface [@Gould2021]. {aggreCAT} provides a suite of 29 aggregation methods that explore different approaches to mathematical aggregation from straightforward arithmetic calculations to Bayesian statistical models. In addition, we provide extra functionality such as plotting functions and performance evaluation against known outcomes. While the repliCATS project uses the IDEA protocol to structure their elicitation process, we have generalised {aggreCAT} to work with a variety of elicitation protocols. {aggreCAT} fills a large void in open software for aggregating elicited judgements, providing enormous benefit to researchers and decision makers across any field of research.

# Using {aggreCAT}

{aggreCAT} v1.0.0 is available on [CRAN](https://cran.r-project.org/package=aggreCAT) while a development version is maintained on [GitHub (metamelb-repliCATS/aggreCAT)](https://github.com/metamelb-repliCATS/aggreCAT). For the walkthrough below we make use of the {aggreCAT}’s inbuilt datasets, which were originally published in Wintle et al. (2023) [-@Wintle2023]. This study served as a pilot of the repliCATS approach to the SCORE program and consists of evaluations by 25 experts of 25 published papers that underwent attempted replication.

## Load Packages

```{r}
#| label: load-libs
#| message: false
#| warning: false
#| echo: true
library(aggreCAT)
library(tidyverse)
```

## Look at the data object

All aggregation methods require a data frame input of the elicited data, and we’ve included `data_ratings` in the package as an example dataset. Let's take a look:

```{r}
#| label: examine-data-ratings
data_ratings
```

The data frame is in long format with six required columns (`group` here is additional and ignored). The full details are explained in `?data_ratings`, but these columns represent the structure of an IDEA protocol elicitation. You have an elicitation round, IDs for paper and elicitation participant, names of questions and elements (we support three-point elicitation questions), and then the value of the participants’ responses. While set up this way to support the IDEA protocol, {aggreCAT} isn’t limited to that elicitation format. As we’ll show later, we have some function arguments to allow for other elicitation styles.

## Mathematical aggregation

Once you have a dataset in the correct format, conducting the mathematical aggregation is straightforward. All aggregation methods are grouped by type into a series of wrapper functions with a `*WAgg()` naming convention (where `WAgg` is short for 'weighted aggregation' and `*` indicating the group). To start with, let’s just do a simple arithmetic mean aggregation with the `AverageWAgg()` function:

```{r}
#| label: ArMean
AverageWAgg(expert_judgements = data_ratings)
```

The output of all `*WAgg()` functions is a four-column data frame. The name of the chosen aggregation method, the ID of the elicitation being aggregated, the aggregated elicitation value (known as a confidence score), and the number of experts whose elicitation was aggregated. `ArMean`, the arithmetic mean aggregation method, is the default method for `AverageWAgg()` but we can use the `type` argument to apply a different aggregation method. Let’s aggregate using the median value instead:

```{r}
#| label: ArMedian
AverageWAgg(expert_judgements = data_ratings,
            type = "Median")
```

You may have noticed the list of pre-processing methods that are output to the console marked with an <i class="bi bi-info"></i> icon. Every `*WAgg()` function has a different combination of options available, which let the user toggle certain behaviours on or off to suit their chosen elicitation style. IDEA is a two round elicitation where only the second round of data gets used in the aggregation, but the `round_2_filter` argument lets you turn this step off if you wanted to retain both rounds, or if you used a single round elicitation approach. The `percent_toggle` argument lets you convert percentages into probabilities because the original requirements for {aggreCAT} included the need to accept elicited probabilities as percentages. By keeping `round_2_filter` set to `FALSE` you can aggregate other elicitation data formats (e.g. counts) but note that some aggregation methods are only applicable to probabilistic data.

```{r}
#| label: alt-data-tpes
cs_ArMean <- AverageWAgg(expert_judgements = data_ratings,
                         type = "ArMean",
                         percent_toggle = TRUE,
                         round_2_filter = TRUE)
```

Let’s generate confidence scores for several aggregation methods. First, we’ll use `IntervalWAgg()` and the `IntWAgg` aggregation type to perform a weighted aggregation where participants are weighted based on the width of their supplied uncertainty bounds.

```{r}
#| label: IntWAgg
cs_IntWAgg <- IntervalWAgg(expert_judgements = data_ratings,
                           type = "IntWAgg",
                           percent_toggle = TRUE,
                           round_2_filter = TRUE)
```

Next, we’ll use `ReasoningWAgg()` and the `ReasonWAgg` method to conduct a weighted aggregation based on the number of unique reasons the participant used to justify their numeric estimates. This method requires a supplementary dataset of these coded reasons to calculate a weight, which exists as an example dataset called `data_supp_reasons`.

```{r}
#| label: ReasonWAgg
cs_ReasonWAgg <- ReasoningWAgg(expert_judgements = data_ratings,
                               reasons = data_supp_reasons,
                               type = "ReasonWAgg",
                               percent_toggle = TRUE,
                               round_2_filter = TRUE)
```

Finally, we will use `LinearWAgg()` and the `Participant` type to conduct a weighted aggregation using a user-supplied weight that applies at the participant level. Here we create our `QuizWAgg` method where we weight users based on how well they performed on a quiz they completed prior to conducting the full elicitation. The `data_supp_quiz` object supplies several different ways that participant quiz responses were scored as different columns and we just need to specify which scoring method is to be used as the weight.

```{r}
#| label: QuizWAgg
cs_QuizWAgg <- LinearWAgg(expert_judgements = data_ratings,
                          type = "Participant",
                          weights = data_supp_quiz |>
                            rename(weight = quiz_score),
                          name = "QuizWAgg",
                          percent_toggle = TRUE,
                          round_2_filter = TRUE)
```

## Performance evaluation

The `confidence_score_evaluation()` function evaluates the performance of the aggregation methods when there are known outcomes, or “truth” values to be evaluated against. In our case, `data_outcomes` provides a binary value indicating whether a replication attempt for a given paper was successful or not.

```{r}
#| label: evaluate-cs-performance
confidence_score_evaluation(confidence_scores = bind_rows(cs_ArMean,
                                                          cs_IntWAgg,
                                                          cs_QuizWAgg,
                                                          cs_ReasonWAgg),
                            outcomes = data_outcomes)
```

This function currently calculates performance using four different metrics: AUC, Brier Score, classification accuracy, and a point-biserial correlation. Performance by each metric is calculated separately for each aggregation method.

## Visualisation

Finally, we have several basic visualisation functions to let you explore your data. Here we use `confidence_score_heatmap()` to generate a heatmap of the aggregated scores across a number of aggregation methods, using the `data_confidence_scores` object, and split by the replication study result within `data_outcomes`.

```{r}
#| label: visualise-cs-heatmap
#| message: false
#| column: body
#| fig-align: center
#| fig-width: 8
#| fig-height: 8
confidence_score_heatmap(confidence_scores = data_confidence_scores,
                         data_outcomes = data_outcomes)
```

{aggreCAT}’s example dataset included some infamous replication studies, so many participants were likely familiar with the outcomes of some or all replication studies before they gave their own estimates. We can see this playing out in the heatmap visualisation, where studies that did replicate mostly had scores above 0.5, while studies that failed to replicate were mostly below 0.5. It also reveals that behaviour was consistent across participants because this pattern largely holds across the different aggregation methods weighting by different behavioural characteristics. The exceptions to this are the methods found in `IntervalWAgg()`, which weights based on the user’s self-reported uncertainty, and `BetaArMean()` which uses a beta distribution to extremise participant scores to account for participant tendencies to provide conservative estimates. Both aggregation methods demonstrate much stronger “will not replicate” signals in the scores.

# What is next for {aggreCAT}?

While {aggreCAT} is perfectly functional in its current form we are always looking for ways to improve it. We have generalised the package functionality to work beyond the IDEA protocol elicitation formats alone, but there are still some hold-over naming conventions for dataset columns or function arguments that could be more generic, which will eventually be changed. We’re also working on some new aggregation methods to be included in future package versions. As a teaser, I have an early version of an aggregation method that implements Cooke’s classical method [@Cooke1991], a method for combining elicited probability distributions. We also have plans to improve and expand visualisation functions. Some of these are new functions based on plots we’ve created for publications, some of them are feature updates to existing functions to allow for some more user customisation.

We’re always happy to hear from existing and would-be users about any additional features that would help them conduct their own research or practitioners conducting their own elicitations. If you have any requests, you can submit them as an [issue on our GitHub repository](https://github.com/metamelb-repliCATS/aggreCAT/issues) and we can add them to our to do lists, or you’re welcome to submit pull requests with additional features that you’ve coded up that we will review and evaluate for inclusion in the package down the line. Finally, we are [contactable by email](mailto:dwilkinson@unimelb.edu.au) should you have any queries about the package, or about our research at [MetaMelb](https://arts.unimelb.edu.au/school-of-historical-and-philosophical-studies/our-research/metamelb).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShiftingWAgg.R
\name{ShiftingWAgg}
\alias{ShiftingWAgg}
\title{Aggregation Method: ShiftingWAgg}
\usage{
ShiftingWAgg(
  expert_judgements,
  type = "ShiftWAgg",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"ShiftWAgg"}, \code{"BestShiftWAgg"}, \code{"IntShiftWAgg"}, \code{"DistShiftWAgg"}, or \code{"DistIntShiftWAgg"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Weighted by judgements that shift the most after discussion
}
\details{
When judgements are elicited using the IDEA protocol (or any other protocol that allows
experts to revisit their original estimates), the second round of estimates may differ
from the original first set of estimates an expert provides. Greater changes between rounds
will be given greater weight.

\code{type} may be one of the following:

\loadmathjax
\strong{ShiftWAgg}: Takes into account the shift in all three estimates

Considers shifts across lower, \mjeqn{L_{i,c}}{ascii}, and upper,
\mjeqn{U_{i,c}}{ascii}, confidence limits, and the best estimate, \mjeqn{B_{i,c}}{ascii}.
More emphasis is placed on changes in the best estimate such that:

\mjdeqn{w\_Shift_{i,c} = |B1_{i,c} - B_{i,c}| + \frac{|L1_{i,c} - L_{i,c}|+|U1_{i,c} - U_{i,c}|}{2},}{ascii}

where \mjeqn{L1_{i,c}, B1_{i,c},U1_{i,c}}{ascii} are the first round lower, best and upper estimates (prior
to discussion) and \mjeqn{L_{i,c}, B_{i,c},U1_{i,c}}{ascii} are the individual’s revised second round estimates
(after discussion).

\mjdeqn{\hat{p}_c(ShiftWAgg) = \sum_{i=1}^N \tilde{w}\_Shift_{i,c}B_{i,c}}{ascii}

\strong{BestShiftWAgg}: Weighted according to shifts in best estimates alone

Taking into account the fact that the scales best estimates are measured on are bounded,
we can calculate shifts relative to the largest possible shift.

\mjtdeqn{
w\_BestShift_{i,c}=
\begin{cases}
\frac{|B1_{i,c} - B_{i,c}|}{B1_{i,c}},
\begin{aligned}
\displaystyle &\ for\ (B1_{i,c} > 0.5\ and\ B_{i,c} \le 0.5) \cr
\displaystyle &\ or\ B_{i,c} < B1_{i,c} \le 0.5\ or\ B1_{i,c} > B_{i,c} > 0.5
\end{aligned} \cr
\frac{|B1_{i,c} - B_{i,c}|}{1- B1_{i,c}},
\begin{aligned}
\displaystyle &\ for\ (B1_{i,c} < 0.5\ and\ B_{i,c} \ge 0.5) \cr
\displaystyle &\ or\ B1_{i,c} < B_{i,c} < 0.5\ or\ B_{i,c} > B1_{i,c} > 0.5.
\end{aligned}
\end{cases}}{%
w\_BestShift_{i,c}=
\begin{cases}
\frac{|B1_{i,c} - B_{i,c}|}{B1_{i,c}},
\begin{aligned}
\displaystyle &\ for\ (B1_{i,c} \gt 0.5\ and\ B_{i,c} \le 0.5) \cr
\displaystyle &\ or\ B_{i,c} \lt B1_{i,c} \le 0.5\ or\ B1_{i,c} \gt B_{i,c} \gt 0.5
\end{aligned} \cr
\frac{|B1_{i,c} - B_{i,c}|}{1- B1_{i,c}},
\begin{aligned}
\displaystyle &\ for\ (B1_{i,c} \lt 0.5\ and\ B_{i,c} \ge 0.5) \cr
\displaystyle &\ or\ B1_{i,c} \lt B_{i,c} \lt 0.5\ or\ B_{i,c} \gt B1_{i,c} \gt 0.5.
\end{aligned}
\end{cases}}{ascii}

\mjdeqn{\hat{p}_c(BestShiftWAgg) = \sum_{i=1}^N \tilde{w}\_BestShift_{i,c}B_{i,c}}{ascii}

\strong{IntShiftWAgg}: Weighted by shifts in interval widths alone.

Individuals whose interval widths narrow between rounds are given more weight.

\mjdeqn{w\_IntShift_{i,c} = \frac{1}{(U_{i,c}-L_{i,c})-(U1_{i,c}-L1_{i,c})+1}}{ascii}

\mjdeqn{\hat{p}_c(IntShiftWAgg) = \sum_{i=1}^N \tilde{w}\_IntShift_{i,c}B_{i,c}}{ascii}

\strong{DistShiftWAgg}: Weighted by whether best estimates become more extreme (closer to 0 or 1) between rounds.

\mjdeqn{w\_DistShift_{i,c} = 1 - (\min (B_{i,c}, 1-B_{i,c}) - \min (B1_{i,c}, 1-B1_{i,c}))}{ascii}

\mjdeqn{\hat{p}_c(DistShiftWAgg) = \sum_{i=1}^N \tilde{w}\_DistShift_{i,c}B_{i,c}}{ascii}

\strong{DistIntShiftWAgg}: Rewards both narrowing of intervals and shifting towards the certainty limits between rounds.

We simply multiply the weights calculated in the "DistShiftWAgg" and "IntShiftWAgg" methods.

\mjdeqn{w\_DistIntShift_{i,c} = \tilde{w}\_IntShift_{i,c} \cdot \tilde{w}\_DistShift_{i,c}}{ascii}

\mjdeqn{\hat{p}_c(DistIntShiftWAgg) = \sum_{i=1}^N \tilde{w}\_DistIntShift_{i,c}B_{i,c}}{ascii}
}
\examples{
\donttest{ShiftingWAgg(data_ratings)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesianWAgg.R
\name{BayesianWAgg}
\alias{BayesianWAgg}
\title{Aggregation Method: BayesianWAgg}
\usage{
BayesianWAgg(
  expert_judgements,
  type = "BayTriVar",
  priors = NULL,
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"BayTriVar"}, or \code{"BayPRIORsAgg"}.}

\item{priors}{(Optional) A dataframe of priors in the format of \link{data_supp_priors}, required for \code{type} \code{BayPRIORsAgg}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Bayesian aggregation methods with either uninformative or informative prior distributions.

\strong{JAGS Install}

For instructions on installing JAGS onto your system visit \url{https://gist.github.com/dennisprangle/e26923fae7477566510757ab3341f54c}
}
\details{
\code{type} may be one of the following:

\strong{BayTriVar}: The Bayesian Triple-Variability Method, fit with JAGS.

\loadmathjax
Three kinds of variability around best estimates are considered:
\enumerate{
\item generic claim variability: variation across individuals within a claim
\item generic participant variability: variation within an individual across claims
\item claim - participant specific uncertainty (operationalised by bounds): informed
by interval widths given by individual \mjeqn{i}{ascii} for claim \mjeqn{c}{ascii}.
}

The model takes the log odds transformed individual best estimates as input (data),
uses a normal likelihood function and derives a posterior distribution for the
probability of replication.

\mjdeqn{log( \frac{B_{i,c}}{1-B_{i,c}}) \sim N(\mu_c, \sigma_{i,c}),}{ascii}

where \mjeqn{\mu_c}{ascii} denotes the mean estimated probability of replication for claim
\mjeqn{c}{ascii}, and  \mjeqn{\sigma_{i,c}}{ascii} denotes the standard deviation of the
estimated probability of replication for claim \mjeqn{c}{ascii} and individual
\mjeqn{i}{ascii} (on the logit scale).  Parameter \mjeqn{\sigma_{i,c}}{ascii} is calculated as:
\mjdeqn{\sigma_{i,c} = (U_{i,c} - L_{i,c} + 0.01) \times \sqrt{\sigma_i^2+\sigma_c^2}}{ascii}
with \mjeqn{\sigma_i}{ascii} denoting the standard deviation of estimated probabilities of
replication for individual \mjeqn{i}{ascii} and  \mjeqn{\sigma_c}{ascii} denoting the standard
deviation of the estimated probability of replication for claim \mjeqn{c}{ascii}.

The uninformative priors for specifying this Bayesian model are
\mjeqn{\mu_c \sim N(0,\ 3)}{ascii}, \mjeqn{\sigma_i \sim U(0,\ 10)}{ascii} and
\mjeqn{\sigma_c \sim U(0,\ 10)}{ascii}. After obtaining the median of the posterior
distribution of \mjeqn{\mu_c}{ascii}, we can back transform to obtain
\mjeqn{\hat{p}_c}{ascii}:

\mjdeqn{\hat{p}_c\left( BayTriVar \right) = \frac{e^{\mu_c}}{1+e^{\mu_c}}}{ascii}

\strong{BayPRIORsAgg}: Priors derived from predictive models, updated with best estimates.

This method uses Bayesian updating to update a prior probability of
replication estimated from a predictive model with an aggregate of the individuals’ best
estimates for any given claim. Methodology is the same as \code{type} \code{"BayTriVar"} except an
informative prior is used for \mjeqn{\mu_c}{ascii}. Conceptually the parameters of the prior
distribution of \mjeqn{\mu_c}{ascii} are informed by the PRIORS model (Gould et al. 2021)
which is a multilevel logistic regression model that predicts the probability of
replication using attributes of the original study. However, any model providing predictions of
the probability of replication can be used to generate the required priors.
}
\section{Warning}{
Both \code{BayTriVar} and \code{BayPRIORsAgg} methods require a minimum of two claims for which judgements are supplied to \code{expert_judgements}. This is due to the mathematical definition of these aggregators: \code{BayesianWAgg} calculates the variance in best estimates across multiple claims as well as the variance in best estimates across claims per individual. Thus when only one claim is provided in \code{expert_judgements}, the variance is 0, hence more than one claim is required for the successful execution of both Bayesian methods.
}

\examples{
\dontrun{BayesianWAgg(data_ratings, percent_toggle = TRUE)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postprocess_judgements.R
\name{postprocess_judgements}
\alias{postprocess_judgements}
\title{Post-processing.}
\usage{
postprocess_judgements(method_output)
}
\arguments{
\item{method_output}{tibble created from one of the aggregation methods after
pre-processing' with columns for the aggregation \code{method}, \code{paper_id}, \code{aggregated_judgement} and
\code{n_experts}}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}, corresponding
with an aggregation \code{method} (character).
}
\description{
Standardise the output from aggregation method's.
This function is called by every aggregation method as a final step.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_outlier.R
\name{weight_outlier}
\alias{weight_outlier}
\title{Weighting method: Down weighting outliers}
\usage{
weight_outlier(expert_judgements)
}
\arguments{
\item{expert_judgements}{A dataframe in the form of \link{data_ratings}}
}
\value{
A tibble in the form of the input \code{expert_judgements} argument with additional columns
supplying the calculated weight for each row's observation.
}
\description{
This method down-weights outliers.
}
\details{
This function is used by \link{LinearWAgg} to weighting functions for the aggregation type
\code{"OutWAgg"}. Outliers are given less weight by using the squared difference between the
median of an individual's best estimates across all claims and their best estimate
for the claim being assessed:
\loadmathjax
\mjdeqn{d_{i,c} = \left(median\{{B_{i,c}}_{_{i=1,...,N}}\} - B_{i,c}\right)^2}{ascii}

Weights are given by 1 minus the proportion of the individual's squared difference
relative to the maximum squared difference for the claim across all individuals:

\mjdeqn{w\_out_{i} = 1 - \frac{d_{i,c}}{\max({d_c})})}{ascii}
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_reason2]{weight_reason2()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_reason2.R
\name{weight_reason2}
\alias{weight_reason2}
\title{Weighting method: Total number and diversity of judgement reasons}
\usage{
weight_reason2(expert_reasons)
}
\arguments{
\item{expert_reasons}{A dataframe in the form of \link{data_supp_reasons}}
}
\value{
A tibble of three columns \code{paper_id}, \code{user_name}, and \code{reason_count}
}
\description{
This function is used by \link{ReasoningWAgg} to weighting functions for the aggregation
type \code{"ReasonWAgg2"}. Weights are based on the number and diversity of reasoning
methods used by the participant to support their judgement.
}
\details{
\loadmathjax
An individual's weight is a product of the number of reasons given in support of
their judgement and the diversity of these reasons.
\mjdeqn{w\_{varReason}_{i,c} =\sum_{r=1}^{R} \mathbf{CR_i}(c,r) \cdot (1 - \frac{\sum_{c=1}^C
\mathbf{CR_i}(c,r)}{C})}{ascii}
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IntervalWAgg.R
\name{IntervalWAgg}
\alias{IntervalWAgg}
\title{Aggregation Method: IntervalWAgg}
\usage{
IntervalWAgg(
  expert_judgements,
  type = "IntWAgg",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"IntWAgg"}, \code{"IndIntWAgg"}, \code{"AsymWAgg"}, \code{"IndIntAsymWAgg"}, \code{"VarIndIntWAgg"},
\code{"KitchSinkWAgg"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate one of several types of linear-weighted best estimates where the weights
are dependent on the lower and upper bounds of three-point elicitation (interval widths).
}
\details{
The width of the interval provided by individuals may be an indicator of certainty,
and arguably of accuracy of the best estimate contained between the bounds of the interval.

\code{type} may be one of the following:

\strong{IntWAgg}: Weighted according to the interval width across individuals for that claim, rewarding narrow interval widths.
\loadmathjax
\mjdeqn{w\_Interval_{i,c}= \frac{1}{U_{i,c} - L_{i,c}}}{ascii}
\mjdeqn{\hat{p}_c( IntWAgg) = \sum_{i=1}^N \tilde{w}\_Interval_{i,c}B_{i,c}}{ascii}

where \mjeqn{U_{i,d}  - L_{i,d}}{ascii} are individual \mjeqn{i}{ascii}'s judgements for claim \mjeqn{d}{ascii}. Then

\strong{IndIntWAgg}: Weighted by the rescaled interval width (interval width relative to largest
interval width provided by that individual)

Because of the variability in interval widths between individuals across claims,
it may be beneficial to account for this individual variability by rescaling interval
widths across all claims per individual. This results in a re-scaled interval width weight,
for individual \mjeqn{i}{ascii} for claim \mjeqn{c}{ascii}, relative to the widest interval provided by
that individual across all claims \mjeqn{C}{ascii}:

\mjdeqn{w\_nIndivInterval_{i,c}= \frac{1}{\frac{U_{i,c} - L_{i,c}}{max({ (U_{i,d}
- L_{i,d}): d = 1,\dots, C})}}}{ascii}

\mjdeqn{\hat{p}_c\left( IndIntWAgg \right) = \sum_{i=1}^N \tilde{w}\_nIndivInterval_{i,c}B_{i,c}}{ascii}

\strong{AsymWAgg}: Weighted by the asymmetry of individuals' intervals, rewarding increasing asymmetry.

We use the asymmetry of an interval relative to the corresponding best estimate
to define the following weights:

\mjdeqn{w\_asym_{i,c}= \begin{cases}
1 - 2 \cdot \frac{U_{i,c}-B_{i,c}}{U_{i,c}-L_{i,c}}, \text{for}\ B_{i,c} \ge
\frac{U_{i,c}-L_{i,c}}{2}+L_{i,c}\cr
1 - 2 \cdot \frac{B_{i,c}-L_{i,c}}{U_{i,c}-L_{i,c}}, \text{otherwise}
\end{cases}}{ascii}

then,

\mjdeqn{\hat{p}_c(AsymWAgg) = \sum_{i=1}^N \tilde{w}\_asym_{i,c}B_{i,c}.}{ascii}

\strong{IndIntAsymWAgg}: Weighted by individuals’ interval widths and asymmetry

This rewards both asymmetric and narrow intervals. We simply multiply the weights calculated
in the \code{"AsymWAgg"} and \code{"IndIntWAgg"} methods.

\mjdeqn{w\_nIndivInterval\_asym_{i,c} = \tilde{w}\_nIndivInterval_{i,c} \cdot \tilde{w}\_asym_{i,c}}{ascii}

\mjdeqn{\hat{p}_c( IndIntAsymWAgg) = \sum_{i=1}^N \tilde{w}\_nIndivInterval\_asym_{i,c}B_{i,c}}{ascii}

\strong{VarIndIntWAgg}: Weighted by the variation in individuals’ interval widths

A higher variance in individuals' interval width across claims may indicate a higher responsiveness
to the supporting evidence of different claims. Such responsiveness might be predictive of more
accurate assessors. We define:

\mjdeqn{w\_varIndivInterval_{i}= var{(U_{i,c} - L_{i,c}): c = 1,\dots, C},}{ascii}

where the variance (\mjeqn{var}{ascii}) is calculated across all claims for individual \mjeqn{i}{ascii}.
Then,

\mjdeqn{\hat{p}_c(VarIndIntWAgg) = \sum_{i=1}^N \tilde{w}\_varIndivInterval_{i}B_{i,c}}{ascii}

\strong{KitchSinkWAgg}: Weighted by everything but the kitchen sink

This method is informed by the intuition that we want to reward narrow and asymmetric intervals,
as well as the variability of individuals' interval widths (across their estimates). Again, we multiply
the weights calculated in the \code{"AsymWAgg"}, \code{"IndIntWAgg"} and \code{"VarIndIntWAgg"} methods above.

\mjdeqn{w\_kitchSink_{i,c} = \tilde{w}\_nIndivInterval_{i,c} \cdot \tilde{w}\_asym_{i,c} \cdot \tilde{w}\_varIndivInterval_{i}}{ascii}

\mjdeqn{\hat{p}_c(KitchSinkWAgg) = \sum_{i=1}^N \tilde{w}\_kitchSink_{i,c}B_{i,c}}{ascii}
}
\examples{
\donttest{IntervalWAgg(data_ratings)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggreCAT_package.R
\docType{package}
\name{aggreCAT}
\alias{aggreCAT-package}
\alias{aggreCAT}
\title{aggreCAT: mathematically aggregating expert judgements}
\description{
To learn more about \pkg{aggreCAT}, start with the vignettes: \code{vignette(package = "aggreCAT")}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://replicats.research.unimelb.edu.au/}
  \item \url{https://github.com/metamelb-repliCATS/aggreCAT}
  \item \url{https://metamelb-replicats.github.io/aggreCAT/}
}

}
\author{
\strong{Maintainer}: David Wilkinson \email{david.wilkinson.research@gmail.com} (\href{https://orcid.org/0000-0002-9560-6499}{ORCID})

Authors:
\itemize{
  \item David Wilkinson \email{david.wilkinson.research@gmail.com} (\href{https://orcid.org/0000-0002-9560-6499}{ORCID})
  \item Elliot Gould (\href{https://orcid.org/0000-0002-6585-538X}{ORCID})
  \item Aaron Willcox (\href{https://orcid.org/0000-0003-2536-2596}{ORCID})
  \item Charles T. Gray
  \item Rose E. O'Dea (\href{https://orcid.org/0000-0001-8177-5075}{ORCID})
  \item Rebecca Groenewegen (\href{https://orcid.org/0000-0001-9177-8536}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method_placeholder.R
\name{method_placeholder}
\alias{method_placeholder}
\title{Placeholder function with TA2 output}
\usage{
method_placeholder(expert_judgements, method_name)
}
\arguments{
\item{expert_judgements}{A data frame in the form of ratings}

\item{method_name}{Aggregation method to place into placeholder mode}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
This functions stands in for when we haven't completed coding the method.
}
\details{
This function expects input from \link{preprocess_judgements} and outputs
for \link{postprocess_judgements}.
}
\examples{
\dontrun{method_placeholder(data_ratings, method_name = "TestMethod")}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DistributionWAgg.R
\name{DistributionWAgg}
\alias{DistributionWAgg}
\title{Aggregation Method: DistributionWAgg}
\usage{
DistributionWAgg(
  expert_judgements,
  type = "DistribArMean",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"DistribArMean"} or \code{"TriDistribArMean"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate the arithmetic mean of distributions created with expert judgements.
The aggregate is the median of the average distribution fitted on the individual estimates.
}
\details{
\loadmathjax

This method assumes that the elicited probabilities and bounds can be considered
to represent participants' subjective distributions associated with relative
frequencies (rather than unique events). That is to say that we considered that
the lower bound of the individual per claim corresponds to the 5th percentile
of their subjective distribution on the probability of replication, denoted
\mjeqn{q_{5,i}}{ascii}, the best estimate corresponds to the median, \mjeqn{q_{50,i}}{ascii}, and the upper
bound corresponds to the 95th percentile, \mjeqn{q_{95,i}}{ascii}. With these three
percentiles, we can fit parametric or non-parametric distributions and aggregate
them rather than the (point) best estimates.

\code{type} may be one of the following:

\strong{DistribArMean}: Applies a non-parametric distribution evenly across upper, lower and best estimates.

Using the three percentiles we can build the minimally
informative non-parametric distribution that spreads the mass uniformly between
the three percentiles.

\mjtdeqn{
F_{i}(x) = \begin{cases}
\displaystyle 0, \text{ for } x < 0 \cr
\displaystyle \frac{0.05}{q_{5,i}}\cdot x, \text{ for } 0 < x < q_{5,i}\cr
\displaystyle \frac{0.45}{q_{50,i}-q_{5,i}}\cdot(x-q_{5,i})+0.05, \text{ for } q_{5,i}\le x < q_{50,i}\cr
\displaystyle \frac{0.45}{q_{95,i}-q_{50,i}}\cdot(x-q_{50,i})+0.5, \text{ for } q_{50,i}\le x < q_{95,i}\cr
\displaystyle \frac{0.05}{1 - q_{95,i}}\cdot(x-q_{95,i})+0.95, \text{ for } q_{95,i}\le x < 1\cr
\displaystyle 1,  \text{ for } x \ge 1.
\end{cases}}{%
F_{i}(x) = \begin{cases}
\displaystyle 0, \text{ for } x \lt 0 \cr
\displaystyle \frac{0.05}{q_{5,i}}\cdot x, \text{ for } 0 \le x \lt q_{5,i}\cr
\displaystyle \frac{0.45}{q_{50,i}-q_{5,i}}\cdot(x-q_{5,i})+0.05, \text{ for } q_{5,i}\le x \lt q_{50,i}\cr
\displaystyle \frac{0.45}{q_{95,i}-q_{50,i}}\cdot(x-q_{50,i})+0.5, \text{ for } q_{50,i}\le x \lt q_{95,i}\cr
\displaystyle \frac{0.05}{1 - q_{95,i}}\cdot(x-q_{95,i})+0.95, \text{ for } q_{95,i}\le x \lt 1\cr
\displaystyle 1,  \text{ for } x \ge 1.
\end{cases}}{ascii}

Then take the average of all constructed distributions of participants for each claim:

\mjdeqn{AvDistribution = \frac{1}{N}\sum_{i=1}^N F_i(x),}{ascii}

and the aggregation is the median of the average distribution:

\mjdeqn{\hat{p}_c\left( DistribArMean \right) = AvDistribution^{-1}(0.5).}{ascii}

\strong{TriDistribArMean}: Applies a triangular distribution to the upper, lower and best estimates.

A more restrictive fit with different assumptions about the
elicited best estimates, upper and lower bounds. We can assume that the lower and upper
bounds form the support of the distribution, and the best estimate corresponds to the mode.

\mjtdeqn{
F_{i}(x) = \begin{cases}
\displaystyle 0, \text{ for } x < L_{i} \cr
\displaystyle \frac{\left(x-L_{i}\right)^2}{\left( U_{i}-L_{i}\right)\left(B_{i}-L_{i}\right)}, \text{ for } L_{i} \le x < B_{i}\cr
\displaystyle 1 - \frac{\left( U_{i}-x\right)^2}{\left( U_{i}-L_{i}\right)\left( U_{i}-B_{i}\right)}, \text{ for } B_{i} \le x < U_{i}\cr
\displaystyle 1,  \text{ for } x \ge U_{i}.
\end{cases}}{%
F_{i}(x) = \begin{cases}
\displaystyle 0, \text{ for } x \lt L_{i} \cr
\displaystyle \frac{\left(x-L_{i}\right)^2}{\left( U_{i}-L_{i}\right)\left(B_{i}-L_{i}\right)}, \text{ for } L_{i} \le x \lt B_{i}\cr
\displaystyle 1 - \frac{\left( U_{i}-x\right)^2}{\left( U_{i}-L_{i}\right)\left( U_{i}-B_{i}\right)}, \text{ for } B_{i} \le x \lt U_{i}\cr
\displaystyle 1,  \text{ for } x \ge U_{i}.
\end{cases}}{ascii}

Then take the average of all constructed distributions of participants for each claim:

\mjdeqn{    AvDistribution = \frac{1}{N}\sum_{i=1}^N F_i(x),}{ascii}

and the aggregation is the median of the average distribution:

\mjdeqn{    \hat{p}_c\left(TriDistribArMean\right) = AvDistribution^{-1}(0.5).}{ascii}
}
\examples{
\donttest{DistributionWAgg(data_ratings, percent_toggle = TRUE)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_ratings}
\alias{data_ratings}
\title{P1_ratings}
\format{
A table with 6880 rows and 7 columns:
\describe{
\item{round}{character string identifying whether the round was 1 (pre-discussion) or 2 (post-discussion)}
\item{paper_id}{character string of the claim ids (25 unique claims total)}
\item{user_name}{character string of anonymized IDs for each participant (25 participants included in this dataset)}
\item{question}{character string for the question type, with four options: direct_replication, involved_binary, belief_binary, or comprehension}
\item{element}{character string for the type of response coded in the row, with five options: three_point_lower, three_point_best, three_point_upper, binary_question, or likert_binary}
\item{value}{numeric value for the participant's response}
\item{group}{character string of group IDs that contained the participants}
}
}
\usage{
data_ratings
}
\description{
Anonymized expert judgements of known-outcome
claims, assessed at the 2019 SIPS repliCATS workshop
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_varIndivInterval.R
\name{weight_varIndivInterval}
\alias{weight_varIndivInterval}
\title{Weighting method: Variation in individuals’ interval widths}
\usage{
weight_varIndivInterval(expert_judgements)
}
\arguments{
\item{expert_judgements}{A dataframe in the form of \link{data_ratings}}
}
\value{
A tibble in the form of the input \code{expert_judgements} argument with additional columns
supplying the calculated weight for each row's observation.
}
\description{
Calculates weights based on the variability of interval widths
within individuals.
}
\details{
This function is used inside \link{IntervalWAgg} for aggregation types \code{"VarIndIntWAgg"}
and \code{"KitchSinkWAgg"}. It calculates the difference between individual's upper and
lower estimates, then calculates the variance in this interval across each individual's
claim assessments.
\loadmathjax
\mjdeqn{w\_varIndivInterval_{i} = var\{(U_{i,d}-L_{i,d}):c=1,...,C\}}{ascii}
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_reason2]{weight_reason2()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_justifications}
\alias{data_justifications}
\title{Free-text justifications for expert judgements}
\format{
A table with 5630 rows and 9 columns:
\describe{
\item{round}{character string identifying whether the round was 1 (pre-discussion) or 2 (post-discussion)}
\item{paper_id}{character string of the paper ids (25 papers total)}
\item{user_name}{character string of anonymized IDs for each participant (25 participants included in this dataset)}
\item{question}{character string for the question type, with five options: flushing_freetext, involved_binary, belief_binary, direct_replication, and comprehension}
\item{justification}{character string with participant's free-text rationale for their responses}
\item{justification_id}{character string with a unique ID for each row}
\item{vote_count}{numeric of recorded votes (all 0 or 1)}
\item{vote_sum}{numeric of summed vote counts(all 0 or 1)}
\item{group}{character string of group IDs that contained the participants}
}
}
\usage{
data_justifications
}
\description{
Free-text justifications for expert judgements
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AverageWAgg.R
\name{AverageWAgg}
\alias{AverageWAgg}
\title{Aggregation Method: AverageWAgg}
\usage{
AverageWAgg(
  expert_judgements,
  type = "ArMean",
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"ArMean"}, \code{"Median"}, \code{"GeoMean"}, \code{"LOArMean"}, or \code{"ProbitArMean"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate one of several types of averaged best estimates.
}
\details{
This function returns the average, median and transformed averages of
best-estimate judgements for each claim.

\code{type} may be one of the following:
\loadmathjax

\strong{ArMean}: Arithmetic mean of the best estimates
\mjdeqn{\hat{p}_c\left(ArMean \right ) = \frac{1}{N}\sum_{i=1}^N B_{i,c}}{ascii}
\strong{Median}: Median of the best estimates
\mjdeqn{\hat{p}_c \left(\text{median} \right) = \text{median} \{ B^i_c\}_{i=1,...,N}}{ascii}
\strong{GeoMean}: Geometric mean of the best estimates
\mjdeqn{GeoMean_{c}=  \left(\prod_{i=1}^N  B_{i,c}\right)^{\frac{1}{N}}}{ascii}
\strong{LOArMean}: Arithmetic mean of the log odds transformed best estimates
\mjdeqn{LogOdds_{i,c}= \frac{1}{N} \sum_{i=1}^N log\left( \frac{B_{i,c}}{1-B_{i,c}}\right)}{ascii}
The average log odds estimate is then back transformed to give a final group estimate:
\mjdeqn{\hat{p}_c\left( LOArMean \right) = \frac{e^{LogOdds_{i,c}}}{1+e^{LogOdds_{i,c}}}}{ascii}
\strong{ProbitArMean}: Arithmetic mean of the probit transformed best estimates
\mjdeqn{Probit_{c}= \frac{1}{N} \sum_{i=1}^N \Phi^{-1}\left( B_{i,c}\right)}{ascii}
The average probit estimate is then back transformed to give a final group estimate:
\mjdeqn{\hat{p}_c\left(ProbitArMean \right) = \Phi\left({Probit_{c}}\right)}{ascii}
}
\examples{
\donttest{AverageWAgg(data_ratings)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_confidence_scores}
\alias{data_confidence_scores}
\title{Confidence Scores generated for 25 papers with 22 aggregation methods}
\format{
a tibble with 550 rows and 5 columns
\describe{
\item{method}{character string of method name}
\item{paper_id}{character string of paper IDs}
\item{cs}{numeric of generated confidence scores}
\item{n_experts}{numeric of the number of expert judgements aggregated in confidence score}
}
}
\usage{
data_confidence_scores
}
\description{
Confidence Scores generated for 25 papers with 22 aggregation methods
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_comments}
\alias{data_comments}
\title{data_comments}
\format{
A tibble with 2 rows and 10 columns
\describe{
\item{round}{character string, both 'round_1' (before discussion)}
\item{paper_id}{character string identifying 2 unique papers}
\item{user_name}{factor for anonymized IDs for two participants}
\item{question}{character string for the type of question, both 'comprehension'}
\item{justification_id}{character string identifying 2 unique justifications}
\item{comment_id}{character string identifying 2 unique comments}
\item{commenter}{redundant column, same as user_name}
\item{comment}{character string with free-text response for the user}
\item{vote_count}{numeric, both 0}
\item{vote_sum}{numeric, both 0}
\item{group}{character string of group IDs that contained the participants}
}
}
\usage{
data_comments
}
\description{
data_comments
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence_score_evaluation.R
\name{confidence_score_evaluation}
\alias{confidence_score_evaluation}
\title{Confidence Score Evaluation}
\usage{
confidence_score_evaluation(confidence_scores, outcomes)
}
\arguments{
\item{confidence_scores}{A dataframe in the format output by the \verb{aggreCAT::} aggregation methods}

\item{outcomes}{A dataframe with two columns: \code{paper_id} (corresponding to the id's from the
confidence_scores), and \code{outcome} containing the known outcome of replication studies}
}
\value{
Evaluated dataframe with five columns: \code{method} (character variable describing the aggregation method),
\code{AUC} (Area Under the Curve (AUC) scores of ROC curves - see \code{?precrec::auc}), \code{Brier_Score} (see
\code{?DescTools::BrierScore}), \code{Classification_Accuracy}(classification accuracy measured for pcc =
percent correctly classified; see \code{?MLmetrics::Accuracy}), and \code{Correlation} (point-biserial correlation;
see \code{?stats::cor}).
}
\description{
Evaluate the performance of the confidence scores generated by one or more aggregation
methods. Assumes probabilistic confidence scores for the metrics selected.
}
\examples{
\donttest{
confidence_score_evaluation(data_confidence_scores,
                            data_outcomes)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence_score_ridgeplot.R
\name{confidence_score_ridgeplot}
\alias{confidence_score_ridgeplot}
\title{Confidence Score Ridge Plot}
\usage{
confidence_score_ridgeplot(confidence_scores = NULL)
}
\arguments{
\item{confidence_scores}{A data frame of confidence scores in long format in the form of \link{data_confidence_scores}}
}
\value{
A density ridge plot of aggregation methods
}
\description{
Display a ridge plot of confidence scores for each aggregation method
}
\examples{
\donttest{confidence_score_ridgeplot(data_confidence_scores)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_reason.R
\name{weight_reason}
\alias{weight_reason}
\title{Weighting method: Total number of judgement reasons}
\usage{
weight_reason(expert_reasons)
}
\arguments{
\item{expert_reasons}{A dataframe in the form of \link{data_supp_reasons}}
}
\value{
A tibble of three columns \code{paper_id}, \code{user_name}, and \code{reason_count}
}
\description{
This function is used by \link{ReasoningWAgg} to weighting functions for the aggregation
type \code{"ReasonWAgg"}. Calculates weights based on the number of judgement reasoning
methods used by an individual
}
\details{
Individuals' weight is equal to the maximum number of judgement reasons given
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason2]{weight_reason2()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_supp_priors}
\alias{data_supp_priors}
\title{A table of prior means, to be fed into the BayPRIORsAgg aggregation method}
\format{
A tibble of 25 rows and 2 columns
\describe{
\item{paper_id}{character string with a unique id for each row corresponding to the assessed claim (from 125 papers total)}
\item{prior_means}{numeric with the average prior probability for the claim corresponding to the paper_id}
}
}
\usage{
data_supp_priors
}
\description{
A table of prior means, to be fed into the BayPRIORsAgg aggregation method
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LinearWAgg.R
\name{LinearWAgg}
\alias{LinearWAgg}
\title{Aggregation Method: LinearWAgg}
\usage{
LinearWAgg(
  expert_judgements,
  type = "DistLimitWAgg",
  weights = NULL,
  name = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  flag_loarmean = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"Judgement"}, \code{"Participant"}, \code{"DistLimitWAgg"}, \code{"GranWAgg"}, or \code{"OutWAgg"}.}

\item{weights}{(Optional) A two column dataframe (\code{user_name} and \code{weight}) for \code{type = "Participant"}
or a three two column dataframe (\verb{paper_id', 'user_name} and \code{weight}) for \code{type = "Judgement"}}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{flag_loarmean}{A toggle to impute log mean (defaults \code{FALSE}).}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate one of several types of linear-weighted best estimates.
}
\details{
This function returns weighted linear combinations of the best-estimate judgements for each claim.

\code{type} may be one of the following:
\loadmathjax

\strong{Judgement}: Weighted by user-supplied weights at the judgement level
\mjdeqn{\hat{p}_c\left( JudgementWeights \right) = \sum_{i=1}^N
\tilde{w}\_judgement_{i,c}B_{i,c}}{ascii}

\strong{Participant}: Weighted by user-supplied weights at the participant level
\mjdeqn{\hat{p}_c\left( ParticipantWeights \right) = \sum_{i=1}^N
\tilde{w}\_participant_{i}B_{i,c}}{ascii}

\strong{DistLimitWAgg}: Weighted by the distance of the best estimate from the
closest certainty limit. Giving greater weight to best estimates that are closer to certainty
limits may be beneficial.
\mjdeqn{w\_distLimit_{i,c} = \max \left(B_{i,c}, 1-B_{i,c}\right)}{ascii}
\mjdeqn{\hat{p}_c\left( DistLimitWAgg \right) = \sum_{i=1}^N
\tilde{w}\_distLimit_{i,c}B_{i,c}}{ascii}

\strong{GranWAgg}: Weighted by the granularity of best estimates

Individuals are weighted by whether or not their best estimates are more
granular than a level of 0.05 (i.e., not a multiple of 0.05).
\mjdeqn{w\_gran_{i} = \frac{1}{C} \sum_{d=1}^C \left\lceil{\frac{B_{i,d}}
{0.05}-\left\lfloor{\frac{B_{i,d}}{0.05}}\right\rfloor}\right\rceil,}{ascii}

where \mjeqn{\lfloor{\ }\rfloor}{ascii} and \mjeqn{\lceil{\ }\rceil}{ascii}
are the mathematical floor and ceiling functions respectively.
\mjdeqn{\hat{p}_c\left( GranWAgg \right) = \sum_{i=1}^N \tilde{w}\_gran_{i}
B_{i,c}}{ascii}

\strong{OutWAgg}: Down weighting outliers

This method down-weights outliers by using the differences from the central
tendency (median) of an individual's best estimates.
\mjdeqn{d_{i,c} = \left(median\{{B_{i,c}}_{_{i=1,...,N}}\} - B_{i,c}\right)^2}{ascii}
\mjdeqn{w\_out_{i} = 1 - \frac{d_{i,c}}{\max({d_c})})}{ascii}
\mjdeqn{\hat{p}_c\left( OutWAgg \right) = \sum_{i=1}^N \tilde{w}\_out_{i}B_{i,c}}{ascii}
}
\examples{
\donttest{LinearWAgg(data_ratings, percent_toggle = TRUE)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess_judgements.R
\name{preprocess_judgements}
\alias{preprocess_judgements}
\title{Pre-process the data}
\usage{
preprocess_judgements(
  expert_judgements,
  round_2_filter = TRUE,
  three_point_filter = TRUE,
  percent_toggle = FALSE
)
}
\arguments{
\item{expert_judgements}{A dataframe with the same variables (columns) as
\link{data_ratings}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}

\item{three_point_filter}{Defaults \code{TRUE} to filter three point estimates. \code{FALSE} will
filter the involved_binary question.}

\item{percent_toggle}{Change the values to probabilities from percentages. Default is \code{FALSE}}
}
\value{
a long tibble of expert judgements, with six columns:
\code{round}, \code{paper_id}, \code{user_name}, \code{element} (i.e. question type),
and \code{value} (i.e. participant response).
}
\description{
Process input data with filters and meaningful variable names.

This function is called at the head of every aggregation method function.
}
\details{
This pre-processing function takes input data in the format of
\link{data_ratings} and outputs a dataframe that:
\enumerate{
\item Applies any filters or manipulations required by the aggregation method.
\item Converts the input data into variables with more
meaningful names for coding, to avoid errors in the wrangling process.
}
}
\examples{
\donttest{preprocess_judgements(data_ratings)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_supp_reasons}
\alias{data_supp_reasons}
\title{Categories of reasons provided by participants for their expert judgements}
\format{
a tibble with 625 rows and 15 columns
\describe{
\item{paper_id}{character string for the paper ID}
\item{user_name}{character string for participant ID}
\item{RW04 Date of publication}{numeric; references to the date of publication, for example in relation to something being published prior to the 'replication crisis' within the relevant discipline, or a study being difficult to re-run now because of changes in social expectations. }
\item{RW15 Effect size}{numeric; any references to the effect size that indicate that the participant considered the size of the effect when assessing the claim. Don’t use if the term "effect size" is used in unrelated ways, but err on the side of considering statements as relevant to the participant’s assessment.}
\item{RW16 Interaction effect}{numeric; references to when the effect was an interaction effect (rather than a direct effect).}
\item{RW17 Interval or range measure for statistical uncertainty (CI, SD, etc )}{numeric; references to the inclusion, absence, or size of the uncertainty measure for a given effect.}
\item{RW18 Outside participants areas of expertise}{numeric; references to the claim under assessment being outside the participant's areas of expertise.}
\item{RW20 Plausibility}{numeric; references to the plausibility of the claim.}
\item{RW21 Population or subject characteristics (sampling practices)}{numeric; references to the characteristics of the sample population or subjects used in a study that affect the participant’s assessment of the claim, including references to low response rate and any other questions or appreciation of the sampling practices.}
\item{RW22 Power adequacy and or sample size}{numeric; combines 2 nodes for references to the adequacy (or not) of the statistical power of the study &/or sample size.}
\item{RW32 Reputation}{numeric; references to the reputation of the journal/institute/author.}
\item{RW37 Revision statements}{numeric; .}
\item{RW42 Significance, statistical (p-value etc )}{numeric; references to a test of statistical significance for the claim as it impacts on the participant’s assessment. This explicitly includes p-values, t-values, critical alpha and p-rep.}
}
}
\usage{
data_supp_reasons
}
\description{
Categories of reasons provided by participants for their expert judgements
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_outcomes}
\alias{data_outcomes}
\title{Replication outcomes for the papers}
\format{
a tibble with 25 rows and 2 columns
\describe{
\item{paper_id}{character string for the paper ID}
\item{outcome}{numeric value of replication outcome. 1 = replication success, 0 = replication failure}
}
}
\usage{
data_outcomes
}
\description{
Replication outcomes for the papers
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_supp_quiz}
\alias{data_supp_quiz}
\title{A table of scores on the quiz to assess prior knowledge, to be fed into the QuizWAgg aggregation method}
\format{
A tibble 19 rows and 2 columns
\describe{
\item{user_name}{factor for anonymized IDs for each participant}
\item{quiz_score}{numeric for the participant's score on the quiz (min of 0, max of 16, NA if no questions answered)}
\item{quiz_score_even}{numeric quiz scores where all questions are weighted equally (the original quiz score assigned only half marks for true/false questions)}
\item{quiz_score_stats}{numeric participant score based only on questions pertaining to statistics}
}
}
\usage{
data_supp_quiz
}
\description{
A table of scores on the quiz to assess prior knowledge, to be fed into the QuizWAgg aggregation method
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReasoningWAgg.R
\name{ReasoningWAgg}
\alias{ReasoningWAgg}
\title{Aggregation Method: ReasoningWAgg}
\usage{
ReasoningWAgg(
  expert_judgements,
  reasons = NULL,
  type = "ReasonWAgg",
  name = NULL,
  beta_transform = FALSE,
  beta_param = c(6, 6),
  placeholder = FALSE,
  percent_toggle = FALSE,
  flag_loarmean = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{reasons}{A dataframe in the form of \link{data_supp_reasons}}

\item{type}{One of \code{"ReasonWAgg"}, \code{"ReasonWAgg2"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{beta_transform}{Toggle switch to extremise confidence scores with the beta distribution. Defaults to \code{FALSE}.}

\item{beta_param}{Length two vector of alpha and beta parameters of the beta distribution. Defaults to \code{c(6,6)}.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{flag_loarmean}{A toggle to impute LOArMean instead of ArMean when no participants have a reasoning weight for a specific claim (defaults \code{FALSE}).}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate one of several types of linear-weighted best estimates using supplementary
participant reasoning data to create weights.
}
\details{
Weighted by the breadth of reasoning provided to support the individuals’ estimate.

\code{type} may be one of the following:

\loadmathjax
\strong{ReasonWAgg}: Weighted by the number of supporting reasons

Giving greater weight to best estimates that are accompanied by a greater number of
supporting reasons may be beneficial. We will consider \mjeqn{w\_{reason}_{i,c}}{ascii}
to be the number of unique reasons provided by that individual \mjeqn{i}{ascii} in
support of their estimate for claim \mjeqn{c}{ascii}.

\mjdeqn{\hat{p}_c(ReasonWAgg) = \sum_{i=1}^N \tilde{w}\_reason_{i,c}B_{i,c}}{ascii}

See Hanea et al. (2021) for an example of reason coding.

\strong{ReasonWAgg2}: Incorporates both the number of reasons and their diversity across claims.

The claim diversity component of this score is calculated per individual from all claims
they assessed. We assume each individual answers at least two claims. If an individual has
assessed only one claim, there weighting for that claim is equivalent to "ReasonWAgg".

We will consider \mjeqn{w\_{varReason}_{i,c}}{ascii} to be the weighted "number of unique
reasons" provided by participant \mjeqn{i}{ascii} in support of their estimate for claim
\mjeqn{c}{ascii}. Assume there are \mjeqn{R}{ascii} total unique reasons any participant
can use to justify their numerical answers. Then, for each participant \mjeqn{i}{ascii} we
can construct a matrix \mjeqn{\mathbf{CR_i}}{ascii} with \mjeqn{R}{ascii} columns, each
corresponding to a unique reason, \mjeqn{r}{ascii}, and \mjeqn{C}{ascii} rows, where
\mjeqn{C}{ascii} is the number of claims assessed by that participant. Each element of
this matrix \mjeqn{\mathbf{CR_i}(r,c)}{ascii} can be either 1 or 0.
\mjeqn{\mathbf{CR_i}(r,c) = 1}{ascii} if reason \mjeqn{R_r}{ascii} was used to justify
the estimates assessed for \mjeqn{c}{ascii}, and \mjeqn{\mathbf{CR_i}(r,c) = 0}{ascii} if
reason \mjeqn{R_r}{ascii} was not mentioned when assessing claim \mjeqn{c}{ascii}. The
more frequently that a participant uses a given reason reduces the amount it contributes
to the weight assigned to that participant.

\mjdeqn{w\_{varReason}_{i,c} =\sum_{r=1}^{R} \mathbf{CR_i}(c,r) \cdot (1 - \frac{\sum_{c=1}^C
\mathbf{CR_i}(c,r)}{C})}{ascii}

\mjdeqn{\hat{p}_c(ReasonWAgg2) = \sum_{i=1}^N \tilde{w}\_varReason_{i,c}B_{i,c}}{ascii}
}
\note{
When \code{flag_loarmean} is set to \code{TRUE}, two additional columns will be returned; \code{method_applied} (a character variable describing the method actually applied with values of either \code{LoArMean} or \code{ReasonWAgg}) and \code{no_reason_score} (a logical variable describing whether no reasoning scores were supplied for any user for the given claim, where \code{TRUE} indicates no reasoning scores supplied and \code{FALSE} indicates that at least one participant for that claim had a reasoning score greater than 0).

named method_applied (with values LoArMean or ReasonWAgg), and no_reason_score, a logical variable describing whether or not there were no reasoning scores for that claim.
}
\examples{
\donttest{ReasoningWAgg(data_ratings, data_supp_reasons)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_interval.R
\name{weight_interval}
\alias{weight_interval}
\title{Weighting method: Width of intervals}
\usage{
weight_interval(expert_judgements)
}
\arguments{
\item{expert_judgements}{A dataframe in the form of \link{data_ratings}}
}
\value{
A tibble in the form of the input \code{expert_judgements} argument with additional columns
supplying the calculated weight for each row's observation.
}
\description{
Calculates weights by interval width
}
\details{
This function is used inside \link{IntervalWAgg} for aggregation type \code{"IntWAgg"}. It
calculates the width of each three-point judgement (upper - lower), then returns
the weight as the inverse of this interval.
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_reason2]{weight_reason2()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence_score_heatmap.R
\name{confidence_score_heatmap}
\alias{confidence_score_heatmap}
\title{Confidence Score Heat Map}
\usage{
confidence_score_heatmap(confidence_scores = NULL, data_outcomes = NULL)
}
\arguments{
\item{confidence_scores}{A data frame of confidence scores generated from the aggregation methods in the form of \link{data_confidence_scores}.
Defaults to \link{data_confidence_scores} if no argument supplied.}

\item{data_outcomes}{A data frame of unique claims and the associated binary outcome in the form of \link{data_outcomes}. If no argument supplied
then defaults to \link{data_outcomes} supplied within package.}
}
\value{
Plot in viewer
}
\description{
Confidence scores displayed on a colour spectrum across generated methods and
assessed claims, split into predicted replication outcomes (TRUE/FALSE).
}
\examples{
\donttest{confidence_score_heatmap(data_confidence_scores, data_outcomes)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_nIndivInterval.R
\name{weight_nIndivInterval}
\alias{weight_nIndivInterval}
\title{Weighting method: Individually scaled interval widths}
\usage{
weight_nIndivInterval(expert_judgements)
}
\arguments{
\item{expert_judgements}{A dataframe in the form of \link{data_ratings}}
}
\value{
A tibble in the form of the input \code{expert_judgements} argument with additional columns
supplying the calculated weight for each row's observation.
}
\description{
Weighted by the rescaled interval width within individuals across claims.
}
\details{
\loadmathjax
This function is used inside \link{IntervalWAgg} for aggregation types \code{"IndIntWAgg"},
\code{"IndIntAsymWAgg"} and \code{"KitchSinkWAgg"}. Interval width weights are rescaled
relative to an individuals interval widths across all claims.

\mjdeqn{w\_nIndivInterval_{i,c} = \frac{1}{\frac{U_{i,c}-L_{i,c}}{\max\left(\{(U_{i,d}-L_{i,d}):d=1,...,C\}\right)}}}{ascii}
}
\seealso{
Other weighting functions:
\code{\link[=weight_asym]{weight_asym()}},
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_reason2]{weight_reason2()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ExtremisationWAgg.R
\name{ExtremisationWAgg}
\alias{ExtremisationWAgg}
\title{Aggregation Method: ExtremisationWAgg}
\usage{
ExtremisationWAgg(
  expert_judgements,
  type = "BetaArMean",
  name = NULL,
  alpha = 6,
  beta = 6,
  cutoff_lower = NULL,
  cutoff_upper = NULL,
  placeholder = FALSE,
  percent_toggle = FALSE,
  round_2_filter = TRUE
)
}
\arguments{
\item{expert_judgements}{A dataframe in the format of \link{data_ratings}.}

\item{type}{One of \code{"BetaArMean"} or \code{"BetaArMean2"}.}

\item{name}{Name for aggregation method. Defaults to \code{type} unless specified.}

\item{alpha}{parameter for the 'shape1' argument in the \code{stats::pbeta} function (defaults to 6)}

\item{beta}{parameter for the 'shape2' argument in the \code{stats::pbeta} function (defaults to 6)}

\item{cutoff_lower}{Lower bound of middle region without extremisation in \code{"BetaArMean2"} aggregation \code{type}s.}

\item{cutoff_upper}{Upper bound of middle region without extremisation in \code{"BetaArMean2"} aggregation \code{type}s.}

\item{placeholder}{Toggle the output of the aggregation method to impute placeholder data.}

\item{percent_toggle}{Change the values to probabilities. Default is \code{FALSE}.}

\item{round_2_filter}{Note that the IDEA protocol results in both a Round 1
and Round 2 set of probabilities for each claim. Unless otherwise specified,
we will assume that the final Round 2 responses (after discussion) are being
referred to.}
}
\value{
A tibble of confidence scores \code{cs} for each \code{paper_id}.
}
\description{
Calculate beta-transformed arithmetic means of best estimates.
}
\details{
This method takes the average of best estimates and transforms it using the cumulative
distribution function of a beta distribution.

\code{type} may be one of the following:

\strong{BetaArMean}: Beta transformation applied across the entire range of calculated confidence scores.
\loadmathjax

\mjdeqn{\hat{p}_c\left( \text{BetaArMean} \right) = H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right),}{ascii}

where \mjeqn{H_{\alpha \beta}}{ascii} is the cumulative distribution function of the beta distribution
with parameters \mjeqn{\alpha}{ascii} and \mjeqn{\beta}{ascii}, which default to 6 in the function.

The justification for equal parameters (the 'shape1' and 'shape2' arguments in the \code{stats::pbeta} function)
are outlined in Satopää et al (2014) and the references therein (note that the method outlined in that paper
is called a beta-transformed linear opinion pool).
To decide on the default shape value of \code{6}, we explored the \code{data_ratings} dataset with random subsets of 5 assessments per claim,
which we expect to have for most of the claims assessed by repliCATS.

\strong{BetaArMean2}: Beta transformation applied only to calculated confidence scores that are outside a specified middle range. The premise being that we don't extremise "fence-sitter" confidence scores.

\mjtdeqn{
\hat{p}_c\left( \text{BetaArMean2} \right) = \begin{cases}
\displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} < \textit{cutoff\_lower} \cr
\displaystyle \frac{1}{N} \sum_{i=1}^N B_{i,c}, \text{ for } \textit{cutoff\_lower} \leq \frac{1}{N} \sum_{i=1}^N B_{i,c} \le \textit{cutoff\_upper} \cr
\displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} > \textit{cutoff\_upper} \cr
\end{cases}}{%
\hat{p}_c\left( \text{BetaArMean2} \right) = \begin{cases}
\displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} \lt \textit{cutoff\_lower} \cr
\displaystyle \frac{1}{N} \sum_{i=1}^N B_{i,c}, \text{ for } \textit{cutoff\_lower} \leq \frac{1}{N} \sum_{i=1}^N B_{i,c} \le \textit{cutoff\_upper} \cr
\displaystyle H_{\alpha \beta}\left(\frac{1}{N} \sum_{i=1}^N B_{i,c} \right), \text{ for } \frac{1}{N} \sum_{i=1}^N B_{i,c} \gt \textit{cutoff\_upper} \cr
\end{cases}}{ascii}
}
\examples{
\donttest{ExtremisationWAgg(data_ratings, percent_toggle = TRUE)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weight_asym.R
\name{weight_asym}
\alias{weight_asym}
\title{Weighting method: Asymmetry of intervals}
\usage{
weight_asym(expert_judgements)
}
\arguments{
\item{expert_judgements}{the long tibble exported from the \code{preprocess_judgements} function.}
}
\value{
A tibble in the form of the input \code{expert_judgements} argument with additional columns
supplying the calculated weight for each row's observation.
}
\description{
Calculates weights by asymmetry of intervals
}
\details{
This function is used inside \link{IntervalWAgg} to calculate the weights for the
aggregation type \code{"AsymWAgg"}, \code{"IndIntAsymWAgg"} and \code{"KitchSinkWAgg"}. Pre-processed
expert judgements (long format) are first converted to wide format then weighted by:
\loadmathjax
\mjdeqn{w\_asym_{i,c}= \begin{cases}
1 - 2 \cdot \frac{U_{i,c}-B_{i,c}}{U_{i,c}-L_{i,c}}, \text{for}\ B_{i,c} \ge
\frac{U_{i,c}-L_{i,c}}{2}+L_{i,c}\cr
1 - 2 \cdot \frac{B_{i,c}-L_{i,c}}{U_{i,c}-L_{i,c}}, \text{otherwise}
\end{cases}}{ascii}

Data is converted back to long format, with only the weighted best estimates
retained.
}
\examples{
weight_asym(preprocess_judgements(data_ratings))
}
\seealso{
Other weighting functions:
\code{\link[=weight_interval]{weight_interval()}},
\code{\link[=weight_nIndivInterval]{weight_nIndivInterval()}},
\code{\link[=weight_outlier]{weight_outlier()}},
\code{\link[=weight_reason]{weight_reason()}},
\code{\link[=weight_reason2]{weight_reason2()}},
\code{\link[=weight_varIndivInterval]{weight_varIndivInterval()}}
}
\concept{weighting functions}
